﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Zcu.CurvatureBenchmark.Framework;
using Zcu.CurvatureBenchmark.Framework.Estimators;

namespace CurvatureVisualizer
{
    class Map
    {
		private static void FindExtreme(double[][] values, out double minValue, out double maxValue) 
		{
			double min = double.MaxValue;
            double max = double.MinValue;
            for (int i = 0; i < values.Length; i++)
            {
                for (int j = 0; j < values[i].Length; j++)
                {
                    if (values[i][j] < min) min = values[i][j];
                    if (values[i][j] > max) max = values[i][j];
                }
            }

			minValue = min;
			maxValue = max;
		}

        public static void Constant(TriangleMesh[] meshes, double[] values, ColorRGBA[] colors = null)
        {
            for (int j = 0; j < meshes.Length; j++)
            {
                TriangleMesh mesh = meshes[j];
                ColorRGBA color = new ColorRGBA(0.5f, 0.5f, 0.5f);
                if (colors != null && colors.Length > 0) color = colors[0];
                mesh.Colors = new ColorRGBA[mesh.Points.Length];
                for (int i = 0; i < mesh.Colors.Length; i++)
                {
                    mesh.Colors[i] = color;
                }
            }
        }

		public static void Gradient(TriangleMesh[] meshes, double[][] values) 		
		{
			//http://www.shaderfx.com/2015/04/playing-with-mesh-curvature-in-real-time.html

			double min, max;
			FindExtreme(values, out min, out max);

            for (int j = 0; j < meshes.Length; j++)
            {
				Gradient(meshes[j], values[j], min, max);
                              
            }
		}


		public static void GradientFromExact(TriangleMesh[] meshes, ICurvatureEstimator[] ests, double[][] values)
		{

			double min = double.MaxValue;
			double max = double.MinValue;
			
			for (int j = 0; j < meshes.Length; j++)
            {
				if (ests[j] is ExactEstimator)
				{
					min = double.MaxValue;
					max = double.MinValue;
					for (int i = 0; i < values[j].Length; i++)
					{
						if (values[j][i] < min) min = values[j][i];
						if (values[j][i] > max) max = values[j][i];
					}                  
				}

				Gradient(meshes[j], values[j], min, max);
                              
            }

		}

		private static void Gradient(TriangleMesh mesh, double[] values, double min, double max)
		{


            //symetric interval
            max = Math.Max(Math.Abs(min), Math.Abs(max));
            min = -max;

               mesh.Colors = new ColorRGBA[mesh.Points.Length];


                for (int i = 0; i < mesh.Colors.Length; i++)
                {
				   double value = values[i];

                    ColorRGBA c = new ColorRGBA(1.0f, 1.0f, 1.0f);
                    double dv;
                    // Unknown values in our kernel
                    if (value == 0.0f)
                    {
                        mesh.Colors[i] = c;
                        continue;
                    }
                    // Threshold
                    if (value < min)
                    {
                        value = min;
					   c = new ColorRGBA(0, 1.0f, 0);
					   mesh.Colors[i] = c;
					   continue;
                    }
                    if (value > max)
                    {
                        value = max;

					   c = new ColorRGBA(0, 1.0f, 0);
					   mesh.Colors[i] = c;
					   continue;
                    }



                    // Interval
                    dv = max - min;
                    // Seismic color map like
                    /*if (value < (min + 0.5 * dv))
                    {
                        c.RedF = (float)(2.0 * (value - min) / dv);
                        c.GreenF = (float)(2.0 * (value - min) / dv);
                        c.BlueF = 1;
                    }
                    else
                    {
                        c.BlueF = (float)(2 - 2 * (value - min) / dv);
                        c.GreenF = (float)(2 - 2 * (value - min) / dv);
                        c.RedF = 1;
                    }*/
                    dv = Math.Max(Math.Abs(max),Math.Abs(min));

                    if (value < 0)
                    {
                        c.RedF = 0;
                        c.GreenF = 1-(float)((-value) / dv);
                        c.BlueF = 1;
                    }
                    else
                    {
                        c.BlueF = 0;
                        c.GreenF = 1-(float)((value) / dv);
                        c.RedF = 1;
                    }

                    mesh.Colors[i] = c;
                }
            }


        public static void PositiveNegative(TriangleMesh[] meshes, double[][] values, ColorRGBA[] colors = null)
        {
            ColorRGBA negative = new ColorRGBA(1.0f, 0, 0);
            ColorRGBA positive = new ColorRGBA(0, 1.0f, 0);
            ColorRGBA zero = new ColorRGBA(0, 0, 1.0f);
            if (colors != null && colors.Length > 2)
            {
                negative = colors[0];
                positive = colors[1];
                zero = colors[2];
            }
            double min, max;

			FindExtreme(values, out min, out max);

            double span = max;
			if (max < -min)
			{
				span = -min;
			}

            for (int j = 0; j < meshes.Length; j++)
            {
                TriangleMesh mesh = meshes[j];
                mesh.Colors = new ColorRGBA[mesh.Points.Length];

                for (int i = 0; i < mesh.Colors.Length; i++)
                {
                    if (Utils.IsZero(values[j][i]))
                    {
                        mesh.Colors[i] = zero;
                    }
                    else if (values[j][i] < 0)
                    {
                        var coef = (float)(-values[j][i] / span);
                        mesh.Colors[i] = new ColorRGBA(negative.RedF * coef, negative.GreenF * coef, negative.BlueF * coef);
                    }
                    else
                    {
                        var coef = (float)(values[j][i] / span);
                        mesh.Colors[i] = new ColorRGBA(positive.RedF * coef, positive.GreenF * coef, positive.BlueF * coef);
                    }
                }
            }
        }
    }
}
