﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Zcu.CurvatureBenchmark.Framework;
using Zcu.CurvatureBenchmark.Framework.DataSources.Explicit;

namespace Zcu.CurvatureBenchmark.DataSources.Explicit
{
    /// <summary>
    /// Abstract implementation of <see cref="IExplicitSampler"/>.
    /// </summary>
    public abstract class AbstractSampler : AbstractModule, IExplicitSampler
    {
        /// <summary>
        /// Function distortion is a value from 0 to 1 that changes the function value  
        /// </summary>
        public double Distorsion { get; set; }

        /// <summary>
        /// Bounding box MinX
        /// </summary>
        public virtual double MinX { get; set; }

        /// <summary>
        /// Bounding box MaxX
        /// </summary>
        public virtual double MaxX { get; set; }

        /// <summary>
        /// Bounding box MinY
        /// </summary>
        public virtual double MinY { get; set; }

        /// <summary>
        /// Bounding box MaxY
        /// </summary>
        public virtual double MaxY { get; set; }

        /// <summary>
        /// Reference to <see cref="IExplicitFunction"/> to be sampled.
        /// </summary>
        public IExplicitFunction Function { get; protected set; }

        /// <inheritdoc/>
        public Point3D[] Points { get; protected set; }

        /// <inheritdoc/>
        public double[] Values { get; protected set; }

        /// <inheritdoc/>
        public double[] K1 { get; protected set; }

        /// <inheritdoc/>
        public double[] K2 { get; protected set; }

        /// <inheritdoc/>
        public Point3D[] Normals { get; protected set; }

        /// <summary>
        /// Sampler setup.
        /// </summary>
        public AbstractSampler()
        {
            MinX = -1;
            MaxX = 1;
            MinY = -1;
            MaxY = 1;
        }

        /// <inheritdoc/>
        public abstract void Sample(IExplicitFunction function);

        /// <summary>
        /// Distort values. Must be called at the end of Sample function.
        /// </summary>
        protected void Distort()
        {
            double maxVal = Points[0].Z;
            double minVal = Points[0].Z;
            for (int i=1; i<Points.Length; i++)
            {
                if (Values[i] > maxVal) maxVal = Points[i].Z;
                if (Values[i] < minVal) minVal = Points[i].Z;
            }

            double range = maxVal - minVal;

            for (int i = 0; i < Points.Length; i++)
                Points[i].Z += Rand.NextDouble(-Distorsion, Distorsion) * range;
        }
    }
}
