﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Zcu.CurvatureBenchmark.Framework;
using Zcu.CurvatureBenchmark.Framework.DataSources.Explicit;

namespace Zcu.CurvatureBenchmark.DataSources.Explicit
{
    /// <summary>
    /// Abstract class for RegularGrid sampler. The sampled points are stored row by row.
    /// </summary>
    /// <author>Petr Vanecek</author>
    public abstract class RectangularGridSampler : AbstractSampler
    {
        /// <summary>
        /// Number of samples in x direction
        /// </summary>
        public virtual int SamplesX { get; set; }

        /// <summary>
        /// Number of samples in y direction
        /// </summary>
        public virtual int SamplesY { get; set; }

        /// <summary>
        /// Determines whether or not noise will be added to the XY samples
        /// </summary>
        public bool Noise { get; set; }

        /// <summary>
        /// Determines the noise strength, relative to edge length;
        /// </summary>
        public double NoiseStrength { get; set; }

        /// <summary>
        /// Set default values.
        /// </summary>
        public RectangularGridSampler() : base()
        {
            SamplesX = 2;
            SamplesY = 2;
        }

        /// <summary>
        /// Count of vertices.
        /// </summary>
        protected int count;

        /// <inheritdoc/>
        public override void Sample(IExplicitFunction function) {
            count = SamplesX * SamplesY;
            Function = function;

            Values = new double[count];
            Points = new Point3D[count];
            K1 = new double[count];
            K2 = new double[count];
            Normals = new Point3D[count];
        }
    }
}
