﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Zcu.CurvatureBenchmark.Framework;
using Zcu.CurvatureBenchmark.Framework.DataSources.Explicit;

namespace Zcu.CurvatureBenchmark.DataSources.Explicit
{
    /// <summary>
    /// Regular rectangular grid sampler.
    /// </summary>
    /// <example>
    /// More examples in !TestData/Examples-ExplicitSamplers.xml
    /// </example>
    /// <author>Petr Vanecek</author>
    public class RegularRectangularGridSampler : RectangularGridSampler
    {
        /// <inheritdoc/>
        public override void Sample(IExplicitFunction function) 
        {
            Progress = 0;
            base.Sample(function);
            double stepX = (MaxX - MinX) / (SamplesX - 1);
            double stepY = (MaxY - MinY) / (SamplesY - 1);

            int index = 0;

            for (int iy = 0; iy < SamplesY; iy++)
            {
                double y = MinY + iy * stepY;
                for (int ix = 0; ix < SamplesX; ix++)
                {
                    double x = MinX + ix * stepX;
                    double nx = 0;
                    double ny = 0;
                    if (Noise)
                    {
                        nx = stepX * Rand.NextDouble(-1, 1) * NoiseStrength;
                        ny = stepY * Rand.NextDouble(-1, 1) * NoiseStrength;
                    }
                    function.Evaluate(x+nx, y+ny);
                    Points[index] = new Point3D(x+nx, y+ny, function.F);
                    Values[index] = function.F;
                    K1[index] = function.K1;
                    K2[index] = function.K2;
                    Normals[index] = function.Normal;
                    index++;
                }
            }
            Distort();
            Progress = 100;
        }
    }
}
