﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Zcu.CurvatureBenchmark.Framework;
using Zcu.CurvatureBenchmark.Framework.DataSources.Explicit;

namespace Zcu.CurvatureBenchmark.DataSources.Explicit
{
    /// <summary>
    /// Uniformly sampled rectangle area.
    /// </summary>
    /// <example>
    /// More examples in !TestData/Examples-ExplicitSamplers.xml
    /// </example>
    /// <author>Petr Vanecek</author>
    public class UniformRandomSampler : AbstractSampler
    {
        /// <summary>
        /// Number of samples
        /// </summary>
        public virtual int Samples { get; set; }

        /// <summary>
        /// Default values.
        /// </summary>
        public UniformRandomSampler() : base()
        {
            Samples = 1000;
        }

        /// <inheritdoc/>
        public override void Sample(IExplicitFunction function) {
            Progress = 0;
            Function = function;

            Values = new double[Samples];
            Points = new Point3D[Samples];
            K1 = new double[Samples];
            K2 = new double[Samples];
            Normals = new Point3D[Samples];


            for (int i = 0; i < Samples; i++)
            {
                    double x = Rand.NextDouble(MinX, MaxX);
                    double y = Rand.NextDouble(MinY, MaxY);
                    Function.Evaluate(x, y);
                    Points[i] = new Point3D(x, y, Function.F);
                    Values[i] = Function.F;
                    K1[i] = Function.K1;
                    K2[i] = Function.K2;
                    Normals[i] = Function.Normal;
                    Progress = (byte)(100 * i / Samples);
            }
            Distort();
            Progress = 100;
        }
    }
}
