﻿
using System.Xml;
using Zcu.CurvatureBenchmark.Framework;
using Zcu.CurvatureBenchmark.Framework.DataSources;
using Zcu.CurvatureBenchmark.Framework.DataSources.Explicit;



namespace Zcu.CurvatureBenchmark.DataSources.Explicit
{
    /// <summary>
    /// Generic datasource for IExplicit* interfaces. This datasource loads function, sampler, triangularizator 
    /// from the config xml node and produces TriangleMesh[].
    /// </summary>
    /// <author>Petr Vanecek</author>
    public class ExplicitDataSource : AbstractDataSource
    {
        IExplicitFunction function;
        IExplicitSampler sampler;
        IExplicitTriangularizator triangularizator;

        /// <inheritdoc/>
        public override void SetConfig(XmlNode configNode)
        {
            DatasetCount = 1;
            base.SetConfig(configNode);

            foreach (XmlNode node in configNode.ChildNodes)
            {
                switch (node.Name)
                {
                    case "Function":
                        function = TestScenario.GetModule<IExplicitFunction>(node.ChildNodes[0]);
                        break;
                    case "Sampler":
                        sampler = TestScenario.GetModule<IExplicitSampler>(node.ChildNodes[0]);
                        sampler.ProgressChanged += sampler_ProgressChanged;
                        break;
                    case "Triangularizator":
                        triangularizator = TestScenario.GetModule<IExplicitTriangularizator>(node.ChildNodes[0]);
                        triangularizator.ProgressChanged += triangularizator_ProgressChanged;
                        break;
                    default: break;
                }
            }
        }

        void triangularizator_ProgressChanged(IModule module, NotifyProgressChangedEventArgs args)
        {
            Progress = (byte)(100*currentSet/DatasetCount + (50 + args.Progress / 2) / DatasetCount);
        }

        void sampler_ProgressChanged(IModule module, NotifyProgressChangedEventArgs args)
        {
            Progress = (byte)(100*currentSet/DatasetCount + (0 + args.Progress / 2) / DatasetCount);
        }

        /// <inheritdoc/>
        public override TriangleMesh[] GenerateData()
        {
            TriangleMesh[] result = new TriangleMesh[DatasetCount];
            Progress = 0;
            for (currentSet = 0; currentSet < DatasetCount; currentSet++)
            {
                function.SetParams();
                sampler.Sample(function);
                triangularizator.Triangulate(sampler);
                result[currentSet] = triangularizator.TriangleMesh;
            }
            Progress = 100;
            return result;
        }

        int currentSet;

        /// <inheritdoc/>
        public override string ToString()
        {
            return this.function.ToString();
        }

    }
}
