﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Zcu.CurvatureBenchmark.Framework;
using Zcu.CurvatureBenchmark.Framework.DataSources.Implicit;

namespace Zcu.CurvatureBenchmark.DataSources.Implicit
{
    /// <summary>
    /// Abstract class for rectangular box sampler.
    /// </summary>
    public class RectangularBoxSampler : AbstractModule, IImplicitSampler      
    {
        /// <summary>
        /// Bounding box MinX
        /// </summary>
        public virtual double MinX { get; set; }

        /// <summary>
        /// Bounding box MaxX
        /// </summary>
        public virtual double MaxX { get; set; }

        /// <summary>
        /// Bounding box MinY
        /// </summary>
        public virtual double MinY { get; set; }

        /// <summary>
        /// Bounding box MaxY
        /// </summary>
        public virtual double MaxY { get; set; }

        /// <summary>
        /// Bounding box MinZ
        /// </summary>
        public virtual double MinZ { get; set; }

        /// <summary>
        /// Bounding box MaxZ
        /// </summary>
        public virtual double MaxZ { get; set; }

        /// <summary>
        /// Number of samples in x direction
        /// </summary>
        public virtual int SamplesX { get; set; }

        /// <summary>
        /// Number of samples in y direction
        /// </summary>
        public virtual int SamplesY { get; set; }

        /// <summary>
        /// Number of samples in z direction
        /// </summary>
        public virtual int SamplesZ { get; set; }

        /// <summary>
        /// Initialization.
        /// </summary>
        public RectangularBoxSampler()
        {
            MinX = -1;
            MaxX = 1;
            MinY = -1;
            MaxY = 1;
            MinZ = -1;
            MaxZ = 1;
            SamplesX = 2;
            SamplesY = 2;
            SamplesZ = 2;
        }

        /// <summary>
        /// Number of samples
        /// </summary>
        protected int count;

        /// <inheritdoc/>
        public virtual void Sample(IImplicitFunction function) {
            count = SamplesX * SamplesY * SamplesZ;
            Function = function;

            Values = new double[count];
            Points = new Point3D[count];
            K1 = new double[count];
            K2 = new double[count];
            Normals = new Point3D[count];
        }

        /// <inheritdoc/>
        public Point3D[] Points { get; private set;}

        /// <inheritdoc/>
        public double[] Values { get; private set;}

        /// <inheritdoc/>
        public double[] K1 { get; private set;}

        /// <inheritdoc/>
        public double[] K2 { get; private set;}

        /// <inheritdoc/>
        public Point3D[] Normals { get; private set;}

        /// <inheritdoc/>
        public IImplicitFunction Function {get; private set;}
    }
}
