﻿/*
 * This is free and unencumbered software released into the public domain.
 *
 * Anyone is free to copy, modify, publish, use, compile, sell, or
 * distribute this software, either in source code form or as a compiled
 * binary, for any purpose, commercial or non-commercial, and by any
 * means.
 *
 * In jurisdictions that recognize copyright laws, the author or authors
 * of this software dedicate any and all copyright interest in the
 * software to the public domain. We make this dedication for the benefit
 * of the public at large and to the detriment of our heirs and
 * successors. We intend this dedication to be an overt act of
 * relinquishment in perpetuity of all present and future rights to this
 * software under copyright law.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * For more information, please refer to <http://unlicense.org/>
 */

/*
 * https://github.com/tuckbone/DualContouringCSharp
 * 
 * Please do not use this code as the "Ideal way to code in C#"!!! 
 * It is by no means optimized for managed code. This is nothing but a direct port 
 * of NGildea's Dual Contouring Impl from his blog.
 * 
 */
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Zcu.CurvatureBenchmark.Framework;
using MathNet.Numerics.LinearAlgebra;
using MathNet.Numerics.LinearAlgebra.Double;
using MathNet.Numerics.LinearAlgebra.Factorization;

namespace Zcu.CurvatureBenchmark.DataSources.Implicit
{

    internal class Octree
    {
        int pocetBodu = 0;
        public RectangularBoxSampler gridsampler;
        public bool exact;
        public int iteration;
        public static int MATERIAL_AIR = 1;
        public static int MATERIAL_SOLID = 0;
        double step = 0;
        //public static double QEF_ERROR = 1e-6;
        public static double QEF_ERROR = Utils.EPSILON;
        public static int QEF_SWEEPS = 4;

        #region Readonly Variables
        public Point3D[] CHILD_MIN_OFFSETS = new Point3D[8];

        // data from the original DC impl, drives the contouring process

        public static readonly int[][] edgevmap = new int[12][]
    {
 //   new int[2]{ 0, 1},new int[2]{1, 2 },new int[2]{2, 3 },new int[2]{3, 0},	// x-axis 
	//new int[2]{4, 5 },new int[2]{5, 6 },new int[2]{6, 7},new int[2]{7, 4},	// y-axis
	//new int[2]{0, 4},new int[2]{1, 5},new int[2]{ 2, 6 },new int[2]{ 3, 7 }     // z-axis

        //zatim nej
        new int[2]{2,4},new int[2]{1,5},new int[2]{2,6},new int[2]{3,7},	// x-axis 
	new int[2]{0,2},new int[2]{1,3},new int[2]{4,6},new int[2]{5,7},	// y-axis
	new int[2]{0,1},new int[2]{2,3},new int[2]{4,5},new int[2]{6,7}     // z-axis
       
      };

        public static readonly int[] edgemask = { 5, 3, 6 };

    //    public static readonly int[][] vertMap = new int[8][]
    //{
    //new int[3]{0,0,0},
    //new int[3]{0,0,1},
    //new int[3]{0,1,0},
    //new int[3]{0,1,1},
    //new int[3]{1,0,0},
    //new int[3]{1,0,1},
    //new int[3]{1,1,0},
    //new int[3]{1,1,1}
    //};

    //    public static readonly int[][] faceMap = new int[6][]
    //{
    //new int[4]{4, 8, 5, 9},
    //new int[4]{6, 10, 7, 11},
    //new int[4]{0, 8, 1, 10},
    //new int[4]{2, 9, 3, 11},
    //new int[4]{0, 4, 2, 6},
    //new int[4]{1, 5, 3, 7}
    //};

        public static readonly int[][] cellProcFaceMask = new int[12][]
    {
    new int[3]{0,4,0},
    new int[3]{1,5,0},
    new int[3]{2,6,0},
    new int[3]{3,7,0},
    new int[3]{0,2,1},
    new int[3]{4,6,1},
    new int[3]{1,3,1},
    new int[3]{5,7,1},
    new int[3]{0,1,2},
    new int[3]{2,3,2},
    new int[3]{4,5,2},
    new int[3]{6,7,2}
    };

        public static readonly int[][] cellProcEdgeMask = new int[6][]
    {
    new int[5]{0,1,2,3,0},
    new int[5]{4,5,6,7,0},
    new int[5]{0,4,1,5,1},
    new int[5]{2,6,3,7,1},
    new int[5]{0,2,4,6,2},
    new int[5]{1,3,5,7,2}
    };

        public static readonly int[][][] faceProcFaceMask = new int[3][][]
    {
    new int[4][]{ new int[3]{4,0,0}, new int[3]{5,1,0}, new int[3]{6,2,0}, new int[3]{7,3,0} },
    new int[4][]{ new int[3]{2,0,1}, new int[3]{6,4,1}, new int[3]{3,1,1}, new int[3]{7,5,1} },
    new int[4][]{ new int[3]{1,0,2}, new int[3]{3,2,2}, new int[3]{5,4,2}, new int[3]{7,6,2} }
    };

        public static readonly int[][][] faceProcEdgeMask = new int[3][][]
    {
    new int[4][]{new int[6]{1,4,0,5,1,1},new int[6]{1,6,2,7,3,1},new int[6]{0,4,6,0,2,2},new int[6]{0,5,7,1,3,2}},
    new int[4][]{new int[6]{0,2,3,0,1,0},new int[6]{0,6,7,4,5,0},new int[6]{1,2,0,6,4,2},new int[6]{1,3,1,7,5,2}},
    new int[4][]{new int[6]{1,1,0,3,2,0},new int[6]{1,5,4,7,6,0},new int[6]{0,1,5,0,4,1},new int[6]{0,3,7,2,6,1}}
    };

        public static readonly int[][][] edgeProcEdgeMask = new int[3][][]
    {
    new int[2][]{new int[5]{3,2,1,0,0},new int[5]{7,6,5,4,0}},
    new int[2][]{new int[5]{5,1,4,0,1},new int[5]{7,3,6,2,1}},
    new int[2][]{new int[5]{6,4,2,0,2},new int[5]{7,5,3,1,2}},
    };

        public static readonly int[][] processEdgeMask = new int[3][]
    {
    new int[4]{3,2,1,0},new int[4]{7,5,6,4},new int[4]{11,10,9,8}
    };
        #endregion


        public  OctreeNode SimplifyOctree(OctreeNode node, double threshold)
        {
            if (node == null)
            {
                return null;
            }

            if (node.Type != OctreeNodeType.Node_Internal)
            {
                // can't simplify!
                return node;
            }

            QefSolver qef = new QefSolver();
            int[] signs = new int[8] { -1, -1, -1, -1, -1, -1, -1, -1 };
            int midsign = -1;
            int edgeCount = 0;
            bool isCollapsible = true;

            for (int i = 0; i < 8; i++)
            {
                node.children[i] = SimplifyOctree(node.children[i], threshold);

                if (node.children[i] != null)
                {
                    OctreeNode child = node.children[i];

                    if (child.Type == OctreeNodeType.Node_Internal)
                    {
                        isCollapsible = false;
                    }
                    else
                    {
                        qef.add(child.drawInfo.qef);

                        midsign = (child.drawInfo.corners >> (7 - i)) & 1;
                        signs[i] = (child.drawInfo.corners >> i) & 1;

                        edgeCount++;
                    }
                }
            }

            if (!isCollapsible)
            {
                // at least one child is an internal node, can't collapse
                return node;
            }

            Point3D qefPosition = new Point3D();
            qef.solve(out qefPosition, QEF_ERROR, QEF_SWEEPS, QEF_ERROR);
            double error = qef.getError();

            // convert to glm vec3 for ease of use
            Point3D position = new Point3D(qefPosition.X, qefPosition.Y, qefPosition.Z);

            // at this point the masspoint will actually be a sum, so divide to make it the average
            if (error > threshold)
            {
                // this collapse breaches the threshold
                return node;
            }

            if (position.X < node.min.X || position.X > (node.min.X + node.size) ||
                position.Y < node.min.Y || position.Y > (node.min.Y + node.size) ||
                position.Z < node.min.Z || position.Z > (node.min.Z + node.size))
            {
                position = qef.getMassPoint();
            }

            // change the node from an internal node to a 'psuedo leaf' node
            OctreeDrawInfo drawInfo = new OctreeDrawInfo();
            drawInfo.corners = 0;
            drawInfo.index = -1;

            for (int i = 0; i < 8; i++)
            {
                if (signs[i] == -1)
                {
                    // Undetermined, use centre sign instead
                    drawInfo.corners |= (midsign << i);
                }
                else
                {
                    drawInfo.corners |= (signs[i] << i);
                }
            }

            drawInfo.averageNormal = new Point3D();
            for (int i = 0; i < 8; i++)
            {
                if (node.children[i] != null)
                {
                    OctreeNode child = node.children[i];
                    if (child.Type == OctreeNodeType.Node_Psuedo ||
                        child.Type == OctreeNodeType.Node_Leaf)
                    {
                        drawInfo.averageNormal += child.drawInfo.averageNormal;
                    }
                }
            }

            drawInfo.averageNormal.Normalize();

            drawInfo.position = position;
            drawInfo.qef = qef.getData();

            for (int i = 0; i < 8; i++)
            {
                DestroyOctree(node.children[i]);
                node.children[i] = null;
            }

            node.Type = OctreeNodeType.Node_Psuedo;
            node.drawInfo = drawInfo;

            return node;
        }

        public  void GenerateVertexIndices(OctreeNode node, List<Point3D> vertexBuffer)
        {
            if (node == null)
            {
                return;
            }

            if (node.Type != OctreeNodeType.Node_Leaf)
            {
                for (int i = 0; i < 8; i++)
                {
                    GenerateVertexIndices(node.children[i], vertexBuffer);
                }
            }

            if (node.Type != OctreeNodeType.Node_Internal)
            {
                node.drawInfo.index = vertexBuffer.Count;

                vertexBuffer.Add(node.drawInfo.position);
            }
        }

        public  void ContourProcessEdge(OctreeNode[] node, int dir, List<int> indexBuffer)
        {
            int minSize = 1000000;      // arbitrary big number
            int minIndex = 0;
            int[] indices = new int[4] { -1, -1, -1, -1 };
            bool flip = false;
            bool[] signChange = new bool[4] { false, false, false, false };

            for (int i = 0; i < 4; i++)
            {
                int edge = processEdgeMask[dir][i];
                int c1 = edgevmap[edge][0];
                int c2 = edgevmap[edge][1];

                int m1 = (node[i].drawInfo.corners >> c1) & 1;
                int m2 = (node[i].drawInfo.corners >> c2) & 1;

                if (node[i].size < minSize)
                {
                    minSize = node[i].size;
                    minIndex = i;
                    // flip = m1 != MATERIAL_AIR;
                    if (m1 == 0) {
                        flip = true;
                    }
                }

                indices[i] = node[i].drawInfo.index;

                //signChange[i] =
                //    (m1 == MATERIAL_AIR && m2 != MATERIAL_AIR) ||
                //    (m1 != MATERIAL_AIR && m2 == MATERIAL_AIR);

                signChange[i] =
                    (Math.Abs(m1) == 0 && Math.Abs(m2) > 0+Utils.EPSILON) ||
                    (Math.Abs(m1) > 0 + Utils.EPSILON && Math.Abs(m2) == 0);
            }

            if (signChange[minIndex])
            {
                if (!flip)
                {
                    indexBuffer.Add(indices[0]);
                    indexBuffer.Add(indices[1]);
                    indexBuffer.Add(indices[3]);

                    indexBuffer.Add(indices[0]);
                    indexBuffer.Add(indices[3]);
                    indexBuffer.Add(indices[2]);
                }
                else
                {
                    indexBuffer.Add(indices[0]);
                    indexBuffer.Add(indices[3]);
                    indexBuffer.Add(indices[1]);

                    indexBuffer.Add(indices[0]);
                    indexBuffer.Add(indices[2]);
                    indexBuffer.Add(indices[3]);
                }

            }
        }

        public  void ContourEdgeProc(OctreeNode[] node, int dir, List<int> indexBuffer)
        {
            if (node[0] == null || node[1] == null || node[2] == null || node[3] == null)
            {
                return;
            }

            if (node[0].Type != OctreeNodeType.Node_Internal &&
                node[1].Type != OctreeNodeType.Node_Internal &&
                node[2].Type != OctreeNodeType.Node_Internal &&
                node[3].Type != OctreeNodeType.Node_Internal)
            {
                ContourProcessEdge(node, dir, indexBuffer);
            }
            else
            {
                for (int i = 0; i < 2; i++)
                {
                    OctreeNode[] edgeNodes = new OctreeNode[4];
                    int[] c = new int[4]
                {
                edgeProcEdgeMask[dir][i][0],
                edgeProcEdgeMask[dir][i][1],
                edgeProcEdgeMask[dir][i][2],
                edgeProcEdgeMask[dir][i][3],
                };

                    for (int j = 0; j < 4; j++)
                    {
                        if (node[j].Type == OctreeNodeType.Node_Leaf || node[j].Type == OctreeNodeType.Node_Psuedo)
                        {
                            edgeNodes[j] = node[j];
                        }
                        else
                        {
                            edgeNodes[j] = node[j].children[c[j]];
                        }
                    }

                    ContourEdgeProc(edgeNodes, edgeProcEdgeMask[dir][i][4], indexBuffer);
                }
            }
        }

        public  void ContourFaceProc(OctreeNode[] node, int dir, List<int> indexBuffer)
        {
            if (node[0] == null || node[1] == null)
            {
                return;
            }

            if (node[0].Type == OctreeNodeType.Node_Internal ||
                node[1].Type == OctreeNodeType.Node_Internal)
            {
                for (int i = 0; i < 4; i++)
                {
                    OctreeNode[] faceNodes = new OctreeNode[2];
                    int[] c = new int[2]
                {
                faceProcFaceMask[dir][i][0],
                faceProcFaceMask[dir][i][1],
                };

                    for (int j = 0; j < 2; j++)
                    {
                        if (node[j].Type != OctreeNodeType.Node_Internal)
                        {
                            faceNodes[j] = node[j];
                        }
                        else
                        {
                            faceNodes[j] = node[j].children[c[j]];
                        }
                    }

                    ContourFaceProc(faceNodes, faceProcFaceMask[dir][i][2], indexBuffer);
                }

                int[][] orders = new int[2][]
            {
            new int[4]{ 0, 0, 1, 1 },
            new int[4]{ 0, 1, 0, 1 },
            };

                for (int i = 0; i < 4; i++)
                {
                    OctreeNode[] edgeNodes = new OctreeNode[4];
                    int[] c = new int[4]
                {
                faceProcEdgeMask[dir][i][1],
                faceProcEdgeMask[dir][i][2],
                faceProcEdgeMask[dir][i][3],
                faceProcEdgeMask[dir][i][4],
                };

                    int[] order = orders[faceProcEdgeMask[dir][i][0]];
                    for (int j = 0; j < 4; j++)
                    {
                        if (node[order[j]].Type == OctreeNodeType.Node_Leaf ||
                            node[order[j]].Type == OctreeNodeType.Node_Psuedo)
                        {
                            edgeNodes[j] = node[order[j]];
                        }
                        else
                        {
                            edgeNodes[j] = node[order[j]].children[c[j]];
                        }
                    }

                    ContourEdgeProc(edgeNodes, faceProcEdgeMask[dir][i][5], indexBuffer);
                }
            }
        }

        public  void ContourCellProc(OctreeNode node, List<int> indexBuffer)
        {
            if (node == null)
            {
                return;
            }

            if (node.Type == OctreeNodeType.Node_Internal)
            {
                for (int i = 0; i < 8; i++)
                {
                    ContourCellProc(node.children[i], indexBuffer);
                }

                for (int i = 0; i < 12; i++)
                {
                    OctreeNode[] faceNodes = new OctreeNode[2];
                    int[] c = { cellProcFaceMask[i][0], cellProcFaceMask[i][1] };

                    faceNodes[0] = node.children[c[0]];
                    faceNodes[1] = node.children[c[1]];

                    ContourFaceProc(faceNodes, cellProcFaceMask[i][2], indexBuffer);
                }

                for (int i = 0; i < 6; i++)
                {
                    OctreeNode[] edgeNodes = new OctreeNode[4];
                    int[] c = new int[4]
                {
                cellProcEdgeMask[i][0],
                cellProcEdgeMask[i][1],
                cellProcEdgeMask[i][2],
                cellProcEdgeMask[i][3],
                };

                    for (int j = 0; j < 4; j++)
                    {
                        edgeNodes[j] = node.children[c[j]];
                    }

                    ContourEdgeProc(edgeNodes, cellProcEdgeMask[i][4], indexBuffer);
                }
            }
        }


        public OctreeNode ConstructLeaf(OctreeNode leaf)
        {
            if (leaf == null || leaf.size != 1)
            {
                return null;
            }

            int corners = 0;
            for (int i = 0; i < 8; i++)
            {
                Point3D cornerPos = leaf.min + CHILD_MIN_OFFSETS[i];

                gridsampler.Function.EvaluateValue(cornerPos.X, cornerPos.Y, cornerPos.Z);

                double density = gridsampler.Function.F;
                if (density < 0 + Utils.EPSILON)
                {
                    corners |= (1 << i);
                    //pocet++;
                }

            }

            if (corners == 0 || corners == 255)
            {
                // voxel is full inside or outside the volume
                //delete leaf
                //setting as null isn't required by the GC in C#... but its in the original, so why not!
                leaf = null;
                return null;
            }

            // otherwise the voxel contains the surface, so find the edge intersections
            const int MAX_CROSSINGS = 6;
            int edgeCount = 0;
            Point3D averageNormal = new Point3D();
            QefSolver qef = new QefSolver();


            //compute intersection points and normal and add them to qef function
            for (int i = 0; i < 12 && edgeCount < MAX_CROSSINGS; i++)
            {
                //for (int i = 0; i < 12 ; i++)
                //{
                int c1 = edgevmap[i][0];
                int c2 = edgevmap[i][1];

                int m1 = (corners >> c1) & 1;
                int m2 = (corners >> c2) & 1;

                if (m1 == m2)
                    continue;

                //if ((m1 == MATERIAL_AIR && m2 == MATERIAL_AIR) || (m1 == MATERIAL_SOLID && m2 == MATERIAL_SOLID))
                //{
                //    // no zero crossing on this edge
                //    continue;
                //}

                Point3D p1 = leaf.min + CHILD_MIN_OFFSETS[c1];
                Point3D p2 = leaf.min + CHILD_MIN_OFFSETS[c2];
                // Point3D p = ApproximateZeroCrossingPosition(p1, p2);
                Point3D p = exactValue(p1, p2);
                gridsampler.Function.Evaluate(p.X, p.Y, p.Z);
                Point3D n = gridsampler.Function.Normal; //CalculateSurfaceNormal(p);
                qef.add(p.X, p.Y, p.Z, n.X, n.Y, n.Z);

                averageNormal += n;

                edgeCount++;
            }

            Point3D qefPosition = new Point3D();
            //return one point on the surface of object
            qef.solve(out qefPosition, QEF_ERROR, QEF_SWEEPS, QEF_ERROR);

            if (exact)
            {
                Point3D opf = new Point3D(0, 0, 0);
                gridsampler.Function.Evaluate(qefPosition.X, qefPosition.Y, qefPosition.Z);
                Point3D n1 = gridsampler.Function.Normal; //CalculateSurfaceNormal(p);
                double step = (this.step * Math.Sqrt(3));
                Point3D transVect = new Point3D(n1.X * step, n1.Y * step, n1.Z * step);
                double w = gridsampler.Function.F;     
                Point3D newPoint;

                //direction depend on surface (in / out)
                if (w > 0)
                {
                    newPoint = qefPosition - transVect;
                }
                else
                {
                    newPoint = qefPosition + transVect;
                }


                qefPosition = exactValue(qefPosition, newPoint);
            }

            OctreeDrawInfo drawInfo = new OctreeDrawInfo();
            drawInfo.corners = 0;
            drawInfo.index = -1;
            drawInfo.position = new Point3D(qefPosition.X, qefPosition.Y, qefPosition.Z);
            drawInfo.qef = qef.getData();

            Point3D min = leaf.min;
            Point3D max = new Point3D(leaf.min.X + leaf.size, leaf.min.Y + leaf.size, leaf.min.Z + leaf.size);
            if (drawInfo.position.X < min.X || drawInfo.position.X > max.X ||
                drawInfo.position.Y < min.Y || drawInfo.position.Y > max.Y ||
                drawInfo.position.Z < min.Z || drawInfo.position.Z > max.Z)
            {
                drawInfo.position = qef.getMassPoint();
            }

            Point3D op = new Point3D(averageNormal.X / (double)edgeCount, averageNormal.Y / (double)edgeCount, averageNormal.Z / (double)edgeCount).Normalize();

            drawInfo.averageNormal =  op;
            drawInfo.corners = corners;

            leaf.Type = OctreeNodeType.Node_Leaf;
            leaf.drawInfo = drawInfo;
            pocetBodu++;
            return leaf;
        }

        public  OctreeNode ConstructOctreeNodes(OctreeNode node)
        {
            if (node == null)
            {
                return null;
            }

            if (node.size == 1)
            {
                return ConstructLeaf(node);
            }

            int childSize = node.size / 2;
            bool hasChildren = false;

            for (int i = 0; i < 8; i++)
            {
                OctreeNode child = new OctreeNode();
                child.size = childSize;
                child.min = node.min + (CHILD_MIN_OFFSETS[i] * childSize);
                child.Type = OctreeNodeType.Node_Internal;

                node.children[i] = ConstructOctreeNodes(child);
                hasChildren |= (node.children[i] != null);
            }

            if (!hasChildren)
            {
                //delete leaf
                //setting as null isn't required by the GC in C#... but its in the original, so why not!
                node = null;
                return null;
            }

            return node;
        }

        public OctreeNode BuildOctree(Point3D min, int size, double threshold, RectangularBoxSampler gridsampler, bool exact, int iteration)
        {
            //Debug.Log(string.Format("Building Octree at {0}, with size of {1} and threshold of {2}", min, size, threshold));
           this.gridsampler= gridsampler;
            this.exact = exact;
            this.iteration = iteration;
            OctreeNode root = new OctreeNode();

            double stepX = (gridsampler.MaxX - gridsampler.MinX) / (gridsampler.SamplesX -1);
            double stepY = (gridsampler.MaxY - gridsampler.MinY) / (gridsampler.SamplesY -1);
            double stepZ = (gridsampler.MaxZ - gridsampler.MinZ) / (gridsampler.SamplesZ -1);
            step = stepX;
           

            CHILD_MIN_OFFSETS[0] = new Point3D(0, 0, 0);
            CHILD_MIN_OFFSETS[1] = new Point3D(0, 0, stepZ);
            CHILD_MIN_OFFSETS[2] = new Point3D(0, stepY, 0);
            CHILD_MIN_OFFSETS[3] = new Point3D(0, stepY, stepZ);
            CHILD_MIN_OFFSETS[4] = new Point3D(stepX, 0, 0);
            CHILD_MIN_OFFSETS[5] = new Point3D(stepX, 0, stepZ);
            CHILD_MIN_OFFSETS[6] = new Point3D(stepX, stepY, 0);
            CHILD_MIN_OFFSETS[7] = new Point3D(stepX, stepY, stepZ);

            //CHILD_MIN_OFFSETS[0] = new Point3D(0, 0, 0);
            //CHILD_MIN_OFFSETS[1] = new Point3D(0, stepY, 0);
            //CHILD_MIN_OFFSETS[2] = new Point3D(0, stepY, stepZ);
            //CHILD_MIN_OFFSETS[3] = new Point3D(0, 0, stepZ);
            //CHILD_MIN_OFFSETS[4] = new Point3D(stepX, 0, 0);
            //CHILD_MIN_OFFSETS[5] = new Point3D(stepX, stepY, 0);
            //CHILD_MIN_OFFSETS[6] = new Point3D(stepX, stepY, stepZ);
            //CHILD_MIN_OFFSETS[7] = new Point3D(stepX, 0, stepZ);

            //new Vector3(0, 0, 0),
            //new Vector3(0, 0, 1),
            //new Vector3(0, 1, 0),
            //new Vector3(0, 1, 1),
            //new Vector3(1, 0, 0),
            //new Vector3(1, 0, 1),
            //new Vector3(1, 1, 0),
            //new Vector3(1, 1, 1),

            double numSteps = ((gridsampler.MaxX - gridsampler.MinX) / stepX);
            //root.min = new Point3D(numSteps, numSteps, numSteps);
            root.min = min;            
            root.size = size;
            root.Type = OctreeNodeType.Node_Internal;

            root = ConstructOctreeNodes(root);
            root = SimplifyOctree(root, threshold);
            int t = pocetBodu;
            return root;
        }

        public void GenerateMeshFromOctree(OctreeNode node,out List<Point3D> vertexBuffer,out List<int> indexBuffer)
        {
           

            vertexBuffer = new List<Point3D>();
            indexBuffer = new List<int>();
            if (node == null)
            {
                return;
            }

            GenerateVertexIndices(node, vertexBuffer);
            ContourCellProc(node, indexBuffer);
            

            Point3D[] vertArray = new Point3D[vertexBuffer.Count];
         
            for (int i = 0; i < vertexBuffer.Count; i++)
            {
                vertArray[i] = vertexBuffer[i];
               
            }
            

        }
        
        public  void DestroyOctree(OctreeNode node)
        {
            if (node == null)
            {
                return;
            }

            for (int i = 0; i < 8; i++)
            {
                DestroyOctree(node.children[i]);
            }

            node = null;
        }

        // Bisection method
        public Point3D exactValue(Point3D a, Point3D b)
        {
            Point3D exactP = new Point3D(0, 0, 0);
            double w0 = 0;
            //Point3D a = cube[v1] + orig;
            //Point3D b = cube[v2] + orig;
            gridsampler.Function.EvaluateValue(a.X, a.Y, a.Z);
            double w1 = gridsampler.Function.F;
            gridsampler.Function.EvaluateValue(b.X, b.Y, b.Z);
            double w2 = gridsampler.Function.F;

            for (int i = 0; i < iteration; i++)
            {
                exactP = (a + b) / 2;
                gridsampler.Function.EvaluateValue(exactP.X, exactP.Y, exactP.Z);
                w0 = gridsampler.Function.F;

                if (w0 < 0)
                {
                    if (w1 < w2)
                    {
                        a = exactP;
                        w1 = w0;
                    }
                    else
                    {
                        b = exactP;
                        w2 = w0;
                    }

                }
                else
                {
                    if (w1 > w2)
                    {
                        a = exactP;
                        w1 = w0;
                    }
                    else
                    {
                        b = exactP;
                        w2 = w0;
                    }
                }
            }

            exactP = (a + b) / 2;
            return exactP;
        }



    }
}