﻿/*
 * This is free and unencumbered software released into the public domain.
 *
 * Anyone is free to copy, modify, publish, use, compile, sell, or
 * distribute this software, either in source code form or as a compiled
 * binary, for any purpose, commercial or non-commercial, and by any
 * means.
 *
 * In jurisdictions that recognize copyright laws, the author or authors
 * of this software dedicate any and all copyright interest in the
 * software to the public domain. We make this dedication for the benefit
 * of the public at large and to the detriment of our heirs and
 * successors. We intend this dedication to be an overt act of
 * relinquishment in perpetuity of all present and future rights to this
 * software under copyright law.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * For more information, please refer to <http://unlicense.org/>
 */

/*
 * https://github.com/tuckbone/DualContouringCSharp
 * 
 * Please do not use this code as the "Ideal way to code in C#"!!! 
 * It is by no means optimized for managed code. This is nothing but a direct port 
 * of NGildea's Dual Contouring Impl from his blog.
 * 
 */
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Zcu.CurvatureBenchmark.Framework;
using MathNet.Numerics.LinearAlgebra;
using MathNet.Numerics.LinearAlgebra.Double;
using MathNet.Numerics.LinearAlgebra.Factorization;

namespace Zcu.CurvatureBenchmark.DataSources.Implicit
{
    internal class QEF3D
	{
		public List<Point3D> Intersections { get; set; }
		public List<Point3D> Normals { get; set; }
		public double Error { get; set; }
        Point3D mass_point;
		private static Random rnd = new Random();
		private static Point3D[] deltas;

        private SMat3 ata;
        private Point3D atb, massPoint, x;


        static QEF3D()
		{
           
            int size = 8;
			deltas = new Point3D[size * size * size];
			for (int x = 0; x < size; x++)
			{
				double dx = (double)x / (double)(size / 2) - 1.0f;
				for (int y = 0; y < size; y++)
				{
                    double dy = (double)y / (double)(size / 2) - 1.0f;
					for (int z = 0; z < size; z++)
					{
                        double dz = (double)z / (double)(size / 2) - 1.0f;
						deltas[x * size * size + y * size + z] = new Point3D(dx * 0.01f, dy * 0.01f, dz * 0.01f);
					}
				}
			}           
        }

		public QEF3D()
		{
			Intersections = new List<Point3D>();
			Normals = new List<Point3D>();
            ata = new SMat3();
            atb = new Point3D();
            massPoint = new Point3D();
            x = new Point3D();
        }

		public void Add(Point3D p, Point3D n)
		{
			Intersections.Add(p);
			Normals.Add(n);
                       
            mass_point += p;
		}

		/*public float Solve(out Vector2 p, float svd_tol, int sweeps, float pinv_tol)
		{
			mass_point /= (float)Intersections.Count;
			// this->setAta();
			// this->setAtb();
			//Vec3 tmpv;
			Vector3 tmpv = Vector3.Transform(mass_point, ata);
			//MatUtils::vmul_symmetric(tmpv, this->ata, this->massPoint);
			//  VecUtils::sub(this->atb, this->atb, tmpv);
			atb -= tmpv;
			// this->x.clear();
			x = Vector3.Zero;
			p = new Vector2(mass_point.X, mass_point.Y) / (float)Intersections.Count;
			return 0;
			// const float result = Svd::solveSymmetric(this->ata, this->atb,
			//                       this->x, svd_tol, svd_sweeps, pinv_tol);
			//  VecUtils::addScaled(this->x, 1, this->massPoint);
			// this->setAtb();
			// outx.set(x);
			// this->hasSolution = true;
			// return result;
		}*/

		private double GetDistanceSquared(Point3D x)
		{
            double total = 0;
			for (int i = 0; i < Intersections.Count; i++)
			{
                Point3D d = x - Intersections[i];
                double dot = Normals[i].X * d.X + Normals[i].Y * d.Y + Normals[i].Z * d.Z;
				total += dot * dot;
			}
			return total;
		}

		

		/* Currently disabled; it just returns the mass point, which means sharp features are lost */
		public Point3D Solve2(double svd_tol, int sweeps, double pinv_tol)
		{
			if (Intersections.Count == 0)
			{
				this.Error = 100000;
				return new Point3D(0,0,0);
			}
            Point3D x = mass_point / (double)Intersections.Count;
			double error = GetDistanceSquared(x);
			this.Error = error;
			//return x;

			if (Math.Abs(error) >= 0.0001f)
			{
				for (int i = 0; i < deltas.Length; i++)
				{
                    Point3D new_point = new Point3D(x.X + deltas[i].X, x.Y + deltas[i].Y, x.Z + deltas[i].Z);
                    new_point= Clamp(new_point, new Point3D(0,0,0), new Point3D(1, 1, 1));
                    double e = GetDistanceSquared(new_point);
					if (e <= error)
					{
						x = new_point;
						if (Math.Abs(e) < Utils.EPSILON)
							break;
						error = e;
					}
				}
			}

            

			if (x.X > 1 || x.Y > 1 || x.Z > 1 || x.X < 0 || x.Y < 0 || x.Z < 0)
				return mass_point / (float)Intersections.Count;
			return Clamp(x, new Point3D(0, 0, 0), new Point3D(1, 1, 1));
        }

       
        
        public Point3D vmul_symmetric(Matrix<double> a, Point3D v)
        {
            Point3D k = new Point3D(
                (a[0,0] * v.X) + (a[0, 1] * v.Y) + (a[0, 2] * v.Z),
                (a[0, 1] * v.X) + (a[1, 1] * v.Y) + (a[1, 2] * v.Z),
                (a[0, 2] * v.X) + (a[1, 2] * v.Y) + (a[2, 2] * v.Z));
            return k;
        }



        public Point3D QEFfunkce(int triangles, Point3D[] normaly, Point3D[] body, Point3D massPoint) {
            Point3D poi = new Point3D();

            double[][] matrix = new double [12][];
            for (int i = 0; i < 12; i++) {
                matrix[i] = new double[3];
            }

            double[] vector = new double[12];
            int rows = 0;

            for (int i = 0; i < 12; i++) {
                //zastaveni
                Point3D nor = normaly[i];
                matrix[rows][0]= nor.X;
                matrix[rows][1] = nor.Y;
                matrix[rows][2] = nor.Z;

                Point3D p = body[i] - massPoint;
               // vector[rows] = nor * p;

                ++rows;


            }



            return poi;
        }


        public Point3D Clamp(Point3D start, Point3D a, Point3D b) {

            if (start.X < a.X) start.X = a.X;
            if (start.Y < a.Y) start.Y = a.Y;
            if (start.Z < a.Z) start.Z = a.Z;
            if (start.X > b.X) start.X = b.X;
            if (start.Y > b.Y) start.Y = b.Y;
            if (start.Z > b.Z) start.Z = b.Z;

            return start;
        }

    }
}
