﻿/*
 * This is free and unencumbered software released into the public domain.
 *
 * Anyone is free to copy, modify, publish, use, compile, sell, or
 * distribute this software, either in source code form or as a compiled
 * binary, for any purpose, commercial or non-commercial, and by any
 * means.
 *
 * In jurisdictions that recognize copyright laws, the author or authors
 * of this software dedicate any and all copyright interest in the
 * software to the public domain. We make this dedication for the benefit
 * of the public at large and to the detriment of our heirs and
 * successors. We intend this dedication to be an overt act of
 * relinquishment in perpetuity of all present and future rights to this
 * software under copyright law.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * For more information, please refer to <http://unlicense.org/>
 */

/*
 * https://github.com/tuckbone/DualContouringCSharp
 * 
 * Please do not use this code as the "Ideal way to code in C#"!!! 
 * It is by no means optimized for managed code. This is nothing but a direct port 
 * of NGildea's Dual Contouring Impl from his blog.
 * 
 */
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Zcu.CurvatureBenchmark.Framework;

namespace Zcu.CurvatureBenchmark.DataSources.Implicit
{
    internal class UDualContouring
    {
        double[,,] map;
        Point3D[,,] mappoints;
        int[,,] vertex_indexes;
        Random rnd = new Random();
        int xSize, ySize, zSize;
        RectangularBoxSampler gridsampler;
        public int VertexCount { get; protected set; }
        public int IndexCount { get; protected set; }
        public int OutlineLocation { get; protected set; }
        protected ArrayList Verticies = new ArrayList();
        protected ArrayList TriangleIndicies = new ArrayList();
        int[] inc = new int[8];
        int xInc, yInc, zInc;
        protected readonly double[] data = null;
        protected readonly Point3D[] points = null;
        Point3D[] cube = new Point3D[8];
        double threshold;

        public double QEF_ERROR = Utils.EPSILON ;//1e-6f;
        public  int QEF_SWEEPS = 4;

        int[,] edges ={
    //        { 2,4 }, { 1,5 }, { 2,6 }, { 3,7 },     //x
    //    { 0,2 }, { 1,3 }, { 4,6 }, {5,7 },     //y
    //    { 0,1}, { 2,3 }, { 4,5 }, {6,7 }      //z
    //};
        //    { 0, 1 }, { 1, 2 }, { 2, 3 }, { 3, 0 },     //x
        //    { 4, 5 }, { 5, 6 }, { 6, 7 }, { 7, 4 },     //y
        //    { 0, 4 }, { 1, 5 }, { 2, 6 }, { 3, 7 }      //z
        //};
        //      {
                  {0,4},{1,5},{2,6},{3,7},	// x-axis 
        	{0,2},{1,3},{4,6},{5,7},	// y-axis
        	{0,1},{2,3},{4,5},{6,7}		// z-axis
        };

        int[,] dirs = { { 1, 0, 0 }, { 0, 1, 0 }, { 0, 0, 1 } };
       

        public UDualContouring(RectangularBoxSampler gridsampler, int[] parameters, Point3D[] pointsP, double[] data) 
		{
            this.gridsampler = gridsampler;
            this.data = data;
            this.points = pointsP;
            xSize = parameters[1];
            ySize = parameters[2];
            zSize = parameters[3];
            map = new double[xSize, ySize, zSize];
            mappoints = new Point3D[xSize, ySize, zSize];
            vertex_indexes = new int[xSize, ySize, zSize];

            for (int x = 0; x < xSize - 1; x++)
            {
                for (int y = 0; y < ySize - 1; y++)
                {
                    for (int z = 0; z < zSize - 1; z++)
                    {
                        vertex_indexes[x, y, z] = -1;
                    }
                }
            }

            int index = 0;
            this.threshold = 0;

            xInc = 1;
            yInc = xSize;
            zInc = xSize * ySize;

            inc[0] = 0;
            inc[1] = yInc;
            inc[2] = yInc + zInc;
            inc[3] = zInc;
            inc[4] = xInc;
            inc[5] = xInc + yInc;
            inc[6] = xInc + yInc + zInc;
            inc[7] = xInc + zInc;

            double stepX = (gridsampler.MaxX - gridsampler.MinX) / (gridsampler.SamplesX - 1);
            double stepY = (gridsampler.MaxY - gridsampler.MinY) / (gridsampler.SamplesY - 1);
            double stepZ = (gridsampler.MaxZ - gridsampler.MinZ) / (gridsampler.SamplesZ - 1);

            cube[0] = new Point3D(0, 0, 0);
            cube[1] = new Point3D(0, stepY, 0);
            cube[2] = new Point3D(0, stepY, stepZ);
            cube[3] = new Point3D(0, 0, stepZ);
            cube[4] = new Point3D(stepX, 0, 0);
            cube[5] = new Point3D(stepX, stepY, 0);
            cube[6] = new Point3D(stepX, stepY, stepZ);
            cube[7] = new Point3D(stepX, 0, stepZ);


            for (int z = 0; z < zSize; z++)
            {
                for (int y = 0; y < ySize; y++)
                {
                    for (int x = 0; x < xSize; x++)
                    {
                        map[x, y, z] = data[index];
                        mappoints[x,y,z] = pointsP[index];
                        index++;
                    }
                }
            }
            Contour(0);
        }

        public Point3D[] GVerticies
        {
            get { return ((Point3D[])Verticies.ToArray(typeof(Point3D))); }
        }
        public int[] GTriangleIndicies
        {
            get { return (int[])TriangleIndicies.ToArray(typeof(int)); }
        }
                

        //scan of the whole grid
        public void Contour(double threshold)
        {
            

            VertexCount = 0;
            IndexCount = 0;
            OutlineLocation = 0;

            //searching of vertices
            for (int x = 0; x < xSize - 1; x++)
            {
                for (int y = 0; y < ySize - 1; y++)
                {
                    for (int z = 0; z < zSize - 1; z++)
                    {
                        GenerateAt(x, y, z);
                    }
                }
            }

            //creating of triangles
            for (int x = 0; x < xSize - 1; x++)
            {
                for (int y = 0; y < ySize - 1; y++)
                {
                    for (int z = 0; z < zSize - 1; z++)
                    {
                        GenerateIndexAt(x, y, z);
                    }
                }
            }


        }

        public void GenerateAt(int x, int y, int z)
        {
            int corners = 0;
            int xyzOffset = 0;
            double valueF = 0;
            //for (int i = 0; i < 8; i++)
            //{
            //    if (map[x + i / 4, y + i % 4 / 2, z + i % 2] < 0)
            //        corners |= 1 << i;
            //}

            xyzOffset = z * zInc + y * yInc + x * xInc;
            Point3D ptOrigin = points[xyzOffset];

            // get marching cubes case index
            
            for (int vertex = 0; vertex < 8; vertex++)
            {
                valueF = data[xyzOffset + inc[vertex]];
                if (valueF < 0 + Utils.EPSILON)
                {
                    corners |= (1 << vertex);
                    //pocet++;
                }
            }

            //table of vertices, the same like MC
            int triangles = MarchingCubesTable.NumberOfTriangles[corners];

            if (corners == 0 || corners == 255)
                return;

            int totalVerts = Verticies.Count;

            QEF3D qef = new QEF3D();  //calculate of quadratic error function
            QefSolver qef2 = new QefSolver();
            Point3D average_normal = new Point3D();
            for (int i = 0; i < 12; i++)
            {
                int c1 = edges[i, 0];
                int c2 = edges[i, 1];

                int m1 = (corners >> c1) & 1;
                int m2 = (corners >> c2) & 1;
                if (m1 == m2)
                    continue;

               
                double d1 = data[xyzOffset + inc[c1]];
                double d2 = data[xyzOffset + inc[c2]];
                                
                Point3D p1 = cube[c1];
                Point3D p2 = cube[c2];

                //Point3D intersection = GetIntersection(p1, p2, d1, d2);
                Point3D intersection = exactValue(c1, c2, d1, d2, 10, ptOrigin);
                //Point3D tmp = intersection + mapaBodu[x,y,z];// new Point3D(x, y, z);
                //gridsampler.Function.Evaluate(tmp.X, tmp.Y, tmp.Z);
                gridsampler.Function.Evaluate(intersection.X, intersection.Y, intersection.Z);
                Point3D normal = gridsampler.Function.Normal;
                average_normal += normal;

                qef.Add(intersection, normal);
                qef2.add(intersection.X, intersection.Y, intersection.Z, normal.X, normal.Y, normal.Z);
               // Point3D p = qef.Solve2(0, 16, 0);
               // Verticies.Add(intersection);
                //Verticies.Add(p);
            }
            Point3D v2;
           

            Point3D q = new Point3D();
            qef2.solve(out q, QEF_ERROR, QEF_SWEEPS, QEF_ERROR);
             v2 = q;

            Verticies.Add(v2);
            

            vertex_indexes[x, y, z] = VertexCount;
            VertexCount++;

        }

        /// <summary>
        /// Bisection method
        /// </summary>
        /// <param name="v1">index of v1</param>
        /// <param name="v2">index of v2</param>
        /// <param name="w1">function value of of point with index v1</param>
        /// <param name="w2">function value of of point with index v2</param>
        /// <param name="iteration">count of iteration of bisection method</param>
        /// <param name="orig">point coordinates</param>
        /// <returns></returns>
        public Point3D exactValue(int v1, int v2, double w1, double w2, int iteration, Point3D orig)
        {
            Point3D exactpoint = new Point3D(0, 0, 0);
            double w0 = 0;
            Point3D a = cube[v1] + orig;
            Point3D b = cube[v2] + orig;


            for (int i = 0; i < iteration; i++)
            {
                exactpoint = (a + b) / 2;
                gridsampler.Function.Evaluate(exactpoint.X, exactpoint.Y, exactpoint.Z);
                w0 = gridsampler.Function.F;

                if (w0 < threshold)
                {
                    if (w1 < w2)
                    {
                        a = exactpoint;
                        w1 = w0;
                    }
                    else
                    {
                        b = exactpoint;
                        w2 = w0;
                    }

                }
                else
                {
                    if (w1 > w2)
                    {
                        a = exactpoint;
                        w1 = w0;
                    }
                    else
                    {
                        b = exactpoint;
                        w2 = w0;
                    }
                }
            }

            exactpoint = (a + b) / 2;
            return exactpoint;
        }

        public void GenerateIndexAt(int x, int y, int z)
        {
            //int corners = 0;
           
            int v1 = vertex_indexes[x, y, z];
            //if (v1 == 0)
            //    return;
            //if (v1 >= Verticies.Count)
            //{
            //    return;
            //}
            if (v1 == -1)
                return;

            int[] indices = new int[256];

            int index = 0;

            for (int i = 0; i < 3; i++)
            {
                for (int j = 0; j < i; j++)
                {
                    int v2 = vertex_indexes[x + dirs[i, 0], y + dirs[i, 1], z + dirs[i, 2]];
                    int v3 = vertex_indexes[x + dirs[j, 0], y + dirs[j, 1], z + dirs[j, 2]];
                    int v4 = vertex_indexes[x + dirs[i, 0] + dirs[j, 0], y + dirs[i, 1] + dirs[j, 1], z + dirs[i, 2] + dirs[j, 2]];
                    if (v2 == -1 || v3 == -1 || v4 == -1)
                        continue;


                    indices[index++] = v1;
                    indices[index++] = v2;
                    indices[index++] = v3;

                    indices[index++] = v4;
                    indices[index++] = v3;
                    indices[index++] = v2;
                    
                   
                        TriangleIndicies.Add(v1);
                        TriangleIndicies.Add(v2);
                        TriangleIndicies.Add(v3);

                        TriangleIndicies.Add(v4);
                        TriangleIndicies.Add(v3);
                        TriangleIndicies.Add(v2);
                  
                }
            }


                IndexCount += index;
        }

       
    }
}
