﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;
using Zcu.CurvatureBenchmark.Framework;

namespace Zcu.CurvatureBenchmark.DataSources.Implicit
{
    internal class MT6
    {
        public double[] data = null;
        protected readonly Point3D[] pointsO = null;
        public int xSize, ySize, zSize;
        public double treshold;
        public int xInc, yInc, zInc;
        public Point3D[] cube = new Point3D[8];
        public int[] inc = new int[8];
        protected ArrayList vVerticies = new ArrayList();
        protected ArrayList vTriangleIndicies = new ArrayList();
        public ArrayList edgesL = new ArrayList();
        static protected readonly double epsilon = 0.00001;     

        private int hashIndex = 0;
        public Dictionary<Point3D, int> points;
        private int hashPoint = 0;     
        private Line edge;
        private Line Redge;
        private Point3D newVertex = new Point3D();
        private Dictionary<Line, int> dict;
        RectangularBoxSampler gridsampler;
        int iteration;
        bool exact;

        public Point3D[] GVerticies
        {
            get { return ((Point3D[])vVerticies.ToArray(typeof(Point3D))); }
        }
        public Triangle[] GTriangleIndicies
        {
            get { return (Triangle[])vTriangleIndicies.ToArray(typeof(Triangle)); }
        }
       

        public MT6(Point3D[] pointsO, double[] data, double threshold, int[] parameters, RectangularBoxSampler gridsampler, bool exact, int iteration) {
               this.data   = data;
            this.pointsO = pointsO;
            this.gridsampler = gridsampler;
            this.exact = exact;
            this.iteration = iteration;

              xSize = parameters[1];
               ySize = parameters[2];
               zSize = parameters[2];
   
               this.treshold = threshold;
   
               xInc = 1;
               yInc = xSize;
               zInc = xSize * ySize;
                           
               inc[0] = 0;
               inc[1] = yInc;
               inc[2] = yInc + zInc;
               inc[3] = zInc;
               inc[4] = xInc;
               inc[5] = xInc + yInc;
               inc[6] = xInc + yInc + zInc;
               inc[7] = xInc + zInc;
            
            double stepX = (gridsampler.MaxX - gridsampler.MinX) / (gridsampler.SamplesX - 1);
            double stepY = (gridsampler.MaxY - gridsampler.MinY) / (gridsampler.SamplesY - 1);
            double stepZ = (gridsampler.MaxZ - gridsampler.MinZ) / (gridsampler.SamplesZ - 1);

            //x,y,z
            cube[0] = new Point3D(0, 0, 0);
            cube[1] = new Point3D(0, stepY, 0);
            cube[2] = new Point3D(0, stepY, stepZ);
            cube[3] = new Point3D(0, 0, stepZ);
            cube[4] = new Point3D(stepX, 0, 0);
            cube[5] = new Point3D(stepX, stepY, 0);
            cube[6] = new Point3D(stepX, stepY, stepZ);
            cube[7] = new Point3D(stepX, 0, stepZ);
                        
            
              points = new Dictionary<Point3D, int>();
            dict = new Dictionary<Line, int>();
            TraceContours();
           }


          public void TraceContours()
          {
            int xyzOffset;


            //Go through grid
            for (int z = 0; z < zSize - 1; z++)
              {
                  for (int y = 0; y < ySize - 1; y++)
                  {
                      for (int x = 0; x < xSize - 1; x++)
                      {
                         xyzOffset = z * zInc + y * yInc + x * xInc;
                        //Point3D ptOrig = new Point3D(x, y, z);
                        Point3D ptOrig = pointsO[xyzOffset];
                        
                        Point3D[] cubeB = new Point3D[8];
                        int[] cubeInd = new int[8];
                       
                        for (int i =0;i<8;i++){
                          cubeB[i] = ptOrig + cube[i];
                            cubeInd[i] = xyzOffset + inc[i];                        
                        }
                        
                        GenerateTriangles(cubeB, cubeInd);

                      }
                  }
              }
          
          
          }


          private void GenerateTriangles(Point3D[] cube, int[] cubeInd)
          {           
            CalculateTetraIndex(cube[0], cube[2], cube[3], cube[7], cubeInd[0], cubeInd[2], cubeInd[3], cubeInd[7]);
            CalculateTetraIndex(cube[0], cube[2], cube[6], cube[7], cubeInd[0], cubeInd[2], cubeInd[6], cubeInd[7]);
            CalculateTetraIndex(cube[0], cube[4], cube[6], cube[7], cubeInd[0], cubeInd[4], cubeInd[6], cubeInd[7]);
            CalculateTetraIndex(cube[0], cube[6], cube[1], cube[2], cubeInd[0], cubeInd[6], cubeInd[1], cubeInd[2]);
            CalculateTetraIndex(cube[0], cube[6], cube[1], cube[4], cubeInd[0], cubeInd[6], cubeInd[1], cubeInd[4]);
            CalculateTetraIndex(cube[5], cube[6], cube[1], cube[4], cubeInd[5], cubeInd[6], cubeInd[1], cubeInd[4]);//*/
        } 

        protected virtual void CalculateTetraIndex(Point3D p0, Point3D p1, Point3D p2, Point3D p3,int p0i, int p1i, int p2i, int p3i)
        {
            int tetraIndex = 0;

            // create index which vertices of the cube are under iso-surface
            if (data[p0i] < treshold + epsilon) tetraIndex |= 1;
            if (data[p1i] < treshold + epsilon) tetraIndex |= 2;
            if (data[p2i] < treshold + epsilon) tetraIndex |= 4;
            if (data[p3i] < treshold + epsilon) tetraIndex |= 8;

            // stop if there is no intersection 
            if ((tetraIndex == 0x00) || (tetraIndex == 0x0F))
                return;
                        

            GenerateTrianglesForTetrahedron(tetraIndex, p0, p1, p2, p3, p0i, p1i, p2i, p3i);
        } 

        private void GenerateTrianglesForTetrahedron(int tetraIndex, Point3D p0, Point3D p1, Point3D p2, Point3D p3, int p0i, int p1i, int p2i, int p3i)
        {
            Triangle triangle;

            //cases of possible triangles
            switch (tetraIndex)
            {
                case 0x00:
                case 0x0F:
                    break;
                case 0x0E:
                case 0x01:
                    //break;
                    triangle = new Triangle(GetPointIndex(p0, p1, p0i, p1i), GetPointIndex(p0, p2, p0i, p2i), GetPointIndex(p0, p3,p0i, p3i));
                    vTriangleIndicies.Add(triangle);
                    break;
                case 0x0D:
                case 0x02:
                    
                    triangle = new Triangle(GetPointIndex(p1, p0,p1i, p0i), GetPointIndex(p1, p3, p1i, p3i), GetPointIndex(p1, p2, p1i, p2i));
                    vTriangleIndicies.Add(triangle);
                    break;
                case 0x0C:
                case 0x03:
                    //break;
                    triangle = new Triangle(GetPointIndex(p0, p3, p0i, p3i), GetPointIndex(p0, p2, p0i, p2i), GetPointIndex(p1, p3, p1i, p3i));
                    vTriangleIndicies.Add(triangle);

                    triangle = new Triangle(triangle.V3, GetPointIndex(p1, p2, p1i, p2i), triangle.V2);
                    vTriangleIndicies.Add(triangle);
                    break;
                case 0x0B:
                case 0x04:
                   // break;
                    triangle = new Triangle(GetPointIndex(p2, p0, p2i, p0i), GetPointIndex(p2, p1, p2i, p1i), GetPointIndex(p2, p3, p2i, p3i));
                    vTriangleIndicies.Add(triangle);
                    break;
                case 0x0A:
                case 0x05:
                    //break;
                    triangle = new Triangle(GetPointIndex(p0, p1, p0i, p1i), GetPointIndex(p2, p3, p2i, p3i), GetPointIndex(p0, p3, p0i, p3i));
                    vTriangleIndicies.Add(triangle);

                    triangle = new Triangle(triangle.V1, GetPointIndex(p1, p2, p1i, p2i), triangle.V2);
                    vTriangleIndicies.Add(triangle);
                    break;
                case 0x09:
                case 0x06:
                   // break;
                    triangle = new Triangle(GetPointIndex(p0, p1, p0i, p1i), GetPointIndex(p1, p3, p1i, p3i), GetPointIndex(p2, p3, p2i, p3i));
                    vTriangleIndicies.Add(triangle);

                    triangle = new Triangle(triangle.V1, GetPointIndex(p0, p2, p0i, p2i), triangle.V3);
                    vTriangleIndicies.Add(triangle);
                    break;
                case 0x07:
                case 0x08:
                    //break;
                    triangle = new Triangle(GetPointIndex(p3, p0, p3i, p0i), GetPointIndex(p3, p2, p3i, p2i), GetPointIndex(p3, p1, p3i, p1i));
                    vTriangleIndicies.Add(triangle);
                    break;
            }
        } 


        private int GetPointIndex(Point3D a, Point3D b, int ai, int bi)
        {
            int pocet=0;
            int value;
            // construct edge
            if (data[ai] < data[bi])
            {
                edge.V1 = ai;
                edge.V2 = bi;
            }
            else
            {
                edge.V1 = bi;
                edge.V2 = ai;
            }
            Redge = Reverse(edge);
            // check if on this edge was already computed vertex (on each edge can be max. 1 vertex)
            if (dict.TryGetValue(edge, out value))
                return value;
            else if (dict.TryGetValue(Redge, out value)) {
                pocet++;
                return value;
            }
            else
            {
                if (exact)
                {
                    newVertex = ExactValue(a, b, ai, bi);
                }
                else
                {
                    newVertex = interpolation(a, b, treshold, ai, bi);
                }
                dict.Add(edge, hashIndex);
                vVerticies.Add(newVertex);
                hashIndex++;
                return hashIndex - 1;
            }
        }


                 


        
          private Point3D interpolation(Point3D a, Point3D b, double isoValue, int aw, int bw)
          {
              Point3D ab = new Point3D(0,0,0);

            double w1 = data[aw]; // data[(int)(a.Z * zInc + a.Y * yInc + a.X * xInc)];
            double w2 = data[bw];// data[(int)(b.Z * zInc + b.Y * yInc + b.X * xInc)];


                  double val = (double)((isoValue - w1) / (w2 - w1));

            //pt = _cube[v1] * ( 1 - mu ) + _cube[v2] * mu;
            // pt = _cube[v1] + ((float)mu * (_cube[v2] - _cube[v1]));

                  ab.X = (double)(a.X + val * (b.X - a.X)) ;
                  ab.Y = (double)(a.Y + val * (b.Y - a.Y)) ;
                  ab.Z = (double)(a.Z + val * (b.Z - a.Z)) ;
                  //ab.index = (int)(w1+val *(w2-w1));
                  //a.normal.X = a.normal.X + val * (b.normal.X - a.normal.X);
                  //a.normal.Y = a.normal.Y + val * (b.normal.Y - a.normal.Y);
                  //a.normal.Z = a.normal.Z + val * (b.normal.Z - a.normal.Z);
             

              return ab;
          }


        //Bisection method
        public Point3D ExactValue(Point3D a, Point3D b, int v1, int v2)
        {
            Point3D exact = new Point3D(0, 0, 0);
            double w1 = data[v1]; // data[(int)(a.Z * zInc + a.Y * yInc + a.X * xInc)];
            double w2 = data[v2];// data[(int)(b.Z * zInc + b.Y * yInc + b.X * xInc)];
            double w0 = 0;
            //Point3D a = cube[v1] + orig;
            //Point3D b = cube[v2] + orig;


            for (int i = 0; i < iteration; i++)
            {
                exact = (a + b) / 2;
                gridsampler.Function.Evaluate(exact.X, exact.Y, exact.Z);
                w0 = gridsampler.Function.F;

                if (w0 < treshold)
                {
                    if (w1 < w2)
                    {
                        a = exact;
                        w1 = w0;
                    }
                    else
                    {
                        b = exact;
                        w2 = w0;
                    }

                }
                else
                {
                    if (w1 > w2)
                    {
                        a = exact;
                        w1 = w0;
                    }
                    else
                    {
                        b = exact;
                        w2 = w0;
                    }
                }
            }

            exact = (a + b) / 2;
            return exact;
        }


        public Line Reverse(Line l)
        {
            Line k = new Line(l.V2, l.V1);

            return k;
        }


        //pomocna struktura hrany
        public struct Line
        {
           
            public int V1;
            
            public int V2;
                        
            public Line(int V1, int V2)
            {
                this.V1 = V1;
                this.V2 = V2;
            } 
            
        }



    }
}
