﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Zcu.CurvatureBenchmark.Framework;
using Zcu.CurvatureBenchmark.Framework.DataSources.Implicit;

namespace Zcu.CurvatureBenchmark.DataSources.Implicit
{
    
    /// <summary>
    /// Rectangular grid triangularizator takes samples from RectangularGridSampler and creates regular triangulazation.
    /// </summary>
    /// <author>Petr Martinek</author>      
    public class RGTriangularizator_ADC : AbstractModule, IImplicitTriangularizator
    {
        /// <summary>
        /// Try to find exact position of iso-surface. (default false)
        /// </summary>
        public virtual bool Exact { get; set; } = false;

        /// <summary>
        /// Maximal number of iterations for exact surface computation. (default 10)
        /// </summary>
        public virtual int Iteration { get; set; } = 10;

        RectangularBoxSampler gridsampler;

        /// <inheritdoc/>
        public void Triangulate(IImplicitSampler sampler)
        {
            Progress = 0;
            if (!(sampler is RectangularBoxSampler)) throw new ApplicationException("RectangularBoxTriangularizator needs RectangularBoxSampler");
            gridsampler = sampler as RectangularBoxSampler;


            TriangleMesh = new TriangleMesh();
            //TriangleMesh.Points = sampler.Points;

            int[] parameters = new int[4] { 0, gridsampler.SamplesX, gridsampler.SamplesY, gridsampler.SamplesZ };


            // DualMarchingCubes dmc = new DualMarchingCubes(gridsampler.SamplesX, gridsampler.SamplesY, gridsampler.SamplesZ, 0);
            //dmc.Contour(0);
            // Point3D[] rvVerticies = dmc.GVerticies;
            //int[] rvIndices = dmc.GTriangleIndicies;
            //ADualContouring dc = new ADualContouring(gridsampler, 32, 14);//32,14
            //dc.Contour(0);
            const int MAX_THRESHOLDS = 5;
            double[] THRESHOLDS = new double[MAX_THRESHOLDS] { -1.0, 0.1, 1.0, 10.0, 50.0 };
            int thresholdIndex = -1;

            OctreeNode root = null;
            Octree oct = new Octree();

            // octreeSize must be a power of two!
           
            int samply = gridsampler.SamplesX;           
           

            for (int x = samply; x > 1; x--)
            {

                if ((x != 0) && ((x & (x - 1)) == 0)) {//check if nomber is power of two
                    samply = x;
                    break;
                }

            }

            if (samply < 16) {
                samply = 16;
                gridsampler.SamplesX = samply;
                gridsampler.SamplesY = samply;
                gridsampler.SamplesZ = samply;
                Console.WriteLine();
                Console.WriteLine("Minimal step is 64, your choice was changet to minimal value.");
            }
            //double octreeSize = (double)samply/2;
           
            double octreeSize = Math.Abs(gridsampler.MinX)-1;//1;// Math.Abs(gridsampler.MinX);
            

            root = oct.BuildOctree(new Point3D(-octreeSize , -octreeSize , -octreeSize ), samply, 0, gridsampler, Exact, Iteration);
            List<Point3D> Verticies = new List<Point3D>();
            List<int> Indices = new List<int>();

            oct.GenerateMeshFromOctree(root,out Verticies,out Indices);
            Point3D[] rvVerticies = Verticies.ToArray();
            int[] rvIndices= Indices.ToArray();


            Triangle[] triang;
            ArrayList triangArray = new ArrayList();
            var duplic = dupl(rvVerticies, rvIndices);
            rvVerticies = duplic.Item1;
            rvIndices = duplic.Item2;

            int k = 0;
            int l = 0;
            Triangle tmp;
            for (int i = 0; i < rvIndices.Length; i = i + 3)
            {
                tmp.V1 = (int)rvIndices[i];
                tmp.V2 = (int)rvIndices[i + 1];
                tmp.V3 = (int)rvIndices[i + 2];

                if (!degenerateTriangle(tmp))
                {
                    triangArray.Add(tmp);
                    k++;
                }
                else
                {
                    l++;
                }
            }

            triang = (Triangle[])triangArray.ToArray(typeof(Triangle));
            double[] k1s = new double[rvVerticies.Length];
            double[] k2s = new double[rvVerticies.Length];
            Point3D[] normalS = new Point3D[rvVerticies.Length];

            for (int i = 0; i < triang.Length; i++)
            {
                triang[i] = checkOrientation(triang[i], rvVerticies, sampler);

            }

            for (int i = 0; i < rvVerticies.Length; i++)
            {
                sampler.Function.Evaluate(rvVerticies[i].X, rvVerticies[i].Y, rvVerticies[i].Z);
                k1s[i] = sampler.Function.K1;
                k2s[i] = sampler.Function.K2;
                normalS[i] = sampler.Function.Normal;
            }


            TriangleMesh.Points = rvVerticies;
            TriangleMesh.Triangles = triang;
            TriangleMesh.ExactK1 = k1s;
            TriangleMesh.ExactK2 = k2s;
            TriangleMesh.Normals = normalS;

            //ObjSaver saver = new ObjSaver();

            //saver.FileName = "mesh1.obj";

            //saver.Execute(TriangleMesh);

            Progress = 100;
        }

        
        /// <summary>
        /// Projedu vsuchny vrcholy a vkladam je do hash tabulky
        /// Pokud v tabulce vrchol neni, vlozim ho tam,
        /// dam mu novy index a vrchol vlozim i do seznamu. 
        /// </summary>
        /// <param name="body">Vrcholy meshe</param>
        /// <param name="indexy">Indexy trojuhelniku</param>
        /// <returns></returns>
        public Tuple<Point3D[], int[]> dupl(Point3D[] body, int[] indexy)
        {
            ArrayList cisteIndexy = new ArrayList();
            ArrayList cisteBody = new ArrayList();

            Hashtable table = new Hashtable();
            Point3D item;
            int k = 0;
            for (int i = 0; i < body.Length; i++)
            {
                item = body[i];
                if (!table.Contains(item))
                {
                    table.Add(item, k);
                    cisteBody.Add(item);
                    cisteIndexy.Add(k);

                    k++;
                }
                else
                {
                    cisteIndexy.Add((int)table[item]);

                }

            }

            int[] noveIndexy = new int[indexy.Length];
            for (int i = 0; i < indexy.Length; i++)
                noveIndexy[i] = (int)cisteIndexy[indexy[i]];

            Point3D[] noveBody = ((Point3D[])cisteBody.ToArray(typeof(Point3D)));

            return Tuple.Create(noveBody, noveIndexy);
        }

        bool degenerateTriangle(Triangle triang)
        {
            if (triang.V1 == triang.V2) {
                return true;
            } else if(triang.V1 == triang.V3)
            {
                return true;
            }
            else if (triang.V2 == triang.V3)
            {
                return true;
            }
            else
            {
                return false;
            }
            
        }

        Triangle checkOrientation(Triangle t, Point3D[] vrcholy, IImplicitSampler sampler)
        {
            Triangle g = t;
            Point3D a = vrcholy[t.V1];
            Point3D b = vrcholy[t.V2];
            Point3D c = vrcholy[t.V3];

            // sampler.Function.Evaluate((a.X+b.X+c.X)/3, (a.Y + b.Y + c.Y) / 3, (a.Z + b.Z + c.Z) / 3);
            sampler.Function.Evaluate(a.X, a.Y, a.Z);
            Point3D presnaNorm = sampler.Function.Normal;

            //Point3D normala1 = Point3D.CrossProduct((b - a), (c - a));
            //Point3D normala2 = Point3D.CrossProduct((c - b), (a - b));
            //Point3D normala3 = Point3D.CrossProduct((a - c), (b - c));

            // Point3D normalatroj = Point3D.CrossProduct((c - a), (b - a));
            Point3D normalatroj = NewellNormal(b, a, c);
            double test = Point3D.DotProduct(presnaNorm, normalatroj);
            if (test < 0)
            {
                t.V2 = g.V3;
                t.V3 = g.V2;
            }


            return t;
        }

        private Point3D NewellNormal(Point3D p1, Point3D p2, Point3D p3)
        {
            Point3D normal = new Point3D();
            normal.X = (p1.Y - p2.Y) * (p1.Z + p2.Z) + (p2.Y - p3.Y) * (p2.Z + p3.Z) + (p3.Y - p1.Y) * (p3.Z + p1.Z);
            normal.Y = (p1.Z - p2.Z) * (p1.X + p2.X) + (p2.Z - p3.Z) * (p2.X + p3.X) + (p3.Z - p1.Z) * (p3.X + p1.X);
            normal.Z = (p1.X - p2.X) * (p1.Y + p2.Y) + (p2.X - p3.X) * (p2.Y + p3.Y) + (p3.X - p1.X) * (p3.Y + p1.Y);
            return normal;
        }


        /// <inheritdoc/>
        public TriangleMesh TriangleMesh {get ;set;}

       


    }
}
