﻿using System;
using System.Xml;
using Zcu.CurvatureBenchmark.Framework;
using Zcu.CurvatureBenchmark.Framework.DataSources;
using Zcu.CurvatureBenchmark.Framework.DataSources.Implicit;

namespace Zcu.CurvatureBenchmark.DataSources.Implicit
{
    /// <summary>
    /// Implementation of AbstractDataSource used for implicit functions. The implicit data source is composed 
    /// from implicit function, sampler and triangularizator.
    /// </summary>
    public class ImplicitDataSource : AbstractDataSource
    {
        IImplicitFunction function;
        IImplicitSampler sampler;
        IImplicitTriangularizator triangularizator;

        /// <inheritdoc/>
        public override void SetConfig(System.Xml.XmlNode configNode)
        {
            DatasetCount = 1;
            base.SetConfig(configNode);

            foreach (XmlNode node in configNode.ChildNodes)
            {
                switch (node.Name)
                {
                    case "Function":
                        function = TestScenario.GetModule<IImplicitFunction>(node.ChildNodes[0]);
                        break;
                    case "Sampler":
                        sampler = TestScenario.GetModule<IImplicitSampler>(node.ChildNodes[0]);
                        sampler.ProgressChanged += sampler_ProgressChanged;
                        break;
                    case "Triangularizator":
                        triangularizator = TestScenario.GetModule<IImplicitTriangularizator>(node.ChildNodes[0]);
                        triangularizator.ProgressChanged += triangularizator_ProgressChanged;
                        break;
                    default: break;
                }
            }
        }

        void triangularizator_ProgressChanged(IModule module, NotifyProgressChangedEventArgs args)
        {
            Progress = (byte)(100 * currentSet / DatasetCount + (50 + args.Progress / 2) / DatasetCount);
        }

        void sampler_ProgressChanged(IModule module, NotifyProgressChangedEventArgs args)
        {
            Progress = (byte)(100 * currentSet / DatasetCount + (0 + args.Progress / 2) / DatasetCount);
        }

        /// <inheritdoc/>
        public override TriangleMesh[] GenerateData()
        {
            TriangleMesh[] result = new TriangleMesh[DatasetCount];
            Progress = 0;
            for (currentSet = 0; currentSet < DatasetCount; currentSet++)
            {
                // Terrible terrible hack. I hope the appropriate diety will forgive me.
                bool manifold = false;
                while (!manifold)
                {
                    manifold = true; // we hope...
                    function.SetParams();
                    sampler.Sample(function);
                    triangularizator.Triangulate(sampler);
                    try
                    {
                        CornerTable ct = new CornerTable(triangularizator.TriangleMesh);
                    }
                    catch (Exception)
                    {
                        manifold = false;
                    }
                    if (triangularizator.TriangleMesh.Triangles.Length < 1000)
                        manifold = false;
                }
                result[currentSet] = triangularizator.TriangleMesh;
            }
            Progress = 100;
            return result;
        }

        int currentSet;

        /// <inheritdoc/>
        /// <preliminary/>
        public override string ToString()
        {
            return this.function.ToString();
        }
    }
}
