﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml;
using Zcu.CurvatureBenchmark.Framework;
using Zcu.CurvatureBenchmark.Framework.Distorters;

namespace Zcu.CurvatureBenchmark.Distorters
{
    /// <summary>
    /// A special type of distorter that can compose several distorters to a single one. The distorters are applied in 
    /// the same order as in the config file.
    /// </summary>
    public class CompositeDistorter : AbstractDistorter
    {
        List<IDistorter> distorters = new List<IDistorter>();

        /// <inheritdoc/>
        public override TriangleMesh Distort(TriangleMesh input)
        {
            TriangleMesh result = input;
            foreach (var distorter in distorters)
            {
                if (!distorter.Active) continue;
                result = distorter.Distort(result);
            }
            return result;
        }

        /// <inheritdoc/>
        public override void SetConfig(XmlNode configNode)
        {
            var properties =
            this.GetType().GetProperties().Where(p => p.CanWrite).ToDictionary(p => p.Name, p => p);

            foreach (XmlAttribute atr in configNode.Attributes)
            {
                this.SetConfig(properties, atr.Name, atr.InnerText);
            }

            foreach (XmlNode node in configNode.ChildNodes)
                distorters.Add(TestScenario.GetModule<IDistorter>(node));
        }

        /// <inheritdoc/>
        public override string ToString()
        {
            string result = "Composite ";
            foreach (IDistorter d in distorters)
                result += d.ToString() + " ";
            return result;
        }
    }
}
