﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Zcu.CurvatureBenchmark.Framework;
using Zcu.CurvatureBenchmark.Framework.Distorters;


namespace Zcu.CurvatureBenchmark.Distorters
{
    /// <summary>
    /// Module shifts vertices guided by Gaussian distribution with standard deviation Sigma * AverageEdgeLength.
    /// </summary>
    public class GaussianNoise : AbstractDistorter
    {
        /// <summary>
        /// Gets/sets standard deviation.
        /// </summary>
        public double Sigma { get; set; }
        
        /// <summary>
        /// Initialization.
        /// </summary>
        public GaussianNoise()
        {
            ToStringFormat = "$Name$ ($Sigma$)";
        }

        /// <inheritdoc/>
        public override TriangleMesh Distort(TriangleMesh input)
        {
            double avgEL = input.AverageEL();
            var newPoints = new Point3D[input.Points.Length];
            for (int i = 0;i<input.Points.Length;i++)
            {
                newPoints[i] = input.Points[i] + randomGaussPoint(avgEL);
            }
            var result = input.ShallowCopy();
            result.Points = newPoints;
            return result;
        }

        private Point3D randomGaussPoint(double avgEL)
        {
            return new Point3D(gauss(0, Sigma*avgEL), gauss(0, Sigma * avgEL), gauss(0, Sigma * avgEL));
        }

        // method generates one random number with gaussian distribution, mean e and deviation sigma
        double gauss(double e, double sigma)
        {
            int i;
            double r;
            r = 0;
            for (i = 0; i < 120; i++)
                r += Rand.NextDouble();
            r = ((r - 60.0) / 3.333333) * sigma + e;
            return (r);
        }
    }
}
