﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Zcu.CurvatureBenchmark.Framework;

namespace Zcu.CurvatureBenchmark.Distorters
{
    public class NormalsGouraudMN : AbstractDistorter
    {
        public override TriangleMesh Distort(TriangleMesh input)
        {
            CornerTable ct = new CornerTable(input);
            Point3D[] newNormals = new Point3D[input.Points.Length];
            Point3D Normal = new Point3D();
            foreach (Triangle t in input.Triangles)
            {

                Point3D v1 = input.Points[t.V1];
                Point3D v2 = input.Points[t.V2];
                Point3D v3 = input.Points[t.V3];

                //Newell
                Normal.X = (v1.Y + v2.Y) * (v1.Z - v2.Z) + (v2.Y + v3.Y) * (v2.Z - v3.Z) + (v3.Y + v1.Y) * (v3.Z - v1.Z);
                Normal.Y = (v1.Z + v2.Z) * (v1.X - v2.X) + (v2.Z + v3.Z) * (v2.X - v3.X) + (v3.Z + v1.Z) * (v3.X - v1.X);
                Normal.Z = (v1.X + v2.X) * (v1.Y - v2.Y) + (v2.X + v3.X) * (v2.Y - v3.Y) + (v3.X + v1.X) * (v3.Y - v1.Y);

                newNormals[t.V1] += Normal;
                newNormals[t.V2] += Normal;
                newNormals[t.V3] += Normal;
            }


            for (int i = 0; i < newNormals.Length; i++)
            {
                double n = ct.VF(i).Length; 
                newNormals[i] = (newNormals[i]/n).Normalize();
            }
            TriangleMesh result = input.ShallowCopy();
            result.Normals = newNormals;
            return result;
        }
    }
}
