﻿using System;
using Zcu.CurvatureBenchmark.Framework;
using Zcu.CurvatureBenchmark.Framework.Distorters;


namespace Zcu.CurvatureBenchmark.Distorters
{
    /// <summary>
    /// Module computes new normals based on Laplacian <cite>des99</cite>.
    /// </summary>
    public class NormalsLaplace : AbstractDistorter
    {
        /// <inheritdoc/>
        public override TriangleMesh Distort(TriangleMesh input)
        {
            CornerTable ct = new CornerTable(input);
            Point3D[] newNormals = new Point3D[input.Points.Length];

        
                 for (int i = 0; i < input.Points.Length; i++)
                {
                     int [] V = ct.VV(i); 


                     double alfa = 0;
                     double beta = 0;                  

                     Point3D u= new Point3D();
                     Point3D v = new Point3D();

                     Point3D z = new Point3D();
                     Point3D w = new Point3D();

                     int k = V.Length;
                                     
                         for (int j = 1; j <= V.Length; j++)
                         {
                             u = input.Points[V[(j+1)%k]] - input.Points[V[j%k]];
                             v = input.Points[V[(j + 1)%k]] - input.Points[i];                            
                             u = u.Normalize();
                             v = v.Normalize();
                   
                             alfa= (Math.Acos(u.DotProduct(v)));

                              z = input.Points[V[j - 1]] - input.Points[V[j%k]];
                              w = input.Points[V[j - 1]] - input.Points[i];
                              z = z.Normalize();
                              w = w.Normalize();
                    
                             beta = (Math.Acos(z.DotProduct(w)));

                             //(cot(alfa)+cot(beta)*(xj-xi);
                             newNormals[i] += ((((Math.Cos(alfa) / Math.Sin(alfa)) + (Math.Cos(beta) / Math.Sin(beta))) * (input.Points[V[j % k]] - input.Points[i])));
                }
                newNormals[i].Normalize();
                if (newNormals[i].DotProduct(input.Normals[i]) < 0)
                    newNormals[i] = -newNormals[i];

                }                          
            
            TriangleMesh result = input.ShallowCopy();
            result.Normals = newNormals;
            return result;
        }        
    }
}
