﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Zcu.CurvatureBenchmark.Framework;
using Zcu.CurvatureBenchmark.Framework.Distorters;


namespace Zcu.CurvatureBenchmark.Distorters
{
    /// <summary>
    /// Module computes new normals using weighted normals <cite>max99</cite>.
    /// </summary>
    public class NormalsMax : AbstractDistorter
    {
        /// <inheritdoc/>
        public override TriangleMesh Distort(TriangleMesh input)
        {
            Point3D[] newNormals = new Point3D[input.Points.Length];
            foreach (Triangle t in input.Triangles)
            {
                Point3D v1 = Point3D.Subtract(input.Points[t.V2], input.Points[t.V1]);
                Point3D v2 = Point3D.Subtract(input.Points[t.V3], input.Points[t.V1]);
                Point3D v3 = Point3D.Subtract(input.Points[t.V3], input.Points[t.V2]);
                Point3D cp = v2.CrossProduct(v1);

                newNormals[t.V1] += cp * (1 / (v1.LengthSquared() * v2.LengthSquared()));
                newNormals[t.V2] += cp * (1 / (v1.LengthSquared() * v3.LengthSquared()));
                newNormals[t.V3] += cp * (1 / (v2.LengthSquared() * v3.LengthSquared()));
            }
            for (int i = 0; i < newNormals.Length; i++)
                newNormals[i] = newNormals[i].Normalize();
            TriangleMesh result = input.ShallowCopy();
            result.Normals = newNormals;
            return result;
        }        
    }
}
