﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Zcu.CurvatureBenchmark.Framework;

namespace Zcu.CurvatureBenchmark.Distorters
{
    public class NormalsMaxMN:AbstractDistorter
    {
        public override TriangleMesh Distort(TriangleMesh input)
        {
            Point3D[] newNormals = new Point3D[input.Points.Length];
            Point3D Normal = new Point3D();

            foreach (Triangle t in input.Triangles)
            {
                Point3D v1 = input.Points[t.V1];
                Point3D v2 = input.Points[t.V2];
                Point3D v3 = input.Points[t.V3];

                //Newell
                Normal.X = (v1.Y + v2.Y) * (v1.Z - v2.Z) + (v2.Y + v3.Y) * (v2.Z - v3.Z) + (v3.Y + v1.Y) * (v3.Z - v1.Z);
                Normal.Y = (v1.Z + v2.Z) * (v1.X - v2.X) + (v2.Z + v3.Z) * (v2.X - v3.X) + (v3.Z + v1.Z) * (v3.X - v1.X);
                Normal.Z = (v1.X + v2.X) * (v1.Y - v2.Y) + (v2.X + v3.X) * (v2.Y - v3.Y) + (v3.X + v1.X) * (v3.Y - v1.Y);

                Point3D c1 = Point3D.Subtract(input.Points[t.V2], input.Points[t.V1]);
                Point3D c2 = Point3D.Subtract(input.Points[t.V3], input.Points[t.V1]);
                Point3D c3 = Point3D.Subtract(input.Points[t.V3], input.Points[t.V2]);

                newNormals[t.V1] += Normal * (1 / (c1.LengthSquared() * c2.LengthSquared()));
                newNormals[t.V2] += Normal * (1 / (c1.LengthSquared() * c3.LengthSquared()));
                newNormals[t.V3] += Normal * (1 / (c2.LengthSquared() * c3.LengthSquared()));
            }
            for (int i = 0; i < newNormals.Length; i++)
                newNormals[i] = newNormals[i].Normalize();
            TriangleMesh result = input.ShallowCopy();
            result.Normals = newNormals;
            return result;
        }        
    }
}
