﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Zcu.CurvatureBenchmark.Framework;
using Zcu.CurvatureBenchmark.Framework.Distorters;


namespace Zcu.CurvatureBenchmark.Distorters
{
    /// <summary>
    /// Module shifts vertices guided by uniform distribution with rand(-0.5,0.5) * Scale * AverageEdgeLength.
    /// </summary>
    public class UniformNoise : AbstractDistorter
    {
        double scale = 1;

        /// <summary>
        /// Gets/sets scale.
        /// </summary>
        public double Scale
        {
            get
            {
                return scale;
            }

            set
            {
                scale = value;
            }
        }

        /// <summary>
        /// Initialization.
        /// </summary>
        public UniformNoise()
        {
            ToStringFormat = "$Name$ ($Scale$)";
        }

        /// <inheritdoc/>
        public override TriangleMesh Distort(TriangleMesh input)
        {
            var newPoints = new Point3D[input.Points.Length];
            double avgEL = input.AverageEL();
            for (int i = 0;i<input.Points.Length;i++)
            {
                newPoints[i] = input.Points[i] + randomPoint() * scale * avgEL;
            }
            var result = input.ShallowCopy();
            result.Points = newPoints;
            return result;
        }

        private Point3D randomPoint()
        {
            return new Point3D(this.Rand.NextDouble() - 0.5, this.Rand.NextDouble() - 0.5, this.Rand.NextDouble() - 0.5);
        }
    }
}
