﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Zcu.CurvatureBenchmark.Framework;
using Zcu.CurvatureBenchmark.Framework.Estimators;
using Zcu.CurvatureBenchmark.Framework.Evaluators;
using System.Xml;

namespace Zcu.CurvatureBenchmark.Evaluators
{
    /// <summary>
    /// Module evaluates averages and deviations of principal curvatures.
    /// </summary>
    public class BasicEvaluator : AbstractEvaluator
    {
        /// <inheritdoc/>
        override public Dictionary<string, object> Evaluate(TriangleMesh mesh, ICurvatureEstimator estimator)
        {
            SetExclusionZone(mesh);

            double k1avg = 0;
            double k2avg = 0;

            double[] k1 = estimator.K1;
            double[] k2 = estimator.K2;
            int count = 0;

            double avgk1 = 0;
            double avgk2 = 0;

            for (int i = 0;i<mesh.Points.Length;i++)
            {
                if (!excluded[i])
                {
                    k1avg += Math.Abs(mesh.ExactK1[i] - k1[i]);
                    k2avg += Math.Abs(mesh.ExactK2[i] - k2[i]);
                    avgk1 += k1[i];
                    avgk2 += k2[i];
                    count++;
                }
            }

            k1avg /= count;
            k2avg /= count;
            avgk1 /= count;
            avgk2 /= count;

            this.Info("Average k1 error: {0}", k1avg);
            this.Info("Average k2 error: {0}", k2avg);

            double k1dev = 0;
            double k2dev = 0;

            for (int i = 0; i < mesh.Points.Length; i++)
            {
                if (!excluded[i])
                {
                    k1dev += ((mesh.ExactK1[i] - k1[i]) - k1avg) * ((mesh.ExactK1[i] - k1[i]) - k1avg);
                    k2dev += ((mesh.ExactK2[i] - k2[i]) - k2avg) * ((mesh.ExactK2[i] - k2[i]) - k2avg);
                }
            }

            k1dev /= count;
            k2dev /= count;

            this.Info("k1 deviation: {0}", k1dev);
            this.Info("k2 deviation: {0}", k2dev);

            return new Dictionary<string, object>()
            {
                { "AvgK1e", k1avg },
                { "AvgK2e", k2avg },
                { "DevK1", k1dev },
                { "DevK2", k2dev }
            };
        }

        
    }
}
