﻿using System.Collections.Generic;
using Zcu.CurvatureBenchmark.Framework;
using Zcu.CurvatureBenchmark.Framework.Estimators;
using Zcu.CurvatureBenchmark.Framework.Evaluators;


namespace Zcu.CurvatureBenchmark.Evaluators
{
    /// <summary>
    /// Module evaluates dot product of exact and computed normals.
    /// </summary>
    public class NormalEvaluator : AbstractEvaluator
    {
        /// <summary>
        /// Initialization.
        /// </summary>
        public NormalEvaluator()
        {
            excludeBorderTopological = true;
            excludeBorderGeometric = false;
            topologicalExclusionZone = 1;
            geometricExclusionZone = 0.0;
        }

        /// <inheritdoc/>
        public override Dictionary<string, object> Evaluate(TriangleMesh mesh, ICurvatureEstimator estimator)
        {
                SetExclusionZone(mesh);
                double avgDP = 0;
                int count = 0;
                for (int i = 0;i<mesh.Normals.Length;i++)
                {
                    if (excluded[i])
                        continue;
                    Point3D n1 = mesh.Normals[i];
                    Point3D n2 = mesh.ExactNormals[i];
                    avgDP += (n1.X * n2.X + n1.Y * n2.Y + n1.Z * n2.Z);
                    //avgDP += n1.DotProduct(n2);
                    count++;
                }

                avgDP /= count;
                return new Dictionary<string, object>() { { "avgDP", avgDP } };
        }
    }
}
