﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Zcu.CurvatureBenchmark.Framework.DataSources.Explicit
{
    /// <summary>
    /// Explicit function z = f(x,y)
    /// </summary>
    public interface IExplicitFunction : IModule
    {
        /// <summary>
        /// Setup function parameters. Must be called before first Evaluate for given Dataset.
        /// </summary>
        void SetParams();

        /// <summary>
        /// Evaluates only function value and gradient (non-normalized normal) at given point. Other properties are not set.
        /// </summary>
        /// <param name="x">x coordinate</param>
        /// <param name="y">y coordinate</param>
        void EvaluateGradient(double x, double y);

        /// <summary>
        /// Fully evaluates function at given point. After calling this method, all properties are valid.
        /// </summary>
        /// <param name="x">x coordinate</param>
        /// <param name="y">y coordinate</param>
        void Evaluate(double x, double y);

        /// <summary>
        /// Function value - valid after Evaluate and EvaluateGradient call
        /// </summary>
        double F { get; }

        /// <summary>
        /// Gets principal curvature K1 - valid after Evaluate call
        /// </summary>
        double K1 { get; }

        /// <summary>
        /// Gets principal curvature K2 - valid after Evaluate call
        /// </summary>
        double K2 { get; }

        /// <summary>
        /// Gets normal - valid after Evaluate call
        /// </summary>
        Point3D Normal { get; }
    }
}
