﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Zcu.CurvatureBenchmark.Framework.DataSources.Explicit
{
    /// <summary>
    /// Sampler of explicit function
    /// </summary>
    public interface IExplicitSampler : IModule
    {
        /// <summary>
        /// Samples function
        /// </summary>
        /// <param name="function">Function to sample</param>
        void Sample(IExplicitFunction function);

        /// <summary>
        /// Gets used IExplicitFunction
        /// </summary>
        IExplicitFunction Function { get; }

        /// <summary>
        /// Gets sampled points
        /// </summary>
        Point3D[] Points { get;  }

        /// <summary>
        /// Values at sampled points
        /// </summary>
        double[] Values { get; }

        /// <summary>
        /// Gets principal curvatures K1 at sampled points
        /// </summary>
        double[] K1 { get; }

        /// <summary>
        /// Gets principal curvatures K2 at sampled points
        /// </summary>
        double[] K2 { get; }

        /// <summary>
        /// Gets normals at sampled points
        /// </summary>
        Point3D[] Normals { get; }
    }
}
