﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Zcu.CurvatureBenchmark.Framework.DataSources.Implicit
{
    /// <summary>
    /// Implicit function f(x,y,z)
    /// </summary>
    public interface IImplicitFunction : IModule
    {
        /// <summary>
        /// Setup function parameters. Must be called before first Evaluate for given Dataset.
        /// </summary>
        void SetParams();


        /// <summary>
        /// Evaluates only function value at given point. Other properties are not set.
        /// </summary>
        /// <param name="x">x coordinate</param>
        /// <param name="y">y coordinate</param>
        /// <param name="z">z coordinate</param>
		void EvaluateValue(double x, double y, double z);

        /// <summary>
        /// Evaluates only function value and gradient (non-normalized normal) at given point. Other properties are not set.
        /// </summary>
        /// <param name="x">x coordinate</param>
        /// <param name="y">y coordinate</param>
        /// <param name="z">z coordinate</param>
        void EvaluateGradient(double x, double y, double z);

        /// <summary>
        /// Fully evaluates function at given point. After calling this method, all properties are valid.
        /// </summary>
        /// <param name="x">x coordinate</param>
        /// <param name="y">y coordinate</param>
        /// <param name="z">z coordinate</param>
        void Evaluate(double x, double y, double z);

        /// <summary>
        /// Gets function value - valid after Evaluate, EvaluateGradient or EvaluateValue call
        /// </summary>
        double F {get;}

        /// <summary>
        /// Gets principal curvature K1 - valid after Evaluate call
        /// </summary>
        double K1 {get;}

        /// <summary>
        /// Gets principal curvature K2 - valid after Evaluate call
        /// </summary>
        double K2 {get; }

        /// <summary>
        /// Gets normal - valid after Evaluate call
        /// </summary>
        Point3D Normal {get;}

        /// <summary>
        /// Gets gradient - valid after Evaluate and EvaluateGradient call 
        /// </summary>
        Point3D Grad { get; }
    }
}
