﻿namespace Zcu.CurvatureBenchmark.Framework.Distorters
{
    /// <summary>
    /// Represents a module that distorts a triangle mesh, usually by changing either vertex positions or normals.
    /// </summary>
    public interface IDistorter:IModule
    {
        /// <summary>
        /// Distorts the mesh. The method must preserve the original mesh!
        /// This can be done by either making a deep copy or shallow copy with replacing the distorted arrays.
        /// </summary>
        /// <remarks>
        /// To preserve the original mesh, it is necessary to make a deep copy or a shallow copy and replace the
        /// original data, where modified. Example:
        /// <code>
        /// public override TriangleMesh Distort(TriangleMesh input)
        /// {
        ///    var newPoints = ...
        ///    var result = input.ShallowCopy();
        ///    result.Points = newPoints;
        ///    return result;
        /// }
        /// </code>
        /// </remarks>
        /// <param name="input">Original mesh</param>
        /// <returns>Distorted mesh</returns>
        TriangleMesh Distort(TriangleMesh input);
    }
}
