﻿using System;
using System.Linq;

namespace Zcu.CurvatureBenchmark.Framework
{
    /// <summary>
    /// Computes statistics for double array.
    /// </summary>
    public class DoubleArrayStatistic
    {

        /// <summary>
        /// Gets minimum 
        /// </summary>
        public double Min { get; private set; }

        /// <summary>
        /// Gets maximum
        /// </summary>
        public double Max { get; private set; }

        /// <summary>
        /// Gets mean
        /// </summary>
        public double Mean { get; private set; }

        /// <summary>
        /// Gets median
        /// </summary>
        public double Median { get; private set; }

        /// <summary>
        /// Gets standard deviation
        /// </summary>
        public double StDev { get; private set; }

        /// <summary>
        /// Gets variance
        /// </summary>
        public double Var { get; private set; }

        /// <summary>
        /// Computes all statistics.
        /// </summary>
        /// <param name="array">Array of doubles</param>
        public DoubleArrayStatistic(double[] array)
        {
            if(array.Length==0)
            {
                return;
            }
            Array.Sort(array);
            Min = array[0];
            Max = array[array.Length-1];
            Median = array[array.Length / 2];
            Mean = array.Average();
            Var = array.Average(i => (i - Mean) * (i - Mean));
            StDev = Math.Sqrt(Var);
        }
    }
}
