﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml;

namespace Zcu.CurvatureBenchmark.Framework.Estimators
{
    /// <summary>
    /// Represents a module that estimates the principal curvatures of a triangle mesh.
    /// </summary>
    public interface ICurvatureEstimator : IModule
    {
        /// <summary>
        /// Estimates the principal curvatures.
        /// </summary>
        /// <param name="mesh">Input mesh with normals</param>
        void EstimateCurvatures(TriangleMesh mesh);

        /// <summary>
        /// Gets the running time of a module.
        /// </summary>
        /// <remarks>
        /// <para>
        /// Usually modules can be measured by the calling method. In case of wrappers, such a measurement would be unfair,
        /// as there is a lot of other operations (i/o, etc.). For this reason, it is possible to expose RunningTime &gt;=0.
        /// </para>
        /// <para>
        /// This part of API is subject to change. 
        /// </para>
        /// <para>
        /// Q: Do we really use it? 
        /// </para>
        /// <para>
        /// Q: Is there an easy way to unify the measurement, ie. set default measurement for standard module and override it for wrappers?
        /// </para>
        /// </remarks>
        /// <preliminary/>
        int RunningTime { get; }

        /// <summary>
        /// Gets the array of K1 curvature associated with vertices.
        /// </summary>
        double[] K1 { get; }

        /// <summary>
        /// Gets the array of K2 curvature associated with vertices.
        /// </summary>
        double[] K2 { get; }

        /// <summary>
        /// Creates a shallow copy of ICurvatureEstimator
        /// </summary>
        /// <returns>Shallow copy</returns>
        ICurvatureEstimator ShallowCopy();

    }
}
