﻿using System.Collections.Generic;
using Zcu.CurvatureBenchmark.Framework.Estimators;

namespace Zcu.CurvatureBenchmark.Framework.Evaluators
{
    /// <summary>
    /// Abstract ICurvatureEvaluator implementation, using default SetConfig method.
    /// </summary>

    public abstract class AbstractEvaluator : AbstractModule, ICurvatureEvaluator
    {
        /// <summary>
        /// <see cref="ICurvatureEvaluator.ExcludeBorderTopological"/>
        /// </summary>
        protected bool excludeBorderTopological = false;

        /// <summary>
        /// <see cref="ICurvatureEvaluator.ExcludeBorderGeometric"/>
        /// </summary>
        protected bool excludeBorderGeometric = false;

        /// <summary>
        /// <see cref="ICurvatureEvaluator.TopologicalExclusionZone"/>
        /// </summary>
        protected int topologicalExclusionZone = 1;

        /// <summary>
        /// <see cref="ICurvatureEvaluator.GeometricExclusionZone"/>
        /// </summary>
        protected double geometricExclusionZone = 0.03;

        /// <inheritdoc/>
        public bool ExcludeBorderTopological
        {
            get
            {
                return excludeBorderTopological;
            }

            set
            {
                excludeBorderTopological = value;
            }
        }

        /// <inheritdoc/>
        public bool ExcludeBorderGeometric
        {
            get
            {
                return excludeBorderGeometric;
            }

            set
            {
                excludeBorderGeometric = value;
            }
        }

        /// <inheritdoc/>
        public int TopologicalExclusionZone
        {
            get
            {
                return topologicalExclusionZone;
            }

            set
            {
                topologicalExclusionZone = value;
            }
        }

        /// <inheritdoc/>
        public double GeometricExclusionZone
        {
            get
            {
                return geometricExclusionZone;
            }

            set
            {
                geometricExclusionZone = value;
            }
        }

        /// <summary>
        /// An array with exclusion status of corresponding vertices.
        /// </summary>
        protected bool[] excluded;

        /// <inheritdoc/>
        public virtual void SetExclusionZone(TriangleMesh mesh)
        {
            excluded = mesh.excludeVertices(excludeBorderTopological, excludeBorderGeometric, topologicalExclusionZone, geometricExclusionZone);
        }

        /// <inheritdoc/>
        public abstract Dictionary<string, object> Evaluate(TriangleMesh mesh, ICurvatureEstimator estimator);
    }
}
