﻿namespace Zcu.CurvatureBenchmark.Framework
{
    /// <summary>
    /// Represents a line given by two vertices.
    /// </summary>
    public struct Line
    {
        int v1, v2;

        /// <summary>
        /// Gets/sets index of second vertex.
        /// </summary>
        public int V2
        {
            get { return v2; }
            set { v2 = value; }
        }

        /// <summary>
        /// Gets/sets index of first vertex
        /// </summary>
        public int V1
        {
            get { return v1; }
            set { v1 = value; }
        }

        /// <summary>
        /// Creates a line given by vertices v1, v2. If the line is not oriented, then line.v1 = min(v1,v2) and line.v2 = max(v1,v2).
        /// </summary>
        /// <param name="v1">Index of first vertex</param>
        /// <param name="v2">Index of second vertex</param>
        /// <param name="oriented">Preserve orientation (default FALSE)</param>
        public Line(int v1, int v2, bool oriented = false)
        {
            if (oriented)
            {
                this.v1 = v1;
                this.v2 = v2;
            }
            else
            {
                if (v1 < v2)
                {
                    this.v1 = v1;
                    this.v2 = v2;
                }
                else
                {
                    this.v1 = v2;
                    this.v2 = v1;
                }
            }
        }

        /// <summary>
        /// Gets a hashcode based on the values of v1 and v2.
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            return (v1 + 1000 * v2);
        }
    }
}
