using System;
using System.IO;
using System.Globalization;
using System.ComponentModel;
using System.Collections.Generic;

namespace Zcu.CurvatureBenchmark.Framework
{
    /// <summary>
    /// Obj triangle mesh file loader module.
    /// </summary>
    public class ObjLoader
    {
        bool flipOrientation = false;

        /// <summary>
        /// Gets/sets whether the triangles orientation should be flipped during the loading process
        /// </summary>
        public bool FlipOrientation
        {
            get { return flipOrientation; }
            set { flipOrientation = value; }
        }

        private TriangleMesh output;       

        /// <summary>
        /// Loads the input obj file.
        /// </summary>
        /// <param name="fileName">Filename to load</param>
        /// <returns>Triangle mesh</returns>
        public TriangleMesh Execute(string fileName)
        {                       
            StreamReader sr;
            sr = new StreamReader(fileName);
            List<Point3D> vertices = new List<Point3D>();
            List<Triangle> triangles = new List<Triangle>();
            
            Point3D point;
            Triangle triangle;
            NumberFormatInfo nfi = new NumberFormatInfo();
            nfi.NumberDecimalSeparator = ".";
            nfi.NumberGroupSeparator = ",";
            
            string line = sr.ReadLine();
            while (line != null)
            {
                line = line.Trim();

                //parsing of a vertex
                if (line.StartsWith("v "))
                {
                    string[] coords = line.Split(new char[] { ' ' }, 5, StringSplitOptions.RemoveEmptyEntries);

                    double c1 = double.Parse(coords[1], nfi);
                    double c2 = double.Parse(coords[2], nfi);
                    double c3 = double.Parse(coords[3], nfi);

                    point = new Point3D(c1, c2, c3);
                    vertices.Add(point);
                }                                

                // parsing of a triangle
                if ((line.StartsWith("f ")) || (line.StartsWith("fo ")) || (line.StartsWith("f\t")))
                {
                    string[] indices = line.Split(new char[] { ' ', '\t' }, 5, StringSplitOptions.RemoveEmptyEntries);

                    string[] parts = indices[1].Split(new char[] {'/'}, StringSplitOptions.RemoveEmptyEntries);                        
                    int v1 = int.Parse(parts[0]) - 1;

                    parts = indices[2].Split(new char[] { '/' }, StringSplitOptions.RemoveEmptyEntries);                                                
                    int v2 = int.Parse(parts[0]) - 1;

                    parts = indices[3].Split(new char[] { '/' }, StringSplitOptions.RemoveEmptyEntries);
                    int v3 = int.Parse(parts[0]) - 1;

                    if (flipOrientation)
                        triangle = new Triangle(v1, v3, v2);
                    else
                        triangle = new Triangle(v1, v2, v3);

                    if (indices.Length == 4)
                        triangles.Add(triangle);

                    if (indices.Length == 5)
                    {
                        parts = indices[4].Split(new char[] { '/' }, StringSplitOptions.RemoveEmptyEntries);
                        v2 = int.Parse(parts[0]) - 1;

                        if (flipOrientation)
                            triangle = new Triangle(v1, v2, v3);
                        else
                            triangle = new Triangle(v1, v3, v2);
                        triangles.Add(triangle);                        
                    }
                }
                line = sr.ReadLine();                
            }

            // creating an output Mesh instance
            output = new TriangleMesh();
            output.Points = vertices.ToArray();
            output.Triangles = triangles.ToArray();
            return (output);
        }// Execute
    }//ObjLoader
}
