using System.IO;
using System.Globalization;

namespace Zcu.CurvatureBenchmark.Framework
{
    /// <summary>
    /// Very simple obj saver (vertices and triangles only).
    /// </summary>
    /// <preliminary/>
    public class ObjSaver
    {

        private string fileName = "mesh.obj";

        /// <summary>
        /// Filename
        /// </summary>
        public string FileName
        {
            get { return fileName; }
            set { fileName = value; }
        }

        /// <summary>
        /// Writes triangle mesh to a file.
        /// </summary>
        /// <param name="mesh">Mesh</param>
        public void Execute(TriangleMesh mesh)
        {
            StreamWriter sw = new StreamWriter(fileName);

            Point3D[] points = mesh.Points;

            Triangle[] triangles = mesh.Triangles;

            bool[] used = new bool[mesh.Points.Length];
            if (triangles != null)
            {
                foreach (Triangle t in triangles)
                {
                    used[t.V1] = true;
                    used[t.V2] = true;
                    used[t.V3] = true;
                }
            }

            NumberFormatInfo nfi = new NumberFormatInfo();
            nfi.NumberDecimalSeparator = ".";
            nfi.NumberGroupSeparator = "";

            for (int i = 0; i < points.Length; i++)
            {
                    Point3D p = (Point3D)points[i];
                    sw.WriteLine("v " + p.X.ToString(nfi) + " "
                        + p.Y.ToString(nfi) + " " + p.Z.ToString(nfi));
                
            }

            if (triangles != null)
            {
                for (int i = 0; i < triangles.Length; i++)
                {
                    Triangle t = (Triangle)triangles[i];
                    sw.WriteLine("f " + (t.V1 + 1).ToString() + " " + (t.V2 + 1).ToString() + " " + (t.V3 + 1).ToString());
                }
            }
            sw.Close();
        }
    }
}
