﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;

namespace SlimDXRenderSystem.Core
{
    public class Group : Movable
    {
        private Dictionary<string, Object3D> objects;

        [Category("Relationship"), ReadOnly(true)]
        public Dictionary<string, Object3D> Objects
        {
            get { return objects; }
            set { objects = value; }
        }

        [Category("Information")]
        public int NumberOfObjects
        {
            get { return objects.Count; }
        }

        public Group(string name)
        {
            this.Name = name;

            objects = new Dictionary<string,Object3D>();
        }

        public void AddObject(Object3D obj)
        {
            if (!objects.ContainsKey(obj.Name))
            {
                objects.Add(obj.Name, obj);
                if (obj.Group != null)
                    obj.Group.RemoveObject(obj);
                obj.Group = this;
            }
        }

        public void RemoveObject(Object3D obj)
        {
            if (objects.ContainsKey(obj.Name))
            {
                objects.Remove(obj.Name);
                obj.Group = null;
            }
        }

        public void Dispose()
        {
            foreach (Object3D obj in objects.Values)
            {
                obj.Dispose();
            }
            objects.Clear();
        }
    }
}
