﻿using System;
using System.ComponentModel;
using SlimDX;

namespace SlimDXRenderSystem.Core
{
    public class Material
    {
        private static readonly Vector4 DEF_DIFFUSE = new Vector4(1.0f, 1.0f, 1.0f, 1.0f);
        private static readonly Vector4 DEF_SPECULAR = new Vector4(1.0f, 1.0f, 1.0f, 1.0f);
        private static readonly Vector4 DEF_EMISSIVE = new Vector4(0.0f, 0.0f, 0.0f, 1.0f);
        private static readonly Vector4 DEF_AMBIENT = new Vector4(0.0f, 0.0f, 0.0f, 1.0f);
        private const int DEF_SPECULAR_SHININESS = 50;

        private MaterialStruct material;

        [Browsable(false)]
        public MaterialStruct MaterialStruct
        {
            get { return material; }
            set { material = value; }
        }

        public Vector4 Ambient
        {
            get { return material.Ambient; }
            set { material.Ambient = value; }
        }

        public Vector4 Diffuse
        {
            get { return material.Diffuse; }
            set { material.Diffuse = value; }
        }

        public Vector4 Specular
        {
            get { return material.Specular; }
            set { material.Specular = value; }
        }

        public Vector4 Emissive
        {
            get { return material.Emissive; }
            set { material.Emissive = value; }
        }

        public float SpecularShininess
        {
            get { return material.Shininess; }
            set
            {
                if (value < 0)
                    material.Shininess = 0.0f;
                else
                    material.Shininess = value;
            }
        }

        public Material()
        {
            material = new MaterialStruct();
            material.Diffuse = DEF_DIFFUSE;
            material.Specular = DEF_SPECULAR;
            material.Shininess = DEF_SPECULAR_SHININESS;
            material.Emissive = DEF_EMISSIVE;
            material.Ambient = DEF_AMBIENT;
        }
    }
}
