﻿using System;
using System.Collections.Generic;
using SlimDX;
using SlimDX.Direct3D9;
using SlimDXRenderSystem.Direct3DBase;
using RenderMgr = SlimDXRenderSystem.Managers.RenderManager;
using Effect = SlimDX.Direct3D9.Effect;
using ResourceMgr = SlimDXRenderSystem.Managers.ResourceManager;

namespace SlimDXRenderSystem.Direct3D9
{
    public class Effect9 : SlimDXRenderSystem.Direct3DBase.Effect
    {
        #region Fields
        
        private String compileError;
        private Effect effect;
        private EffectPool effectPool;

        private Dictionary<string, EffectHandle> effectHandles;

        #endregion
        #region Public Methods
        public Effect9()
        {
            compileError = "";
            effectPool = new EffectPool();
            effectHandles = new Dictionary<string, EffectHandle>();
        }

        public override void LoadParameter(string semantic)
        {
            EffectHandle ret;
            if ((ret = effect.GetParameterBySemantic(null, semantic)) == null)
                throw new Exception("Effect file doesn't contain the " + semantic + " semantic parameter!");

            effectHandles.Add(semantic, ret);
        }

        
        public override void LoadFromFile(string filename)
        {
            effect = SlimDX.Direct3D9.Effect.FromFile(((Renderer9)RenderMgr.Instance.Renderer).Device, filename, null, null, null, ShaderFlags.Debug, effectPool, out compileError);
            if (compileError != "")
                throw new Exception(compileError);
            base.LoadFromFile(filename);
        }

        public override void LoadFromResource(string filename)
        {
            string effectString = ResourceMgr.Instance.GetFromResource(filename);
            effect = SlimDX.Direct3D9.Effect.FromString(((Renderer9)RenderMgr.Instance.Renderer).Device, effectString, null, null, null, ShaderFlags.Debug, effectPool, out compileError);
            if (compileError != "")
                throw new Exception(compileError);

            base.LoadFromResource(filename);
        }

        public int Begin()
        {
            return effect.Begin();
        }

        public void End()
        {
            effect.End();
        }

        public void BeginPass(int pass)
        {
            effect.BeginPass(pass);
        }

        public void EndPass()
        {
            effect.EndPass();
        }

        public override void ApplyChanges()
        {
            effect.CommitChanges();
        }

        public override void SetParameter<T>(string param, T value)
        {
            effect.SetValue<T>(effectHandles[param], value);
        }

        public override T GetParameter<T>(string param)
        {
            return effect.GetValue<T>(effectHandles[param]);
        }

        public override void Dispose()
        {
            foreach (EffectHandle efh in effectHandles.Values)
            {
                efh.Dispose();
            }
            effectHandles.Clear();

            if (effectPool != null && !effectPool.Disposed)
            {
                effectPool.Dispose();
                effectPool = null;
            }
            if (effect != null && !effect.Disposed)
            {
                effect.Dispose();
                effect = null;
            }

            base.Dispose();
        }
        #endregion
    }
}
