﻿using System;
using SlimDX;
using SlimDXRenderSystem.Managers;

namespace SlimDXRenderSystem.Direct3DBase
{
    /// <summary>
    /// Base class for concrete Effect implementations
    /// </summary>
    public abstract class Effect
    {
        #region Static constants
        /*Bool variables*/
        public const string LIGHTING = "LIGHTING";
        /*Matrices*/
        public const string WORLD = "WORLD";
        public const string VIEW = "VIEW";
        public const string PROJECTION = "PROJECTION";
        public const string WORLDINVTRANSPOSE = "WORLDINVTRANSPOSE";
        public const string WORLDVIEWPROJECTION = "WORLDVIEWPROJECTION";
        /*Vector4 variables*/
        public const string AMBIENTCOLOR = "AMBIENTCOLOR";
        public const string CAMERAPOSITION = "CAMERAPOSITION";

        /*Float variables*/
        public const string AMBIENTINTENSITY = "AMBIENTINTENSITY";

        /*Structs*/
        public const string LIGHT = "LIGHT";
        public const string MATERIAL = "MATERIAL";
        #endregion

        private bool disposed = true;

        public bool Disposed
        {
            get { return disposed; }
        }

        public Effect()
        {

        }

        public virtual void LoadFromFile(string filename)
        {
            disposed = false;

            LoadAllParameters();
        }

        public virtual void LoadFromResource(string filename)
        {
            disposed = false;

            LoadAllParameters();
        }

        public virtual void LoadAllParameters()
        {
            try
            {
                LoadParameter(AMBIENTCOLOR);
                LoadParameter(AMBIENTINTENSITY);
                LoadParameter(LIGHTING);
                LoadParameter(WORLD);
                LoadParameter(VIEW);
                LoadParameter(PROJECTION);
                LoadParameter(WORLDINVTRANSPOSE);
                LoadParameter(WORLDVIEWPROJECTION);
                LoadParameter(CAMERAPOSITION);
                LoadParameter(LIGHT);
                LoadParameter(MATERIAL);
            }
            catch (Exception e)
            {
                throw e;
            }
        }

        public abstract void ApplyChanges();
        public abstract void LoadParameter(string semantic);
        public abstract void SetParameter<T>(string param, T value) where T : struct;

        public abstract T GetParameter<T>(string param) where T : struct;

        public virtual void Dispose()
        {
            disposed = true;
        }
    }
}
