﻿using System;
using System.Drawing;
using System.Collections.Generic;
using System.ComponentModel;
using SlimDX;

namespace SlimDXRenderSystem.Direct3DBase
{
    /// <summary>
    /// Base class for concrete Renderers
    /// </summary>
    public abstract class Renderer
    {
        private bool disposed = true;
        private Color4 backgroundColor;

        /// <summary>
        /// Determines if the renderer is disposed.
        /// </summary>
        [Browsable(false)]
        public bool Disposed
        {
            get { return disposed; }
        }

        public Color4 BackgroundColor
        {
            get { return backgroundColor; }
            set { backgroundColor = value; }
        }

        public abstract ShadeMode Shademode
        {
            set;
        }

        public abstract FillMode Fillmode
        {
            set;
        }

        public abstract Cull Culling
        {
            set;
        }

        public abstract MultisampleQuality Multisample
        {
            get;
            set;
        }

        public Renderer()
        {
            backgroundColor = new Color4(1.0f, 0.0f, 0.0f, 0.0f);
        }

        public virtual void Initialize()
        {
            disposed = false;
        }

        public abstract void Render();
        public abstract void RenderText();
        public abstract void Reset();

        public abstract void DrawText(int x, int y, string text);
        public abstract void DrawPoint(Vector3 pos);

        public abstract void CaptureScreenshot(String name, int number);

        public abstract void SaveLightTransformations(string filname);
        public abstract void LoadLightTransformations(string filname);

        public virtual void Dispose()
        {
            disposed = true;
        }
    }
}
