﻿using System;

namespace SlimDXRenderSystem
{
    /// <summary>
    /// D3DVersion supported by SlimDXRenderSystem.
    /// </summary>
    public enum D3DVersion
    {
        /// <summary>
        /// Direct3D9 rendering.
        /// </summary>
        Direct3D9/*,
        Direct3D10,
        Direct3D11*/
    }

    public enum MeshType
    {
        /// <summary>
        /// Mesh will be rendered as triangle mesh.
        /// </summary>
        TriangleMesh,

        /// <summary>
        /// Mesh will be rendered as line mesh.
        /// </summary>
        LineMesh
    }
    
    /// <summary>
    /// Render filled mode.
    /// </summary>
    public enum FillMode
    {
        /// <summary>
        ///Render only vertices.
        /// </summary>
        Point = 1,

        /// <summary>
        /// Render wireframe model.
        /// </summary>
        Wireframe,

        /// <summary>
        /// Render solid filled model.
        /// </summary>
        Solid,
    }

    /// <summary>
    /// Mesh shading.
    /// </summary>
    public enum ShadeMode
    {
        /// <summary>
        /// Mesh will be shaded as flat planes.
        /// </summary>
        Flat = 1,

        /// <summary>
        /// Mesh will be smooth shaded.
        /// </summary>
        Gouraud,
    }

    /// <summary>
    /// Backface culling.
    /// </summary>
    public enum Cull
    {
        /// <summary>
        /// All triangles will be drawn.
        /// </summary>
        None = 1,

        /// <summary>
        /// Draw triangles clockwise.
        /// </summary>
        Clockwise,

        /// <summary>
        /// Draw triangles counterclockwise.
        /// </summary>
        Counterclockwise,
    }

    /// <summary>
    /// Antialiasing multisample quality.
    /// </summary>
    public enum MultisampleQuality
    {
        /// <summary>
        /// No antialiasing.
        /// </summary>
        None = 0,

        /// <summary>
        /// Nonmaskable antialiasing.
        /// </summary>
        Nonmaskable = 1,

        /// <summary>
        /// 2 samples of antialiasing.
        /// </summary>
        Samples_2 = 2,

        /// <summary>
        /// 4 samples of antialiasing.
        /// </summary>
        Samples_4 = 4,

        /// <summary>
        /// 6 samples of antialiasing.
        /// </summary>
        Samples_6 = 6,

        /// <summary>
        /// 8 samples of antialiasing.
        /// </summary>
        Samples_8 = 8,

        /// <summary>
        /// 10 samples of antialiasing.
        /// </summary>
        Samples_10 = 10,

        /// <summary>
        /// 12 samples of antialiasing.
        /// </summary>
        Samples_12 = 12,

        /// <summary>
        /// 14 samples of antialiasing.
        /// </summary>
        Samples_14 = 14,

        /// <summary>
        /// 16 samples of antialiasing.
        /// </summary>
        Samples_16 = 16,
    }

    public enum LightType
    {
        /// <summary>
        /// Directional light which is defined by its direction
        /// </summary>
        Directional,
        /// <summary>
        /// Point light which is defined by its position
        /// </summary>
        Point,
        /// <summary>
        /// Spot light which is defined by its direction, position, inner and outer cone.
        /// </summary>
        Spot,
    }
}
