﻿using System;
using System.Diagnostics;

namespace SlimDXRenderSystem.Managers
{
    public class TimeManager : Core.Singleton<TimeManager>
    {
        public static int UPDATE_INTERVAL = 500;
        private Stopwatch sw;
        private long lastUpdate;
        private long frames;
        private float fps;

        public float FPS
        {
            get { return fps; }
        }
        
        public void Initialize()
        {
            sw = new Stopwatch();
            sw.Start();
            lastUpdate = 0;
            fps = 0.0f;
        }

        public void Dispose()
        {
            sw.Stop();
            sw.Reset();
        }

        public void Update()
        {
            frames++;
            long update = sw.ElapsedMilliseconds;
            if (update - lastUpdate > UPDATE_INTERVAL)
            {
                fps = (float)frames / (update - lastUpdate) * 1000; // frames per millisecond * 1000 to get second.
                lastUpdate = update;
                frames = 0;
            }
        }
    }
}
