﻿using System;
using SlimDXRenderSystem;
using SlimDX.DirectInput;
using SlimDX;
using SceneMgr = SlimDXRenderSystem.Managers.SceneManager;
using InputMgr = SlimDXRenderSystem.Managers.InputManager;
using RenderMgr = SlimDXRenderSystem.Managers.RenderManager;

namespace Zcu.Mve.ZMalik.SlimDXRenderer
{
    public class InputSetting
    {
        public InputSetting()
        {
            InputMgr.Instance.AddMouseClickAction(MouseObject.Button1, MousePick);
            InputMgr.Instance.AddMouseMoveButAction(MouseObject.Button1, RotateCamera);
            InputMgr.Instance.AddMouseMoveButAction(MouseObject.Button2, MoveObject);
            InputMgr.Instance.AddMouseMoveButAction(MouseObject.Button3, MoveCamera);
            InputMgr.Instance.AddMouseMoveKeyAction(Key.L, RotateLight);
            InputMgr.Instance.AddMouseMoveAction(ZoomCamera);

            InputMgr.Instance.AddKeyClickAction(Key.Comma, PreviousObject);
            InputMgr.Instance.AddKeyClickAction(Key.Period, NextObject);

            InputMgr.Instance.AddKeyIsPressedAction(Key.W, MoveCameraForward);
            InputMgr.Instance.AddKeyIsPressedAction(Key.S, MoveCameraBackward);
            InputMgr.Instance.AddKeyIsPressedAction(Key.A, MoveCameraLeft);
            InputMgr.Instance.AddKeyIsPressedAction(Key.D, MoveCameraRight);
            InputMgr.Instance.AddKeyIsPressedAction(Key.E, MoveCameraUp);
            InputMgr.Instance.AddKeyIsPressedAction(Key.Q, MoveCameraDown);
            InputMgr.Instance.AddKeyIsPressedAction(Key.X, CameraZoomIn);
            InputMgr.Instance.AddKeyIsPressedAction(Key.Z, CameraZoomOut);
        }

        public void MousePick()
        {
            if (ModuleManager.Instance.Pick != PickType.NONE)
                ModuleManager.Instance.Picking();
        }
        
        public void RotateCamera(int mousex, int mousey, int mousez)
        {
            SceneMgr.Instance.Camera.Rotate((float)mousex / 10, (float)mousey / 10, 0.0f);
        }

        public void MoveCamera(int mousex, int mousey, int mousez)
        {
            if (InputMgr.Instance.Keystate.IsPressed(Key.LeftShift))
                SceneMgr.Instance.Camera.Move((float)mousex / 10, 0.0f, (float)mousey / 10);
            else
                SceneMgr.Instance.Camera.Move((float)mousex / 10, (float)mousey / 10, 0.0f);
        }

        public void ZoomCamera(int mousex, int mousey, int mousez)
        {

            if (InputMgr.Instance.Keystate.IsPressed(Key.LeftShift))
            {
                SceneMgr.Instance.Camera.Zoom((float)-mousez / 25);
            }
            else
            {
                SceneMgr.Instance.Camera.Zoom((float)-mousez / 100);
            }
        }

        public void MoveObject(int mousex, int mousey, int mousez)
        {
            float x = (float)mousex / 100;
            float y = (float)mousey / 100;
            Vector3 move = Vector3.Zero;
            if (InputMgr.Instance.Keystate.IsPressed(Key.LeftShift))
            {
                move = SceneMgr.Instance.Camera.XAxis * x;
                move += SceneMgr.Instance.Camera.ZAxis * y;
            }
            else if (InputMgr.Instance.Keystate.IsPressed(Key.LeftControl))
            {
                if (InputMgr.Instance.Keystate.IsPressed(Key.LeftAlt))
                {
                    ModuleManager.Instance.MeshGroup.Scaling(y, y, y);
                }
                move = new Vector3(x, -y, 0.0f);
            }
            else if (InputMgr.Instance.Keystate.IsPressed(Key.LeftAlt))
            {
                ModuleManager.Instance.MeshGroup.Rotate(-x, -y, 0.0f);
            }
            else
            {
                move = SceneMgr.Instance.Camera.XAxis * x;
                move -= SceneMgr.Instance.Camera.YAxis * y;
            }

            ModuleManager.Instance.MeshGroup.Move(move);
        }

        public void PreviousObject()
        {
            ModuleManager.Instance.CurrentMesh -= 1;
        }

        public void NextObject()
        {
            ModuleManager.Instance.CurrentMesh += 1;
        }

        float elev = 0.0f;
        float azim = 0.0f;
        public void RotateLight(int mousex, int mousey, int mousez)
        {
            elev += (float)mousey / 100;
            azim -= (float)mousex / 100;
            SceneMgr.Instance.Light.Direction = new Vector3((float)(Math.Cos(-azim) * Math.Cos(elev)),
                                                    (float)(Math.Sin(elev)),
                                                    (float)(Math.Sin(-azim) * Math.Cos(elev)));
        }

        public void MoveCameraForward()
        {
            SceneMgr.Instance.Camera.Move(0.0f, 0.0f, -0.5f);
        }

        public void MoveCameraBackward()
        {
            SceneMgr.Instance.Camera.Move(0.0f, 0.0f, 0.5f);
        }

        public void MoveCameraLeft()
        {
            SceneMgr.Instance.Camera.Move(0.5f, 0.0f, 0.0f);
        }

        public void MoveCameraRight()
        {
            SceneMgr.Instance.Camera.Move(-0.5f, 0.0f, 0.0f);
        }

        public void MoveCameraUp()
        {
            SceneMgr.Instance.Camera.Move(0.0f, 0.5f, 0.0f);
        }

        public void MoveCameraDown()
        {
            SceneMgr.Instance.Camera.Move(0.0f, -0.5f, 0.0f);
        }

        public void CameraZoomIn()
        {
            SceneMgr.Instance.Camera.Zoom(-0.1f);
        }

        public void CameraZoomOut()
        {
            SceneMgr.Instance.Camera.Zoom(0.1f);
        }
    }
}
