﻿using System;
using System.Drawing;
using System.ComponentModel;
using SlimDXRenderSystem;
using SlimDX.DirectInput;
using SceneMgr = SlimDXRenderSystem.Managers.SceneManager;
using InputMgr = SlimDXRenderSystem.Managers.InputManager;
using RenderMgr = SlimDXRenderSystem.Managers.RenderManager;

namespace Zcu.Mve.ZMalik.SlimDXRenderer
{
    public class RenderSetting
    {
        #region Fields
        private bool autorun = false;
        private bool shownormals = false;
        private bool displayIndices = false;
        private bool displayTriangleIndices = false;

        #region Properties
        public bool Shownormals
        {
            get { return shownormals; }
            set
            {
				string objName = ModuleManager.Instance.CurrentObject.Name;
				SlimDXRenderSystem.Core.Object3D obj = SceneMgr.Instance.GetObject(objName);
                if (obj != null)
                {
                    shownormals = value;
                    if (shownormals)
                        obj.EnableDrawIndices("normal");
                    else
                        obj.DisableDrawIndices("normal");
                }
            }
        }

        public bool Lighting
        {
            get { return RenderMgr.Instance.Lighting; }
            set { RenderMgr.Instance.Lighting = value; }
        }

        public Color Ambient
        {
            get { return RenderMgr.Instance.AmbientLight; }
            set{ RenderMgr.Instance.AmbientLight = value; }
        }

        public float AmbientIntensity
        {
            get { return RenderMgr.Instance.AmbientIntensity; }
            set { RenderMgr.Instance.AmbientIntensity = value; }
        }

        /// <summary>
        /// Shade mode of mesh.
        /// </summary>
        [Category("Render states")]
        public ShadeMode Shademode
        {
            get { return RenderMgr.Instance.Shademode; }
            set { RenderMgr.Instance.Shademode = value; }
        }

        /// <summary>
        /// Fill mode of mesh.
        /// </summary>
        [Category("Render states")]
        public FillMode Fillmode
        {
            get { return RenderMgr.Instance.Fillmode; }
            set { RenderMgr.Instance.Fillmode = value; }
        }

        /// <summary>
        /// Back face culling.
        /// </summary>
        [Category("Render states")]
        public Cull Culling
        {
            get { return RenderMgr.Instance.Culling; }
            set { RenderMgr.Instance.Culling = value; }
        }

        /// <summary>
        /// Antialiasing multi sample type.
        /// </summary>
        public MultisampleQuality Multisample
        {
            get { return RenderMgr.Instance.Multisample; }
            set { RenderMgr.Instance.Multisample = value; }
        }

        public bool DisplayIndices
        {
            get { return displayIndices; }
            set { displayIndices = value; }
        }

        public bool DisplayTriangleIndices
        {
            get { return displayTriangleIndices; }
            set { displayTriangleIndices = value; }
        }

        public bool PlayAnimation
        {
            get { return autorun; }
            set { autorun = value; }
        }
        #endregion
        #endregion


        public RenderSetting()
        {
            InputMgr.Instance.AddKeyClickAction(Key.M, ToogleAutorun);
            InputMgr.Instance.AddKeyClickAction(Key.N, ToggleShowNormals);
            InputMgr.Instance.AddKeyClickAction(Key.X, ToggleLighting);
            InputMgr.Instance.AddKeyClickAction(Key.U, ToggleDisplayTriangleIndices);
            InputMgr.Instance.AddKeyClickAction(Key.I, ToggleDisplayIndices);
            InputMgr.Instance.AddKeyClickAction(Key.C, CycleCulling);
            InputMgr.Instance.AddKeyClickAction(Key.Y, CycleFillmode);
            InputMgr.Instance.AddKeyClickAction(Key.F9, LoadCameraPosition);
            InputMgr.Instance.AddKeyClickAction(Key.F10, SaveCameraPosition);
            InputMgr.Instance.AddKeyClickAction(Key.F11, SaveSingleFrame);
            InputMgr.Instance.AddKeyClickAction(Key.F12, SaveAnimationFrames);
        }

        public void ToggleShowNormals()
        {
            Shownormals = !Shownormals;
        }

        public void ToggleLighting()
        {
            Lighting = !Lighting;
        }

        public void ToggleDisplayTriangleIndices()
        {
            displayTriangleIndices = !displayTriangleIndices;
        }

        public void ToggleDisplayIndices()
        {
            displayIndices = !displayIndices;
        }

        public void CycleCulling()
        {
            switch (Culling)
            {
                case Cull.Clockwise:
                    Culling = Cull.Counterclockwise;
                    break;
                case Cull.Counterclockwise:
                    Culling = Cull.None;
                    break;
                case Cull.None:
                    Culling = Cull.Clockwise;
                    break;
            }
        }

        public void CycleFillmode()
        {
            switch (Fillmode)
            {
                case FillMode.Point:
                    Fillmode = FillMode.Wireframe;
                    break;
                case FillMode.Wireframe:
                    Fillmode = FillMode.Solid;
                    break;
                case FillMode.Solid:
                    Fillmode = FillMode.Point;
                    break;
            }
        }

        public void ToogleAutorun()
        {
            autorun = !autorun;
        }

        public void SaveCameraPosition()
        {
            SceneMgr.Instance.SaveWorld();
        }

        public void LoadCameraPosition()
        {
            SceneMgr.Instance.LoadWorld();
        }

        public void SaveSingleFrame()
        {
            RenderMgr.Instance.CaptureScreenshot("Screenshot", ModuleManager.Instance.NumScreenShots++);
        }

        public void SaveAnimationFrames()
        {
            int num = 0;
            int beforeMesh = ModuleManager.Instance.CurrentMesh;
            ModuleManager.Instance.CurrentMesh = 0;
            for (int i = 0; i < ModuleManager.Instance.MeshCount; i++)
            {
                RenderMgr.Instance.Render();
                RenderMgr.Instance.CaptureScreenshot("Frame", num++);
                ModuleManager.Instance.CurrentMesh++;
            }
        }
    }
}
