using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Zcu.CurvatureBenchmark.Framework;

using Expr = MathNet.Symbolics.Expression;
using Eval = MathNet.Symbolics.Evaluate;
using MathNet.Numerics;
using MathNet.Numerics.LinearAlgebra;
using MathNet.Symbolics;
using System.Runtime.CompilerServices;
using System.IO;

namespace Zcu.CurvatureBenchmark.Estimators
{
    internal class PNG1Patch_gen
    {
        //public Point3D b300 { get; set; }
        //public Point3D b210 { get; set; }
        //public Point3D b120 { get; set; }
        //public Point3D b030 { get; set; }
        //public Point3D b201 { get; set; }
        //public Point3D b111 { get; set; }
        //public Point3D b021 { get; set; }
        //public Point3D b102 { get; set; }
        //public Point3D b012 { get; set; }
        //public Point3D b003 { get; set; }

        Expr functionExpr;
        Expr duExpr;
        Expr dvExpr;
        //Expr dwExpr;

        Expr duuExpr;
        Expr duvExpr;
        //Expr duwExpr;

        Expr dvuExpr;
        Expr dvvExpr;
        //Expr dvwExpr;

        //Expr dwuExpr;
        //Expr dwvExpr;
        //Expr dwwExpr;

        EdgePatch[] edgePatches;

        public PNG1Patch_gen(EdgePatch[] edgePatches)
        {
            this.edgePatches = edgePatches;
            InitBlendingFunctions();
        }

        public void InitBlendingFunctions()
        {
            string b030 = " (b030) ";
            string b003 = " (b003) ";
            string b300 = " (b300) ";

            string b201 = " ((1 / (u^2 + (1 - u)^2)) * (b1x201 * (1 - u)^2 + b2x201 * u^2)) ";
            string b102 = " ((1 / (u^2 + (1 - u)^2)) * (b3x102 * (1 - u)^2 + b2x102 * u^2)) ";

            string b012 = " ((1 / (w^2 + (1 - w)^2)) * (b3x012 * (1 - w)^2 + b1x012 * w^2)) ";
            string b021 = " ((1 / (w^2 + (1 - w)^2)) * (b2x021 * (1 - w)^2 + b1x021 * w^2)) ";

            string b120 = " ((1 / (v^2 + (1 - v)^2)) * (b2x120 * (1 - v)^2 + b3x120 * v^2)) ";
            string b210 = " ((1 / (v^2 + (1 - v)^2)) * (b1x210 * (1 - v)^2 + b3x210 * v^2)) ";

            string b111 = " ";
            b111 += " ((u*w / (u*v + u*w + v*w)) * ( ((1-u) * w * b12x1x111 + u * (1 - w) * b12x2x111) / (w + u - 2 * u * w) ) + ";
            b111 += "  (u*v / (u*v + u*w + v*w)) * ( ((1-v) * u * b23x2x111 + v * (1 - u) * b23x3x111) / (u + v - 2 * u * v) ) + ";
            b111 += "  (v*w / (u*v + u*w + v*w)) * ( ((1-w) * v * b31x3x111 + w * (1 - v) * b31x1x111) / (w + v - 2 * v * w) )) ";

            string patch = String.Format(
                "(u^3)*{0} + (v^3)*{1} + (w^3)*{2} + (3*w*u^2)*{3} + (3*v*u^2)*{4} + (3*w*v^2)*{5} + (3*u*v^2)*{6} + (3*u*w^2)*{7} + (3*v*w^2)*{8} + (6*u*v*w)*{9}",
                b030,
                b003,
                b300,
                b120,
                b021,
                b102,
                b012,
                b210,
                b201,
                b111
                );

            Expr uExpr = Expr.Symbol("u");
            Expr vExpr = Expr.Symbol("v");
            Expr wExpr = Expr.Symbol("w");


            this.functionExpr = Infix.ParseOrThrow(patch);

            //first derivates
            this.duExpr = Calculus.Differentiate(uExpr, functionExpr);
            this.dvExpr = Calculus.Differentiate(vExpr, functionExpr);
            Expression dwExpr = Calculus.Differentiate(wExpr, functionExpr);

            String du = "(" + Infix.Print(this.duExpr) + ") - (" + Infix.Print(dwExpr) + ")";
            String dv = "(" + Infix.Print(this.dvExpr) + ") - (" + Infix.Print(dwExpr) + ")";

            this.duExpr = Infix.ParseOrThrow(du);
            this.dvExpr = Infix.ParseOrThrow(dv);

            GenerateSourceCode("Du", Infix.Print(this.duExpr));
            GenerateSourceCode("Dv", Infix.Print(this.dvExpr));

            
            //second derivates
            this.duuExpr = Calculus.Differentiate(uExpr, this.duExpr);
            this.duvExpr = Calculus.Differentiate(vExpr, this.duExpr);
            Expression duwExpr = Calculus.Differentiate(wExpr, this.duExpr);

            String duu = "(" + Infix.Print(this.duuExpr) + ") - (" + Infix.Print(duwExpr) + ")";
            String duv = "(" + Infix.Print(this.duvExpr) + ") - (" + Infix.Print(duwExpr) + ")";
            this.duuExpr = Infix.ParseOrThrow(duu);
            this.duvExpr = Infix.ParseOrThrow(duv);

            GenerateSourceCode("Duu", Infix.Print(this.duuExpr));
            GenerateSourceCode("Duv", Infix.Print(this.duvExpr));


            this.dvuExpr = Calculus.Differentiate(uExpr, this.dvExpr);
            this.dvvExpr = Calculus.Differentiate(vExpr, this.dvExpr);
            Expression dvwExpr = Calculus.Differentiate(wExpr, this.dvExpr);

            String dvu = "(" + Infix.Print(this.dvuExpr) + ") - (" + Infix.Print(dvwExpr) + ")";
            String dvv = "(" + Infix.Print(this.dvvExpr) + ") - (" + Infix.Print(dvwExpr) + ")";
            this.dvuExpr = Infix.ParseOrThrow(dvu);
            this.dvvExpr = Infix.ParseOrThrow(dvv);

            GenerateSourceCode("Dvu", Infix.Print(this.dvuExpr));
            GenerateSourceCode("Dvv", Infix.Print(this.dvvExpr));

            Console.WriteLine("Symbolic init");

            /*
            this.dwuExpr = Calculus.Differentiate(uExpr, this.dwExpr);
            this.dwvExpr = Calculus.Differentiate(vExpr, this.dwExpr);
            this.dwwExpr = Calculus.Differentiate(wExpr, this.dwExpr);

            String dwu = "(" + Infix.Print(this.dwuExpr) + ") - (" + Infix.Print(this.dwwExpr) + ")";
            String dwv = "(" + Infix.Print(this.dwvExpr) + ") - (" + Infix.Print(this.dwwExpr) + ")";
            this.dwuExpr = Infix.ParseOrThrow(dwu);
            this.dwvExpr = Infix.ParseOrThrow(dwv);
            */


        }

        //============================================================================
        //============================================================================
        //============================================================================

        
 
 
 
 
 
 /* [GENERATAED CODE] */
public double GetDvv(int triIndex, int index, double u, double v, double w){
double u2 = u * u;
double v2 = v * v;
double w2 = w * w;
double u12 = (1 - u) * (1 - u);
double v12 = (1 - v) * (1 - v);
double w12 = (1 - w) * (1 - w);
 EdgePatch e3 = edgePatches[3 * triIndex];
  EdgePatch e1 = edgePatches[3 * triIndex + 1];
  EdgePatch e2 = edgePatches[3 * triIndex + 2];
 

double b300 = e1.b300[index];
double b030 = e2.b300[index];
double b003 = e3.b300[index];
double b1x201 = e1.bp1_201[index];
double b2x201 = e1.bp2_201[index];
double b3x102 = e2.bp2_021[index];
double b2x102 = e2.bp1_021[index];
double b3x012 = e3.bp1_201[index];
double b1x012 = e3.bp2_201[index];
double b2x021 = e1.bp2_021[index];
double b1x021 = e1.bp1_021[index];
double b2x120 = e2.bp1_201[index];
double b3x120 = e2.bp2_201[index];
double b1x210 = e3.bp2_021[index];
double b3x210 = e3.bp1_021[index];
double b12x1x111 = e1.bp1_111[index];
double b12x2x111 = e1.bp2_111[index];
double b23x2x111 = e2.bp1_111[index];
double b23x3x111 = e2.bp2_111[index];
double b31x3x111 = e3.bp1_111[index];
double b31x1x111 = e3.bp2_111[index];


double res = 6*b003*v + (6*(b2x102*u2 + b3x102*u12)*w)/(u2 + u12) + (6*u*(b1x012*w2 + b3x012*w12))/(w2 + w12) + 3*u2*(-((2*v - 2*(1 - v))*(2*b3x120*v - 2*b2x120*(1 - v))*w)/(Math.Pow((v2 + v12), 2)) + ((2*b2x120 + 2*b3x120)*w)/(v2 + v12) - ((4*(b3x120*v2 + b2x120*v12)*w)/(Math.Pow((v2 + v12), 2)) + (2*v - 2*(1 - v))*(((2*b3x120*v - 2*b2x120*(1 - v))*w)/(Math.Pow((v2 + v12), 2)) - (2*(2*v - 2*(1 - v))*(b3x120*v2 + b2x120*v12)*w)/(Math.Pow((v2 + v12), 3))))) + 3*u*(-((2*v - 2*(1 - v))*(2*b3x210*v - 2*b1x210*(1 - v))*w2)/(Math.Pow((v2 + v12), 2)) + ((2*b1x210 + 2*b3x210)*w2)/(v2 + v12) - ((4*(b3x210*v2 + b1x210*v12)*w2)/(Math.Pow((v2 + v12), 2)) + (2*v - 2*(1 - v))*(((2*b3x210*v - 2*b1x210*(1 - v))*w2)/(Math.Pow((v2 + v12), 2)) - (2*(2*v - 2*(1 - v))*(b3x210*v2 + b1x210*v12)*w2)/(Math.Pow((v2 + v12), 3))))) + 6*u*(2*w*(-(u*w*(u + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((u + w - 2*u*w)*(Math.Pow((u*v + u*w + v*w), 2))) + (w*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((u*v + u*w + v*w)*(v + w - 2*v*w)) + v*w*(-((u + w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + ((-b31x1x111*w + b31x3x111*(1 - w))/(v + w - 2*v*w) - ((1 - 2*w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(u*v + u*w + v*w)) + u*((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))/((u + v - 2*u*v)*(u*v + u*w + v*w)) + v*(-((1 - 2*u)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((Math.Pow((u + v - 2*u*v), 2))*(u*v + u*w + v*w)) + (-((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(u + w))/(Math.Pow((u*v + u*w + v*w), 2)) + (-b23x2x111*u + b23x3x111*(1 - u))/(u*v + u*w + v*w))/(u + v - 2*u*v)))) + v*w*((2*u*w*(Math.Pow((u + w), 2))*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((u + w - 2*u*w)*(Math.Pow((u*v + u*w + v*w), 3))) + 2*w*(-((u + w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + ((-b31x1x111*w + b31x3x111*(1 - w))/(v + w - 2*v*w) - ((1 - 2*w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(u*v + u*w + v*w)) + v*w*(-((u + w)*((-b31x1x111*w + b31x3x111*(1 - w))/(v + w - 2*v*w) - ((1 - 2*w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2))))/(Math.Pow((u*v + u*w + v*w), 2)) + (-((1 - 2*w)*(-b31x1x111*w + b31x3x111*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)) - (1 - 2*w)*((-b31x1x111*w + b31x3x111*(1 - w))/(Math.Pow((v + w - 2*v*w), 2)) - (2*(1 - 2*w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 3))))/(u*v + u*w + v*w) - (u + w)*(-(2*(u + w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 3))*(v + w - 2*v*w)) + ((-b31x1x111*w + b31x3x111*(1 - w))/(v + w - 2*v*w) - ((1 - 2*w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(Math.Pow((u*v + u*w + v*w), 2)))) + u*(-(2*(1 - 2*u)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((Math.Pow((u + v - 2*u*v), 2))*(u*v + u*w + v*w)) + (2*(-((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(u + w))/(Math.Pow((u*v + u*w + v*w), 2)) + (-b23x2x111*u + b23x3x111*(1 - u))/(u*v + u*w + v*w)))/(u + v - 2*u*v) + v*(-((1 - 2*u)*(-((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(u + w))/(Math.Pow((u*v + u*w + v*w), 2)) + (-b23x2x111*u + b23x3x111*(1 - u))/(u*v + u*w + v*w)))/(Math.Pow((u + v - 2*u*v), 2)) + (-((-b23x2x111*u + b23x3x111*(1 - u))*(u + w))/(Math.Pow((u*v + u*w + v*w), 2)) - (-(2*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(Math.Pow((u + w), 2)))/(Math.Pow((u*v + u*w + v*w), 3)) + ((-b23x2x111*u + b23x3x111*(1 - u))*(u + w))/(Math.Pow((u*v + u*w + v*w), 2))))/(u + v - 2*u*v) - (1 - 2*u)*(-(2*(1 - 2*u)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((Math.Pow((u + v - 2*u*v), 3))*(u*v + u*w + v*w)) + (-((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(u + w))/(Math.Pow((u*v + u*w + v*w), 2)) + (-b23x2x111*u + b23x3x111*(1 - u))/(u*v + u*w + v*w))/(Math.Pow((u + v - 2*u*v), 2))))))) - ((6*(b2x102*u2 + b3x102*u12)*v)/(u2 + u12) + 3*u2*((2*b3x120*v - 2*b2x120*(1 - v))/(v2 + v12) - ((2*v - 2*(1 - v))*(b3x120*v2 + b2x120*v12))/(Math.Pow((v2 + v12), 2))) + (6*(b2x201*u2 + b1x201*u12)*w)/(u2 + u12) + 3*u*((2*(2*b3x210*v - 2*b1x210*(1 - v))*w)/(v2 + v12) - (2*(2*v - 2*(1 - v))*(b3x210*v2 + b1x210*v12)*w)/(Math.Pow((v2 + v12), 2))) + 3*u2*((2*b1x021*w - 2*b2x021*(1 - w))/(w2 + w12) - ((2*w - 2*(1 - w))*(b1x021*w2 + b2x021*w12))/(Math.Pow((w2 + w12), 2))) + 6*u*v*((2*b1x012*w - 2*b3x012*(1 - w))/(w2 + w12) - ((2*w - 2*(1 - w))*(b1x012*w2 + b3x012*w12))/(Math.Pow((w2 + w12), 2))) - ((6*(b2x201*u2 + b1x201*u12)*v)/(u2 + u12) + (6*u*(b3x210*v2 + b1x210*v12))/(v2 + v12) + 6*b300*w + 3*u2*v*(-((2*w - 2*(1 - w))*(2*b1x021*w - 2*b2x021*(1 - w)))/(Math.Pow((w2 + w12), 2)) + (2*b1x021 + 2*b2x021)/(w2 + w12) - ((4*(b1x021*w2 + b2x021*w12))/(Math.Pow((w2 + w12), 2)) + (2*w - 2*(1 - w))*((2*b1x021*w - 2*b2x021*(1 - w))/(Math.Pow((w2 + w12), 2)) - (2*(2*w - 2*(1 - w))*(b1x021*w2 + b2x021*w12))/(Math.Pow((w2 + w12), 3))))) + 3*u*v2*(-((2*w - 2*(1 - w))*(2*b1x012*w - 2*b3x012*(1 - w)))/(Math.Pow((w2 + w12), 2)) + (2*b1x012 + 2*b3x012)/(w2 + w12) - ((4*(b1x012*w2 + b3x012*w12))/(Math.Pow((w2 + w12), 2)) + (2*w - 2*(1 - w))*((2*b1x012*w - 2*b3x012*(1 - w))/(Math.Pow((w2 + w12), 2)) - (2*(2*w - 2*(1 - w))*(b1x012*w2 + b3x012*w12))/(Math.Pow((w2 + w12), 3))))) + 6*u*v*(-(2*u*v*(u + v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(Math.Pow((u*v + u*w + v*w), 2))) + 2*u*((b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w))/((u + w - 2*u*w)*(u*v + u*w + v*w)) + w*(-((1 - 2*u)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(u*v + u*w + v*w)) + ((-b12x2x111*u + b12x1x111*(1 - u))/(u*v + u*w + v*w) - ((u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)))/(u + w - 2*u*w))) + 2*v*((b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w))/((u*v + u*w + v*w)*(v + w - 2*v*w)) + w*(-((u + v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + ((-b31x3x111*v + b31x1x111*(1 - v))/(v + w - 2*v*w) - ((1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(u*v + u*w + v*w))) + w*((2*u*v*(Math.Pow((u + v), 2))*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(Math.Pow((u*v + u*w + v*w), 3))) + u*(-(2*(1 - 2*u)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(u*v + u*w + v*w)) + (2*((-b12x2x111*u + b12x1x111*(1 - u))/(u*v + u*w + v*w) - ((u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2))))/(u + w - 2*u*w) + w*(-((1 - 2*u)*((-b12x2x111*u + b12x1x111*(1 - u))/(u*v + u*w + v*w) - ((u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2))))/(Math.Pow((u + w - 2*u*w), 2)) + (-((-b12x2x111*u + b12x1x111*(1 - u))*(u + v))/(Math.Pow((u*v + u*w + v*w), 2)) - (u + v)*((-b12x2x111*u + b12x1x111*(1 - u))/(Math.Pow((u*v + u*w + v*w), 2)) - (2*(u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 3))))/(u + w - 2*u*w) - (1 - 2*u)*(-(2*(1 - 2*u)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 3))*(u*v + u*w + v*w)) + ((-b12x2x111*u + b12x1x111*(1 - u))/(u*v + u*w + v*w) - ((u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)))/(Math.Pow((u + w - 2*u*w), 2))))) + v*(-(2*(u + v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + (2*((-b31x3x111*v + b31x1x111*(1 - v))/(v + w - 2*v*w) - ((1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2))))/(u*v + u*w + v*w) + w*(-((u + v)*((-b31x3x111*v + b31x1x111*(1 - v))/(v + w - 2*v*w) - ((1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2))))/(Math.Pow((u*v + u*w + v*w), 2)) + (-((1 - 2*v)*(-b31x3x111*v + b31x1x111*(1 - v)))/(Math.Pow((v + w - 2*v*w), 2)) - (1 - 2*v)*((-b31x3x111*v + b31x1x111*(1 - v))/(Math.Pow((v + w - 2*v*w), 2)) - (2*(1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 3))))/(u*v + u*w + v*w) - (u + v)*(-(2*(u + v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 3))*(v + w - 2*v*w)) + ((-b31x3x111*v + b31x1x111*(1 - v))/(v + w - 2*v*w) - ((1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(Math.Pow((u*v + u*w + v*w), 2)))))))) + 6*u*((u*v*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(u*v + u*w + v*w)) + (u*w*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((u + w - 2*u*w)*(u*v + u*w + v*w)) + (v*w*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((u*v + u*w + v*w)*(v + w - 2*v*w)) + w*(-(u*v*(u + v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(Math.Pow((u*v + u*w + v*w), 2))) + u*((b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w))/((u + w - 2*u*w)*(u*v + u*w + v*w)) + w*(-((1 - 2*u)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(u*v + u*w + v*w)) + ((-b12x2x111*u + b12x1x111*(1 - u))/(u*v + u*w + v*w) - ((u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)))/(u + w - 2*u*w))) + v*((b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w))/((u*v + u*w + v*w)*(v + w - 2*v*w)) + w*(-((u + v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + ((-b31x3x111*v + b31x1x111*(1 - v))/(v + w - 2*v*w) - ((1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(u*v + u*w + v*w)))) + v*(-(u*w*(u + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((u + w - 2*u*w)*(Math.Pow((u*v + u*w + v*w), 2))) + (w*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((u*v + u*w + v*w)*(v + w - 2*v*w)) + v*w*(-((u + w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + ((-b31x1x111*w + b31x3x111*(1 - w))/(v + w - 2*v*w) - ((1 - 2*w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(u*v + u*w + v*w)) + u*((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))/((u + v - 2*u*v)*(u*v + u*w + v*w)) + v*(-((1 - 2*u)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((Math.Pow((u + v - 2*u*v), 2))*(u*v + u*w + v*w)) + (-((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(u + w))/(Math.Pow((u*v + u*w + v*w), 2)) + (-b23x2x111*u + b23x3x111*(1 - u))/(u*v + u*w + v*w))/(u + v - 2*u*v))) + w*((b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w))/((u*v + u*w + v*w)*(v + w - 2*v*w)) + w*(-((u + v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + ((-b31x3x111*v + b31x1x111*(1 - v))/(v + w - 2*v*w) - ((1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(u*v + u*w + v*w)) + u*(-((u + v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(Math.Pow((u*v + u*w + v*w), 2))) + v*(((1 - 2*u)*(u + v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((Math.Pow((u + v - 2*u*v), 2))*(Math.Pow((u*v + u*w + v*w), 2))) + (-((-b23x2x111*u + b23x3x111*(1 - u))*(u + v))/(Math.Pow((u*v + u*w + v*w), 2)) - (b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(-(2*(u + v)*(u + w))/(Math.Pow((u*v + u*w + v*w), 3)) + 1/(Math.Pow((u*v + u*w + v*w), 2))))/(u + v - 2*u*v))) - u*(((u + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((u + w - 2*u*w)*(Math.Pow((u*v + u*w + v*w), 2))) + w*((b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w))/((u + w - 2*u*w)*(Math.Pow((u*v + u*w + v*w), 2))) + (u + w)*(-((1 - 2*u)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(Math.Pow((u*v + u*w + v*w), 2))) + ((-b12x2x111*u + b12x1x111*(1 - u))/(Math.Pow((u*v + u*w + v*w), 2)) - (2*(u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 3)))/(u + w - 2*u*w)))) + v*(-((u + w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + ((-b31x1x111*w + b31x3x111*(1 - w))/(v + w - 2*v*w) - ((1 - 2*w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(u*v + u*w + v*w) + w*(-((u + v)*((-b31x1x111*w + b31x3x111*(1 - w))/(v + w - 2*v*w) - ((1 - 2*w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2))))/(Math.Pow((u*v + u*w + v*w), 2)) + ((-b31x1x111 - b31x3x111)/(v + w - 2*v*w) - ((1 - 2*v)*(-b31x1x111*w + b31x3x111*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)) - (-(2*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)) + (1 - 2*w)*((-b31x3x111*v + b31x1x111*(1 - v))/(Math.Pow((v + w - 2*v*w), 2)) - (2*(1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 3)))))/(u*v + u*w + v*w) - ((b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + (u + w)*(-(2*(u + v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 3))*(v + w - 2*v*w)) + ((-b31x3x111*v + b31x1x111*(1 - v))/(v + w - 2*v*w) - ((1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(Math.Pow((u*v + u*w + v*w), 2)))))))))) - ((6*(b2x102*u2 + b3x102*u12)*v)/(u2 + u12) + 3*u2*((2*b3x120*v - 2*b2x120*(1 - v))/(v2 + v12) - ((2*v - 2*(1 - v))*(b3x120*v2 + b2x120*v12))/(Math.Pow((v2 + v12), 2))) + (6*(b2x201*u2 + b1x201*u12)*w)/(u2 + u12) + 6*u*(((2*b3x210*v - 2*b1x210*(1 - v))*w)/(v2 + v12) - ((2*v - 2*(1 - v))*(b3x210*v2 + b1x210*v12)*w)/(Math.Pow((v2 + v12), 2))) + 3*u2*((2*b1x021*w - 2*b2x021*(1 - w))/(w2 + w12) - ((2*w - 2*(1 - w))*(b1x021*w2 + b2x021*w12))/(Math.Pow((w2 + w12), 2))) + 6*u*v*((2*b1x012*w - 2*b3x012*(1 - w))/(w2 + w12) - ((2*w - 2*(1 - w))*(b1x012*w2 + b3x012*w12))/(Math.Pow((w2 + w12), 2))) + 6*u*((u*v*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(u*v + u*w + v*w)) + (u*w*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((u + w - 2*u*w)*(u*v + u*w + v*w)) + (v*w*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((u*v + u*w + v*w)*(v + w - 2*v*w)) + w*(-(u*v*(u + v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(Math.Pow((u*v + u*w + v*w), 2))) + u*((b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w))/((u + w - 2*u*w)*(u*v + u*w + v*w)) + w*(-((1 - 2*u)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(u*v + u*w + v*w)) + ((-b12x2x111*u + b12x1x111*(1 - u))/(u*v + u*w + v*w) - ((u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)))/(u + w - 2*u*w))) + v*((b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w))/((u*v + u*w + v*w)*(v + w - 2*v*w)) + w*(-((u + v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + ((-b31x3x111*v + b31x1x111*(1 - v))/(v + w - 2*v*w) - ((1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(u*v + u*w + v*w)))) + v*(-(u*w*(u + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((u + w - 2*u*w)*(Math.Pow((u*v + u*w + v*w), 2))) + (w*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((u*v + u*w + v*w)*(v + w - 2*v*w)) + v*w*(-((u + w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + ((-b31x1x111*w + b31x3x111*(1 - w))/(v + w - 2*v*w) - ((1 - 2*w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(u*v + u*w + v*w)) + u*((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))/((u + v - 2*u*v)*(u*v + u*w + v*w)) + v*(-((1 - 2*u)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((Math.Pow((u + v - 2*u*v), 2))*(u*v + u*w + v*w)) + (-((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(u + w))/(Math.Pow((u*v + u*w + v*w), 2)) + (-b23x2x111*u + b23x3x111*(1 - u))/(u*v + u*w + v*w))/(u + v - 2*u*v))) + w*((b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w))/((u*v + u*w + v*w)*(v + w - 2*v*w)) + w*(-((u + v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + ((-b31x3x111*v + b31x1x111*(1 - v))/(v + w - 2*v*w) - ((1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(u*v + u*w + v*w)) - u*(((u + v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(Math.Pow((u*v + u*w + v*w), 2))) + v*((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))/((u + v - 2*u*v)*(Math.Pow((u*v + u*w + v*w), 2))) + (u + v)*(-((1 - 2*u)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((Math.Pow((u + v - 2*u*v), 2))*(Math.Pow((u*v + u*w + v*w), 2))) + (-(2*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(u + w))/(Math.Pow((u*v + u*w + v*w), 3)) + (-b23x2x111*u + b23x3x111*(1 - u))/(Math.Pow((u*v + u*w + v*w), 2)))/(u + v - 2*u*v)))) + u*(-((u + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((u + w - 2*u*w)*(Math.Pow((u*v + u*w + v*w), 2))) + w*(((1 - 2*u)*(u + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(Math.Pow((u*v + u*w + v*w), 2))) + (-((-b12x2x111*u + b12x1x111*(1 - u))*(u + w))/(Math.Pow((u*v + u*w + v*w), 2)) - (-(2*(u + v)*(u + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 3)) + (b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w))/(Math.Pow((u*v + u*w + v*w), 2))))/(u + w - 2*u*w))) + v*(-((u + w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + ((-b31x1x111*w + b31x3x111*(1 - w))/(v + w - 2*v*w) - ((1 - 2*w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(u*v + u*w + v*w) + w*(-((u + w)*((-b31x3x111*v + b31x1x111*(1 - v))/(v + w - 2*v*w) - ((1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2))))/(Math.Pow((u*v + u*w + v*w), 2)) + (-((-b31x3x111*v + b31x1x111*(1 - v))*(1 - 2*w))/(Math.Pow((v + w - 2*v*w), 2)) + (-b31x1x111 - b31x3x111)/(v + w - 2*v*w) - (-(2*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)) + (1 - 2*v)*((-b31x1x111*w + b31x3x111*(1 - w))/(Math.Pow((v + w - 2*v*w), 2)) - (2*(1 - 2*w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 3)))))/(u*v + u*w + v*w) - ((b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + (u + v)*(-(2*(u + w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 3))*(v + w - 2*v*w)) + ((-b31x1x111*w + b31x3x111*(1 - w))/(v + w - 2*v*w) - ((1 - 2*w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(Math.Pow((u*v + u*w + v*w), 2))))))))));


return res;
}

public double GetDvu(int triIndex, int index, double u, double v, double w){
double u2 = u * u;
double v2 = v * v;
double w2 = w * w;
double u12 = (1 - u) * (1 - u);
double v12 = (1 - v) * (1 - v);
double w12 = (1 - w) * (1 - w);
 EdgePatch e3 = edgePatches[3 * triIndex];
  EdgePatch e1 = edgePatches[3 * triIndex + 1];
  EdgePatch e2 = edgePatches[3 * triIndex + 2];
 

double b300 = e1.b300[index];
double b030 = e2.b300[index];
double b003 = e3.b300[index];
double b1x201 = e1.bp1_201[index];
double b2x201 = e1.bp2_201[index];
double b3x102 = e2.bp2_021[index];
double b2x102 = e2.bp1_021[index];
double b3x012 = e3.bp1_201[index];
double b1x012 = e3.bp2_201[index];
double b2x021 = e1.bp2_021[index];
double b1x021 = e1.bp1_021[index];
double b2x120 = e2.bp1_201[index];
double b3x120 = e2.bp2_201[index];
double b1x210 = e3.bp2_021[index];
double b3x210 = e3.bp1_021[index];
double b12x1x111 = e1.bp1_111[index];
double b12x2x111 = e1.bp2_111[index];
double b23x2x111 = e2.bp1_111[index];
double b23x3x111 = e2.bp2_111[index];
double b31x3x111 = e3.bp1_111[index];
double b31x1x111 = e3.bp2_111[index];


double res = 6*(((2*b2x102*u - 2*b3x102*(1 - u))*v*w)/(u2 + u12) - ((2*u - 2*(1 - u))*(b2x102*u2 + b3x102*u12)*v*w)/(Math.Pow((u2 + u12), 2))) + 6*u*(((2*b3x120*v - 2*b2x120*(1 - v))*w)/(v2 + v12) - ((2*v - 2*(1 - v))*(b3x120*v2 + b2x120*v12)*w)/(Math.Pow((v2 + v12), 2))) + 3*(((2*b2x201*u - 2*b1x201*(1 - u))*w2)/(u2 + u12) - ((2*u - 2*(1 - u))*(b2x201*u2 + b1x201*u12)*w2)/(Math.Pow((u2 + u12), 2))) + 3*(((2*b3x210*v - 2*b1x210*(1 - v))*w2)/(v2 + v12) - ((2*v - 2*(1 - v))*(b3x210*v2 + b1x210*v12)*w2)/(Math.Pow((v2 + v12), 2))) + (6*u*(b1x021*w2 + b2x021*w12))/(w2 + w12) + (6*v*(b1x012*w2 + b3x012*w12))/(w2 + w12) + 6*(w*((u*v*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(u*v + u*w + v*w)) + (u*w*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((u + w - 2*u*w)*(u*v + u*w + v*w)) + (v*w*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((u*v + u*w + v*w)*(v + w - 2*v*w))) + v*w*(-(u*w*(u + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((u + w - 2*u*w)*(Math.Pow((u*v + u*w + v*w), 2))) + (w*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((u*v + u*w + v*w)*(v + w - 2*v*w)) + v*w*(-((u + w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + ((-b31x1x111*w + b31x3x111*(1 - w))/(v + w - 2*v*w) - ((1 - 2*w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(u*v + u*w + v*w)) + u*((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))/((u + v - 2*u*v)*(u*v + u*w + v*w)) + v*(-((1 - 2*u)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((Math.Pow((u + v - 2*u*v), 2))*(u*v + u*w + v*w)) + (-((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(u + w))/(Math.Pow((u*v + u*w + v*w), 2)) + (-b23x2x111*u + b23x3x111*(1 - u))/(u*v + u*w + v*w))/(u + v - 2*u*v)))) + u*(w*((v*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(u*v + u*w + v*w)) + (w*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((u + w - 2*u*w)*(u*v + u*w + v*w)) - (v*w*(v + w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + u*v*(-((1 - 2*v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((Math.Pow((u + v - 2*u*v), 2))*(u*v + u*w + v*w)) + (-((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(v + w))/(Math.Pow((u*v + u*w + v*w), 2)) + (-b23x3x111*v + b23x2x111*(1 - v))/(u*v + u*w + v*w))/(u + v - 2*u*v)) + u*w*(-((1 - 2*w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(u*v + u*w + v*w)) + ((-b12x1x111*w + b12x2x111*(1 - w))/(u*v + u*w + v*w) - ((v + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)))/(u + w - 2*u*w))) + v*w*((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))/((u + v - 2*u*v)*(u*v + u*w + v*w)) - (w*(v + w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + v*(-((1 - 2*u)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((Math.Pow((u + v - 2*u*v), 2))*(u*v + u*w + v*w)) + (-((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(u + w))/(Math.Pow((u*v + u*w + v*w), 2)) + (-b23x2x111*u + b23x3x111*(1 - u))/(u*v + u*w + v*w))/(u + v - 2*u*v)) + v*w*(-((v + w)*((-b31x1x111*w + b31x3x111*(1 - w))/(v + w - 2*v*w) - ((1 - 2*w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2))))/(Math.Pow((u*v + u*w + v*w), 2)) - (-(2*(u + w)*(v + w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 3))*(v + w - 2*v*w)) + (b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)))) - ((w*(u + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((u + w - 2*u*w)*(Math.Pow((u*v + u*w + v*w), 2))) + u*w*((b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w))/((u + w - 2*u*w)*(Math.Pow((u*v + u*w + v*w), 2))) + (u + w)*(-((1 - 2*w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(Math.Pow((u*v + u*w + v*w), 2))) + ((-b12x1x111*w + b12x2x111*(1 - w))/(Math.Pow((u*v + u*w + v*w), 2)) - (2*(v + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 3)))/(u + w - 2*u*w)))) + u*(-((1 - 2*v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((Math.Pow((u + v - 2*u*v), 2))*(u*v + u*w + v*w)) + (-((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(v + w))/(Math.Pow((u*v + u*w + v*w), 2)) + (-b23x3x111*v + b23x2x111*(1 - v))/(u*v + u*w + v*w))/(u + v - 2*u*v) + v*(-((1 - 2*v)*(-((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(u + w))/(Math.Pow((u*v + u*w + v*w), 2)) + (-b23x2x111*u + b23x3x111*(1 - u))/(u*v + u*w + v*w)))/(Math.Pow((u + v - 2*u*v), 2)) + (-((-b23x2x111*u + b23x3x111*(1 - u))*(v + w))/(Math.Pow((u*v + u*w + v*w), 2)) + (-b23x2x111 - b23x3x111)/(u*v + u*w + v*w) - (((-b23x3x111*v + b23x2x111*(1 - v))*(u + w))/(Math.Pow((u*v + u*w + v*w), 2)) + (b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(-(2*(u + w)*(v + w))/(Math.Pow((u*v + u*w + v*w), 3)) + 1/(Math.Pow((u*v + u*w + v*w), 2)))))/(u + v - 2*u*v) - (-(2*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((Math.Pow((u + v - 2*u*v), 2))*(u*v + u*w + v*w)) + (1 - 2*u)*(-(2*(1 - 2*v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((Math.Pow((u + v - 2*u*v), 3))*(u*v + u*w + v*w)) + (-((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(v + w))/(Math.Pow((u*v + u*w + v*w), 2)) + (-b23x3x111*v + b23x2x111*(1 - v))/(u*v + u*w + v*w))/(Math.Pow((u + v - 2*u*v), 2))))))))) - (3*(((2*b2x102*u - 2*b3x102*(1 - u))*v2)/(u2 + u12) - ((2*u - 2*(1 - u))*(b2x102*u2 + b3x102*u12)*v2)/(Math.Pow((u2 + u12), 2))) + (6*u*(b3x120*v2 + b2x120*v12))/(v2 + v12) + (6*(b3x210*v2 + b1x210*v12)*w)/(v2 + v12) + 6*(((2*b2x201*u - 2*b1x201*(1 - u))*v*w)/(u2 + u12) - ((2*u - 2*(1 - u))*(b2x201*u2 + b1x201*u12)*v*w)/(Math.Pow((u2 + u12), 2))) + 6*u*v*((2*b1x021*w - 2*b2x021*(1 - w))/(w2 + w12) - ((2*w - 2*(1 - w))*(b1x021*w2 + b2x021*w12))/(Math.Pow((w2 + w12), 2))) + 3*v2*((2*b1x012*w - 2*b3x012*(1 - w))/(w2 + w12) - ((2*w - 2*(1 - w))*(b1x012*w2 + b3x012*w12))/(Math.Pow((w2 + w12), 2))) + 6*(v*((u*v*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(u*v + u*w + v*w)) + (u*w*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((u + w - 2*u*w)*(u*v + u*w + v*w)) + (v*w*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((u*v + u*w + v*w)*(v + w - 2*v*w)) + w*(-(u*v*(u + v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(Math.Pow((u*v + u*w + v*w), 2))) + u*((b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w))/((u + w - 2*u*w)*(u*v + u*w + v*w)) + w*(-((1 - 2*u)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(u*v + u*w + v*w)) + ((-b12x2x111*u + b12x1x111*(1 - u))/(u*v + u*w + v*w) - ((u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)))/(u + w - 2*u*w))) + v*((b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w))/((u*v + u*w + v*w)*(v + w - 2*v*w)) + w*(-((u + v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + ((-b31x3x111*v + b31x1x111*(1 - v))/(v + w - 2*v*w) - ((1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(u*v + u*w + v*w))))) + u*v*((v*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(u*v + u*w + v*w)) + (w*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((u + w - 2*u*w)*(u*v + u*w + v*w)) - (v*w*(v + w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + u*v*(-((1 - 2*v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((Math.Pow((u + v - 2*u*v), 2))*(u*v + u*w + v*w)) + (-((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(v + w))/(Math.Pow((u*v + u*w + v*w), 2)) + (-b23x3x111*v + b23x2x111*(1 - v))/(u*v + u*w + v*w))/(u + v - 2*u*v)) + u*w*(-((1 - 2*w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(u*v + u*w + v*w)) + ((-b12x1x111*w + b12x2x111*(1 - w))/(u*v + u*w + v*w) - ((v + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)))/(u + w - 2*u*w)) + w*((b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w))/((u + w - 2*u*w)*(u*v + u*w + v*w)) + w*(-((1 - 2*u)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(u*v + u*w + v*w)) + ((-b12x2x111*u + b12x1x111*(1 - u))/(u*v + u*w + v*w) - ((u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)))/(u + w - 2*u*w)) + v*(-((v + w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + w*(-((v + w)*((-b31x3x111*v + b31x1x111*(1 - v))/(v + w - 2*v*w) - ((1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2))))/(Math.Pow((u*v + u*w + v*w), 2)) - (-(2*(u + v)*(v + w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 3))*(v + w - 2*v*w)) + (b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w))))) - ((v*(u + v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(Math.Pow((u*v + u*w + v*w), 2))) + u*v*((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))/((u + v - 2*u*v)*(Math.Pow((u*v + u*w + v*w), 2))) + (u + v)*(-((1 - 2*v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((Math.Pow((u + v - 2*u*v), 2))*(Math.Pow((u*v + u*w + v*w), 2))) + (-(2*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(v + w))/(Math.Pow((u*v + u*w + v*w), 3)) + (-b23x3x111*v + b23x2x111*(1 - v))/(Math.Pow((u*v + u*w + v*w), 2)))/(u + v - 2*u*v)))) + u*(-((1 - 2*w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(u*v + u*w + v*w)) + ((-b12x1x111*w + b12x2x111*(1 - w))/(u*v + u*w + v*w) - ((v + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)))/(u + w - 2*u*w) + w*(-((1 - 2*w)*((-b12x2x111*u + b12x1x111*(1 - u))/(u*v + u*w + v*w) - ((u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2))))/(Math.Pow((u + w - 2*u*w), 2)) + (-((-b12x2x111*u + b12x1x111*(1 - u))*(v + w))/(Math.Pow((u*v + u*w + v*w), 2)) + (-b12x1x111 - b12x2x111)/(u*v + u*w + v*w) - ((b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w))/(Math.Pow((u*v + u*w + v*w), 2)) + (u + v)*((-b12x1x111*w + b12x2x111*(1 - w))/(Math.Pow((u*v + u*w + v*w), 2)) - (2*(v + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 3)))))/(u + w - 2*u*w) - (-(2*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(u*v + u*w + v*w)) + (1 - 2*u)*(-(2*(1 - 2*w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 3))*(u*v + u*w + v*w)) + ((-b12x1x111*w + b12x2x111*(1 - w))/(u*v + u*w + v*w) - ((v + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)))/(Math.Pow((u + w - 2*u*w), 2)))))))))) - ((6*(b2x102*u2 + b3x102*u12)*v)/(u2 + u12) + 3*u2*((2*b3x120*v - 2*b2x120*(1 - v))/(v2 + v12) - ((2*v - 2*(1 - v))*(b3x120*v2 + b2x120*v12))/(Math.Pow((v2 + v12), 2))) + (6*(b2x201*u2 + b1x201*u12)*w)/(u2 + u12) + 3*u*((2*(2*b3x210*v - 2*b1x210*(1 - v))*w)/(v2 + v12) - (2*(2*v - 2*(1 - v))*(b3x210*v2 + b1x210*v12)*w)/(Math.Pow((v2 + v12), 2))) + 3*u2*((2*b1x021*w - 2*b2x021*(1 - w))/(w2 + w12) - ((2*w - 2*(1 - w))*(b1x021*w2 + b2x021*w12))/(Math.Pow((w2 + w12), 2))) + 6*u*v*((2*b1x012*w - 2*b3x012*(1 - w))/(w2 + w12) - ((2*w - 2*(1 - w))*(b1x012*w2 + b3x012*w12))/(Math.Pow((w2 + w12), 2))) - ((6*(b2x201*u2 + b1x201*u12)*v)/(u2 + u12) + (6*u*(b3x210*v2 + b1x210*v12))/(v2 + v12) + 6*b300*w + 3*u2*v*(-((2*w - 2*(1 - w))*(2*b1x021*w - 2*b2x021*(1 - w)))/(Math.Pow((w2 + w12), 2)) + (2*b1x021 + 2*b2x021)/(w2 + w12) - ((4*(b1x021*w2 + b2x021*w12))/(Math.Pow((w2 + w12), 2)) + (2*w - 2*(1 - w))*((2*b1x021*w - 2*b2x021*(1 - w))/(Math.Pow((w2 + w12), 2)) - (2*(2*w - 2*(1 - w))*(b1x021*w2 + b2x021*w12))/(Math.Pow((w2 + w12), 3))))) + 3*u*v2*(-((2*w - 2*(1 - w))*(2*b1x012*w - 2*b3x012*(1 - w)))/(Math.Pow((w2 + w12), 2)) + (2*b1x012 + 2*b3x012)/(w2 + w12) - ((4*(b1x012*w2 + b3x012*w12))/(Math.Pow((w2 + w12), 2)) + (2*w - 2*(1 - w))*((2*b1x012*w - 2*b3x012*(1 - w))/(Math.Pow((w2 + w12), 2)) - (2*(2*w - 2*(1 - w))*(b1x012*w2 + b3x012*w12))/(Math.Pow((w2 + w12), 3))))) + 6*u*v*(-(2*u*v*(u + v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(Math.Pow((u*v + u*w + v*w), 2))) + 2*u*((b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w))/((u + w - 2*u*w)*(u*v + u*w + v*w)) + w*(-((1 - 2*u)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(u*v + u*w + v*w)) + ((-b12x2x111*u + b12x1x111*(1 - u))/(u*v + u*w + v*w) - ((u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)))/(u + w - 2*u*w))) + 2*v*((b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w))/((u*v + u*w + v*w)*(v + w - 2*v*w)) + w*(-((u + v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + ((-b31x3x111*v + b31x1x111*(1 - v))/(v + w - 2*v*w) - ((1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(u*v + u*w + v*w))) + w*((2*u*v*(Math.Pow((u + v), 2))*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(Math.Pow((u*v + u*w + v*w), 3))) + u*(-(2*(1 - 2*u)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(u*v + u*w + v*w)) + (2*((-b12x2x111*u + b12x1x111*(1 - u))/(u*v + u*w + v*w) - ((u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2))))/(u + w - 2*u*w) + w*(-((1 - 2*u)*((-b12x2x111*u + b12x1x111*(1 - u))/(u*v + u*w + v*w) - ((u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2))))/(Math.Pow((u + w - 2*u*w), 2)) + (-((-b12x2x111*u + b12x1x111*(1 - u))*(u + v))/(Math.Pow((u*v + u*w + v*w), 2)) - (u + v)*((-b12x2x111*u + b12x1x111*(1 - u))/(Math.Pow((u*v + u*w + v*w), 2)) - (2*(u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 3))))/(u + w - 2*u*w) - (1 - 2*u)*(-(2*(1 - 2*u)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 3))*(u*v + u*w + v*w)) + ((-b12x2x111*u + b12x1x111*(1 - u))/(u*v + u*w + v*w) - ((u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)))/(Math.Pow((u + w - 2*u*w), 2))))) + v*(-(2*(u + v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + (2*((-b31x3x111*v + b31x1x111*(1 - v))/(v + w - 2*v*w) - ((1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2))))/(u*v + u*w + v*w) + w*(-((u + v)*((-b31x3x111*v + b31x1x111*(1 - v))/(v + w - 2*v*w) - ((1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2))))/(Math.Pow((u*v + u*w + v*w), 2)) + (-((1 - 2*v)*(-b31x3x111*v + b31x1x111*(1 - v)))/(Math.Pow((v + w - 2*v*w), 2)) - (1 - 2*v)*((-b31x3x111*v + b31x1x111*(1 - v))/(Math.Pow((v + w - 2*v*w), 2)) - (2*(1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 3))))/(u*v + u*w + v*w) - (u + v)*(-(2*(u + v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 3))*(v + w - 2*v*w)) + ((-b31x3x111*v + b31x1x111*(1 - v))/(v + w - 2*v*w) - ((1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(Math.Pow((u*v + u*w + v*w), 2)))))))) + 6*u*((u*v*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(u*v + u*w + v*w)) + (u*w*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((u + w - 2*u*w)*(u*v + u*w + v*w)) + (v*w*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((u*v + u*w + v*w)*(v + w - 2*v*w)) + w*(-(u*v*(u + v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(Math.Pow((u*v + u*w + v*w), 2))) + u*((b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w))/((u + w - 2*u*w)*(u*v + u*w + v*w)) + w*(-((1 - 2*u)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(u*v + u*w + v*w)) + ((-b12x2x111*u + b12x1x111*(1 - u))/(u*v + u*w + v*w) - ((u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)))/(u + w - 2*u*w))) + v*((b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w))/((u*v + u*w + v*w)*(v + w - 2*v*w)) + w*(-((u + v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + ((-b31x3x111*v + b31x1x111*(1 - v))/(v + w - 2*v*w) - ((1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(u*v + u*w + v*w)))) + v*(-(u*w*(u + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((u + w - 2*u*w)*(Math.Pow((u*v + u*w + v*w), 2))) + (w*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((u*v + u*w + v*w)*(v + w - 2*v*w)) + v*w*(-((u + w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + ((-b31x1x111*w + b31x3x111*(1 - w))/(v + w - 2*v*w) - ((1 - 2*w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(u*v + u*w + v*w)) + u*((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))/((u + v - 2*u*v)*(u*v + u*w + v*w)) + v*(-((1 - 2*u)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((Math.Pow((u + v - 2*u*v), 2))*(u*v + u*w + v*w)) + (-((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(u + w))/(Math.Pow((u*v + u*w + v*w), 2)) + (-b23x2x111*u + b23x3x111*(1 - u))/(u*v + u*w + v*w))/(u + v - 2*u*v))) + w*((b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w))/((u*v + u*w + v*w)*(v + w - 2*v*w)) + w*(-((u + v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + ((-b31x3x111*v + b31x1x111*(1 - v))/(v + w - 2*v*w) - ((1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(u*v + u*w + v*w)) + u*(-((u + v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(Math.Pow((u*v + u*w + v*w), 2))) + v*(((1 - 2*u)*(u + v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((Math.Pow((u + v - 2*u*v), 2))*(Math.Pow((u*v + u*w + v*w), 2))) + (-((-b23x2x111*u + b23x3x111*(1 - u))*(u + v))/(Math.Pow((u*v + u*w + v*w), 2)) - (b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(-(2*(u + v)*(u + w))/(Math.Pow((u*v + u*w + v*w), 3)) + 1/(Math.Pow((u*v + u*w + v*w), 2))))/(u + v - 2*u*v))) - u*(((u + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((u + w - 2*u*w)*(Math.Pow((u*v + u*w + v*w), 2))) + w*((b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w))/((u + w - 2*u*w)*(Math.Pow((u*v + u*w + v*w), 2))) + (u + w)*(-((1 - 2*u)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(Math.Pow((u*v + u*w + v*w), 2))) + ((-b12x2x111*u + b12x1x111*(1 - u))/(Math.Pow((u*v + u*w + v*w), 2)) - (2*(u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 3)))/(u + w - 2*u*w)))) + v*(-((u + w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + ((-b31x1x111*w + b31x3x111*(1 - w))/(v + w - 2*v*w) - ((1 - 2*w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(u*v + u*w + v*w) + w*(-((u + v)*((-b31x1x111*w + b31x3x111*(1 - w))/(v + w - 2*v*w) - ((1 - 2*w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2))))/(Math.Pow((u*v + u*w + v*w), 2)) + ((-b31x1x111 - b31x3x111)/(v + w - 2*v*w) - ((1 - 2*v)*(-b31x1x111*w + b31x3x111*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)) - (-(2*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)) + (1 - 2*w)*((-b31x3x111*v + b31x1x111*(1 - v))/(Math.Pow((v + w - 2*v*w), 2)) - (2*(1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 3)))))/(u*v + u*w + v*w) - ((b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + (u + w)*(-(2*(u + v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 3))*(v + w - 2*v*w)) + ((-b31x3x111*v + b31x1x111*(1 - v))/(v + w - 2*v*w) - ((1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(Math.Pow((u*v + u*w + v*w), 2))))))))));


return res;
}

public double GetDuv(int triIndex, int index, double u, double v, double w){
double u2 = u * u;
double v2 = v * v;
double w2 = w * w;
double u12 = (1 - u) * (1 - u);
double v12 = (1 - v) * (1 - v);
double w12 = (1 - w) * (1 - w);
 EdgePatch e3 = edgePatches[3 * triIndex];
  EdgePatch e1 = edgePatches[3 * triIndex + 1];
  EdgePatch e2 = edgePatches[3 * triIndex + 2];
 

double b300 = e1.b300[index];
double b030 = e2.b300[index];
double b003 = e3.b300[index];
double b1x201 = e1.bp1_201[index];
double b2x201 = e1.bp2_201[index];
double b3x102 = e2.bp2_021[index];
double b2x102 = e2.bp1_021[index];
double b3x012 = e3.bp1_201[index];
double b1x012 = e3.bp2_201[index];
double b2x021 = e1.bp2_021[index];
double b1x021 = e1.bp1_021[index];
double b2x120 = e2.bp1_201[index];
double b3x120 = e2.bp2_201[index];
double b1x210 = e3.bp2_021[index];
double b3x210 = e3.bp1_021[index];
double b12x1x111 = e1.bp1_111[index];
double b12x2x111 = e1.bp2_111[index];
double b23x2x111 = e2.bp1_111[index];
double b23x3x111 = e2.bp2_111[index];
double b31x3x111 = e3.bp1_111[index];
double b31x1x111 = e3.bp2_111[index];


double res = 3*((2*(2*b2x102*u - 2*b3x102*(1 - u))*v*w)/(u2 + u12) - (2*(2*u - 2*(1 - u))*(b2x102*u2 + b3x102*u12)*v*w)/(Math.Pow((u2 + u12), 2))) + 6*u*(((2*b3x120*v - 2*b2x120*(1 - v))*w)/(v2 + v12) - ((2*v - 2*(1 - v))*(b3x120*v2 + b2x120*v12)*w)/(Math.Pow((v2 + v12), 2))) + 3*(((2*b2x201*u - 2*b1x201*(1 - u))*w2)/(u2 + u12) - ((2*u - 2*(1 - u))*(b2x201*u2 + b1x201*u12)*w2)/(Math.Pow((u2 + u12), 2))) + 3*(((2*b3x210*v - 2*b1x210*(1 - v))*w2)/(v2 + v12) - ((2*v - 2*(1 - v))*(b3x210*v2 + b1x210*v12)*w2)/(Math.Pow((v2 + v12), 2))) + (6*u*(b1x021*w2 + b2x021*w12))/(w2 + w12) + (6*v*(b1x012*w2 + b3x012*w12))/(w2 + w12) + 6*(w*((u*v*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(u*v + u*w + v*w)) + (u*w*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((u + w - 2*u*w)*(u*v + u*w + v*w)) + (v*w*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((u*v + u*w + v*w)*(v + w - 2*v*w))) + v*w*(-(u*w*(u + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((u + w - 2*u*w)*(Math.Pow((u*v + u*w + v*w), 2))) + (w*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((u*v + u*w + v*w)*(v + w - 2*v*w)) + v*w*(-((u + w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + ((-b31x1x111*w + b31x3x111*(1 - w))/(v + w - 2*v*w) - ((1 - 2*w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(u*v + u*w + v*w)) + u*((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))/((u + v - 2*u*v)*(u*v + u*w + v*w)) + v*(-((1 - 2*u)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((Math.Pow((u + v - 2*u*v), 2))*(u*v + u*w + v*w)) + (-((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(u + w))/(Math.Pow((u*v + u*w + v*w), 2)) + (-b23x2x111*u + b23x3x111*(1 - u))/(u*v + u*w + v*w))/(u + v - 2*u*v)))) + u*(w*((v*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(u*v + u*w + v*w)) + (w*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((u + w - 2*u*w)*(u*v + u*w + v*w)) - (v*w*(v + w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + u*v*(-((1 - 2*v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((Math.Pow((u + v - 2*u*v), 2))*(u*v + u*w + v*w)) + (-((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(v + w))/(Math.Pow((u*v + u*w + v*w), 2)) + (-b23x3x111*v + b23x2x111*(1 - v))/(u*v + u*w + v*w))/(u + v - 2*u*v)) + u*w*(-((1 - 2*w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(u*v + u*w + v*w)) + ((-b12x1x111*w + b12x2x111*(1 - w))/(u*v + u*w + v*w) - ((v + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)))/(u + w - 2*u*w))) + v*w*((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))/((u + v - 2*u*v)*(u*v + u*w + v*w)) - (w*(u + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((u + w - 2*u*w)*(Math.Pow((u*v + u*w + v*w), 2))) + v*(-((1 - 2*u)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((Math.Pow((u + v - 2*u*v), 2))*(u*v + u*w + v*w)) + (-((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(u + w))/(Math.Pow((u*v + u*w + v*w), 2)) + (-b23x2x111*u + b23x3x111*(1 - u))/(u*v + u*w + v*w))/(u + v - 2*u*v)) + u*w*(((u + w)*(1 - 2*w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(Math.Pow((u*v + u*w + v*w), 2))) + (-((u + w)*(-b12x1x111*w + b12x2x111*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)) - (-(2*(u + w)*(v + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 3)) + (b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w))/(Math.Pow((u*v + u*w + v*w), 2))))/(u + w - 2*u*w)) - ((w*(v + w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + v*w*((b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + (v + w)*(-(2*(u + w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 3))*(v + w - 2*v*w)) + ((-b31x1x111*w + b31x3x111*(1 - w))/(v + w - 2*v*w) - ((1 - 2*w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(Math.Pow((u*v + u*w + v*w), 2))))) + u*(-((1 - 2*v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((Math.Pow((u + v - 2*u*v), 2))*(u*v + u*w + v*w)) + (-((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(v + w))/(Math.Pow((u*v + u*w + v*w), 2)) + (-b23x3x111*v + b23x2x111*(1 - v))/(u*v + u*w + v*w))/(u + v - 2*u*v) + v*(-((1 - 2*u)*(-((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(v + w))/(Math.Pow((u*v + u*w + v*w), 2)) + (-b23x3x111*v + b23x2x111*(1 - v))/(u*v + u*w + v*w)))/(Math.Pow((u + v - 2*u*v), 2)) + (-((-b23x3x111*v + b23x2x111*(1 - v))*(u + w))/(Math.Pow((u*v + u*w + v*w), 2)) + (-b23x2x111 - b23x3x111)/(u*v + u*w + v*w) - (((-b23x2x111*u + b23x3x111*(1 - u))*(v + w))/(Math.Pow((u*v + u*w + v*w), 2)) + (b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(-(2*(u + w)*(v + w))/(Math.Pow((u*v + u*w + v*w), 3)) + 1/(Math.Pow((u*v + u*w + v*w), 2)))))/(u + v - 2*u*v) - (-(2*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((Math.Pow((u + v - 2*u*v), 2))*(u*v + u*w + v*w)) + (1 - 2*v)*(-(2*(1 - 2*u)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((Math.Pow((u + v - 2*u*v), 3))*(u*v + u*w + v*w)) + (-((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(u + w))/(Math.Pow((u*v + u*w + v*w), 2)) + (-b23x2x111*u + b23x3x111*(1 - u))/(u*v + u*w + v*w))/(Math.Pow((u + v - 2*u*v), 2))))))))) - (3*(((2*b2x102*u - 2*b3x102*(1 - u))*v2)/(u2 + u12) - ((2*u - 2*(1 - u))*(b2x102*u2 + b3x102*u12)*v2)/(Math.Pow((u2 + u12), 2))) + (6*u*(b3x120*v2 + b2x120*v12))/(v2 + v12) + (6*(b3x210*v2 + b1x210*v12)*w)/(v2 + v12) + 3*((2*(2*b2x201*u - 2*b1x201*(1 - u))*v*w)/(u2 + u12) - (2*(2*u - 2*(1 - u))*(b2x201*u2 + b1x201*u12)*v*w)/(Math.Pow((u2 + u12), 2))) + 6*u*v*((2*b1x021*w - 2*b2x021*(1 - w))/(w2 + w12) - ((2*w - 2*(1 - w))*(b1x021*w2 + b2x021*w12))/(Math.Pow((w2 + w12), 2))) + 3*v2*((2*b1x012*w - 2*b3x012*(1 - w))/(w2 + w12) - ((2*w - 2*(1 - w))*(b1x012*w2 + b3x012*w12))/(Math.Pow((w2 + w12), 2))) - ((6*(b2x201*u2 + b1x201*u12)*v)/(u2 + u12) + (6*u*(b3x210*v2 + b1x210*v12))/(v2 + v12) + 6*b300*w + 3*u2*v*(-((2*w - 2*(1 - w))*(2*b1x021*w - 2*b2x021*(1 - w)))/(Math.Pow((w2 + w12), 2)) + (2*b1x021 + 2*b2x021)/(w2 + w12) - ((4*(b1x021*w2 + b2x021*w12))/(Math.Pow((w2 + w12), 2)) + (2*w - 2*(1 - w))*((2*b1x021*w - 2*b2x021*(1 - w))/(Math.Pow((w2 + w12), 2)) - (2*(2*w - 2*(1 - w))*(b1x021*w2 + b2x021*w12))/(Math.Pow((w2 + w12), 3))))) + 3*u*v2*(-((2*w - 2*(1 - w))*(2*b1x012*w - 2*b3x012*(1 - w)))/(Math.Pow((w2 + w12), 2)) + (2*b1x012 + 2*b3x012)/(w2 + w12) - ((4*(b1x012*w2 + b3x012*w12))/(Math.Pow((w2 + w12), 2)) + (2*w - 2*(1 - w))*((2*b1x012*w - 2*b3x012*(1 - w))/(Math.Pow((w2 + w12), 2)) - (2*(2*w - 2*(1 - w))*(b1x012*w2 + b3x012*w12))/(Math.Pow((w2 + w12), 3))))) + 6*u*v*(-(2*u*v*(u + v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(Math.Pow((u*v + u*w + v*w), 2))) + 2*u*((b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w))/((u + w - 2*u*w)*(u*v + u*w + v*w)) + w*(-((1 - 2*u)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(u*v + u*w + v*w)) + ((-b12x2x111*u + b12x1x111*(1 - u))/(u*v + u*w + v*w) - ((u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)))/(u + w - 2*u*w))) + 2*v*((b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w))/((u*v + u*w + v*w)*(v + w - 2*v*w)) + w*(-((u + v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + ((-b31x3x111*v + b31x1x111*(1 - v))/(v + w - 2*v*w) - ((1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(u*v + u*w + v*w))) + w*((2*u*v*(Math.Pow((u + v), 2))*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(Math.Pow((u*v + u*w + v*w), 3))) + u*(-(2*(1 - 2*u)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(u*v + u*w + v*w)) + (2*((-b12x2x111*u + b12x1x111*(1 - u))/(u*v + u*w + v*w) - ((u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2))))/(u + w - 2*u*w) + w*(-((1 - 2*u)*((-b12x2x111*u + b12x1x111*(1 - u))/(u*v + u*w + v*w) - ((u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2))))/(Math.Pow((u + w - 2*u*w), 2)) + (-((-b12x2x111*u + b12x1x111*(1 - u))*(u + v))/(Math.Pow((u*v + u*w + v*w), 2)) - (u + v)*((-b12x2x111*u + b12x1x111*(1 - u))/(Math.Pow((u*v + u*w + v*w), 2)) - (2*(u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 3))))/(u + w - 2*u*w) - (1 - 2*u)*(-(2*(1 - 2*u)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 3))*(u*v + u*w + v*w)) + ((-b12x2x111*u + b12x1x111*(1 - u))/(u*v + u*w + v*w) - ((u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)))/(Math.Pow((u + w - 2*u*w), 2))))) + v*(-(2*(u + v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + (2*((-b31x3x111*v + b31x1x111*(1 - v))/(v + w - 2*v*w) - ((1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2))))/(u*v + u*w + v*w) + w*(-((u + v)*((-b31x3x111*v + b31x1x111*(1 - v))/(v + w - 2*v*w) - ((1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2))))/(Math.Pow((u*v + u*w + v*w), 2)) + (-((1 - 2*v)*(-b31x3x111*v + b31x1x111*(1 - v)))/(Math.Pow((v + w - 2*v*w), 2)) - (1 - 2*v)*((-b31x3x111*v + b31x1x111*(1 - v))/(Math.Pow((v + w - 2*v*w), 2)) - (2*(1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 3))))/(u*v + u*w + v*w) - (u + v)*(-(2*(u + v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 3))*(v + w - 2*v*w)) + ((-b31x3x111*v + b31x1x111*(1 - v))/(v + w - 2*v*w) - ((1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(Math.Pow((u*v + u*w + v*w), 2)))))))) + 6*(v*((u*v*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(u*v + u*w + v*w)) + (u*w*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((u + w - 2*u*w)*(u*v + u*w + v*w)) + (v*w*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((u*v + u*w + v*w)*(v + w - 2*v*w)) + w*(-(u*v*(u + v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(Math.Pow((u*v + u*w + v*w), 2))) + u*((b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w))/((u + w - 2*u*w)*(u*v + u*w + v*w)) + w*(-((1 - 2*u)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(u*v + u*w + v*w)) + ((-b12x2x111*u + b12x1x111*(1 - u))/(u*v + u*w + v*w) - ((u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)))/(u + w - 2*u*w))) + v*((b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w))/((u*v + u*w + v*w)*(v + w - 2*v*w)) + w*(-((u + v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + ((-b31x3x111*v + b31x1x111*(1 - v))/(v + w - 2*v*w) - ((1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(u*v + u*w + v*w))))) + u*v*((v*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(u*v + u*w + v*w)) + (w*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((u + w - 2*u*w)*(u*v + u*w + v*w)) - (v*w*(v + w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + u*v*(-((1 - 2*v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((Math.Pow((u + v - 2*u*v), 2))*(u*v + u*w + v*w)) + (-((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(v + w))/(Math.Pow((u*v + u*w + v*w), 2)) + (-b23x3x111*v + b23x2x111*(1 - v))/(u*v + u*w + v*w))/(u + v - 2*u*v)) + u*w*(-((1 - 2*w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(u*v + u*w + v*w)) + ((-b12x1x111*w + b12x2x111*(1 - w))/(u*v + u*w + v*w) - ((v + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)))/(u + w - 2*u*w)) + w*(-(v*(u + v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(Math.Pow((u*v + u*w + v*w), 2))) + (b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w))/((u + w - 2*u*w)*(u*v + u*w + v*w)) + w*(-((1 - 2*u)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(u*v + u*w + v*w)) + ((-b12x2x111*u + b12x1x111*(1 - u))/(u*v + u*w + v*w) - ((u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)))/(u + w - 2*u*w)) + u*v*(((u + v)*(1 - 2*v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((Math.Pow((u + v - 2*u*v), 2))*(Math.Pow((u*v + u*w + v*w), 2))) + (-((u + v)*(-b23x3x111*v + b23x2x111*(1 - v)))/(Math.Pow((u*v + u*w + v*w), 2)) - (b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(-(2*(u + v)*(v + w))/(Math.Pow((u*v + u*w + v*w), 3)) + 1/(Math.Pow((u*v + u*w + v*w), 2))))/(u + v - 2*u*v)) - v*(((v + w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + w*((b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + (v + w)*(-(2*(u + v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 3))*(v + w - 2*v*w)) + ((-b31x3x111*v + b31x1x111*(1 - v))/(v + w - 2*v*w) - ((1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(Math.Pow((u*v + u*w + v*w), 2))))) + u*(-((1 - 2*w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(u*v + u*w + v*w)) + ((-b12x1x111*w + b12x2x111*(1 - w))/(u*v + u*w + v*w) - ((v + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)))/(u + w - 2*u*w) + w*(-((1 - 2*u)*((-b12x1x111*w + b12x2x111*(1 - w))/(u*v + u*w + v*w) - ((v + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2))))/(Math.Pow((u + w - 2*u*w), 2)) + ((-b12x1x111 - b12x2x111)/(u*v + u*w + v*w) - ((u + v)*(-b12x1x111*w + b12x2x111*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)) - ((b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w))/(Math.Pow((u*v + u*w + v*w), 2)) + (v + w)*((-b12x2x111*u + b12x1x111*(1 - u))/(Math.Pow((u*v + u*w + v*w), 2)) - (2*(u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 3)))))/(u + w - 2*u*w) - (-(2*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(u*v + u*w + v*w)) + (1 - 2*w)*(-(2*(1 - 2*u)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 3))*(u*v + u*w + v*w)) + ((-b12x2x111*u + b12x1x111*(1 - u))/(u*v + u*w + v*w) - ((u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)))/(Math.Pow((u + w - 2*u*w), 2)))))))))) - ((6*(b2x102*u2 + b3x102*u12)*v)/(u2 + u12) + 3*u2*((2*b3x120*v - 2*b2x120*(1 - v))/(v2 + v12) - ((2*v - 2*(1 - v))*(b3x120*v2 + b2x120*v12))/(Math.Pow((v2 + v12), 2))) + (6*(b2x201*u2 + b1x201*u12)*w)/(u2 + u12) + 6*u*(((2*b3x210*v - 2*b1x210*(1 - v))*w)/(v2 + v12) - ((2*v - 2*(1 - v))*(b3x210*v2 + b1x210*v12)*w)/(Math.Pow((v2 + v12), 2))) + 3*u2*((2*b1x021*w - 2*b2x021*(1 - w))/(w2 + w12) - ((2*w - 2*(1 - w))*(b1x021*w2 + b2x021*w12))/(Math.Pow((w2 + w12), 2))) + 6*u*v*((2*b1x012*w - 2*b3x012*(1 - w))/(w2 + w12) - ((2*w - 2*(1 - w))*(b1x012*w2 + b3x012*w12))/(Math.Pow((w2 + w12), 2))) + 6*u*((u*v*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(u*v + u*w + v*w)) + (u*w*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((u + w - 2*u*w)*(u*v + u*w + v*w)) + (v*w*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((u*v + u*w + v*w)*(v + w - 2*v*w)) + w*(-(u*v*(u + v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(Math.Pow((u*v + u*w + v*w), 2))) + u*((b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w))/((u + w - 2*u*w)*(u*v + u*w + v*w)) + w*(-((1 - 2*u)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(u*v + u*w + v*w)) + ((-b12x2x111*u + b12x1x111*(1 - u))/(u*v + u*w + v*w) - ((u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)))/(u + w - 2*u*w))) + v*((b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w))/((u*v + u*w + v*w)*(v + w - 2*v*w)) + w*(-((u + v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + ((-b31x3x111*v + b31x1x111*(1 - v))/(v + w - 2*v*w) - ((1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(u*v + u*w + v*w)))) + v*(-(u*w*(u + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((u + w - 2*u*w)*(Math.Pow((u*v + u*w + v*w), 2))) + (w*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((u*v + u*w + v*w)*(v + w - 2*v*w)) + v*w*(-((u + w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + ((-b31x1x111*w + b31x3x111*(1 - w))/(v + w - 2*v*w) - ((1 - 2*w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(u*v + u*w + v*w)) + u*((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))/((u + v - 2*u*v)*(u*v + u*w + v*w)) + v*(-((1 - 2*u)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((Math.Pow((u + v - 2*u*v), 2))*(u*v + u*w + v*w)) + (-((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(u + w))/(Math.Pow((u*v + u*w + v*w), 2)) + (-b23x2x111*u + b23x3x111*(1 - u))/(u*v + u*w + v*w))/(u + v - 2*u*v))) + w*((b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w))/((u*v + u*w + v*w)*(v + w - 2*v*w)) + w*(-((u + v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + ((-b31x3x111*v + b31x1x111*(1 - v))/(v + w - 2*v*w) - ((1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(u*v + u*w + v*w)) - u*(((u + v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(Math.Pow((u*v + u*w + v*w), 2))) + v*((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))/((u + v - 2*u*v)*(Math.Pow((u*v + u*w + v*w), 2))) + (u + v)*(-((1 - 2*u)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((Math.Pow((u + v - 2*u*v), 2))*(Math.Pow((u*v + u*w + v*w), 2))) + (-(2*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(u + w))/(Math.Pow((u*v + u*w + v*w), 3)) + (-b23x2x111*u + b23x3x111*(1 - u))/(Math.Pow((u*v + u*w + v*w), 2)))/(u + v - 2*u*v)))) + u*(-((u + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((u + w - 2*u*w)*(Math.Pow((u*v + u*w + v*w), 2))) + w*(((1 - 2*u)*(u + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(Math.Pow((u*v + u*w + v*w), 2))) + (-((-b12x2x111*u + b12x1x111*(1 - u))*(u + w))/(Math.Pow((u*v + u*w + v*w), 2)) - (-(2*(u + v)*(u + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 3)) + (b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w))/(Math.Pow((u*v + u*w + v*w), 2))))/(u + w - 2*u*w))) + v*(-((u + w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + ((-b31x1x111*w + b31x3x111*(1 - w))/(v + w - 2*v*w) - ((1 - 2*w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(u*v + u*w + v*w) + w*(-((u + w)*((-b31x3x111*v + b31x1x111*(1 - v))/(v + w - 2*v*w) - ((1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2))))/(Math.Pow((u*v + u*w + v*w), 2)) + (-((-b31x3x111*v + b31x1x111*(1 - v))*(1 - 2*w))/(Math.Pow((v + w - 2*v*w), 2)) + (-b31x1x111 - b31x3x111)/(v + w - 2*v*w) - (-(2*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)) + (1 - 2*v)*((-b31x1x111*w + b31x3x111*(1 - w))/(Math.Pow((v + w - 2*v*w), 2)) - (2*(1 - 2*w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 3)))))/(u*v + u*w + v*w) - ((b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + (u + v)*(-(2*(u + w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 3))*(v + w - 2*v*w)) + ((-b31x1x111*w + b31x3x111*(1 - w))/(v + w - 2*v*w) - ((1 - 2*w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(Math.Pow((u*v + u*w + v*w), 2))))))))));


return res;
}

public double GetDuu(int triIndex, int index, double u, double v, double w){
double u2 = u * u;
double v2 = v * v;
double w2 = w * w;
double u12 = (1 - u) * (1 - u);
double v12 = (1 - v) * (1 - v);
double w12 = (1 - w) * (1 - w);
 EdgePatch e3 = edgePatches[3 * triIndex];
  EdgePatch e1 = edgePatches[3 * triIndex + 1];
  EdgePatch e2 = edgePatches[3 * triIndex + 2];
 

double b300 = e1.b300[index];
double b030 = e2.b300[index];
double b003 = e3.b300[index];
double b1x201 = e1.bp1_201[index];
double b2x201 = e1.bp2_201[index];
double b3x102 = e2.bp2_021[index];
double b2x102 = e2.bp1_021[index];
double b3x012 = e3.bp1_201[index];
double b1x012 = e3.bp2_201[index];
double b2x021 = e1.bp2_021[index];
double b1x021 = e1.bp1_021[index];
double b2x120 = e2.bp1_201[index];
double b3x120 = e2.bp2_201[index];
double b1x210 = e3.bp2_021[index];
double b3x210 = e3.bp1_021[index];
double b12x1x111 = e1.bp1_111[index];
double b12x2x111 = e1.bp2_111[index];
double b23x2x111 = e2.bp1_111[index];
double b23x3x111 = e2.bp2_111[index];
double b31x3x111 = e3.bp1_111[index];
double b31x1x111 = e3.bp2_111[index];


double res = 6*b030*u + (6*(b3x120*v2 + b2x120*v12)*w)/(v2 + v12) + (6*v*(b1x021*w2 + b2x021*w12))/(w2 + w12) + 3*(-((2*u - 2*(1 - u))*(2*b2x102*u - 2*b3x102*(1 - u))*v2*w)/(Math.Pow((u2 + u12), 2)) + ((2*b2x102 + 2*b3x102)*v2*w)/(u2 + u12) - ((4*(b2x102*u2 + b3x102*u12)*v2*w)/(Math.Pow((u2 + u12), 2)) + (2*u - 2*(1 - u))*(((2*b2x102*u - 2*b3x102*(1 - u))*v2*w)/(Math.Pow((u2 + u12), 2)) - (2*(2*u - 2*(1 - u))*(b2x102*u2 + b3x102*u12)*v2*w)/(Math.Pow((u2 + u12), 3))))) + 3*(-((2*u - 2*(1 - u))*(2*b2x201*u - 2*b1x201*(1 - u))*v*w2)/(Math.Pow((u2 + u12), 2)) + ((2*b1x201 + 2*b2x201)*v*w2)/(u2 + u12) - ((4*(b2x201*u2 + b1x201*u12)*v*w2)/(Math.Pow((u2 + u12), 2)) + (2*u - 2*(1 - u))*(((2*b2x201*u - 2*b1x201*(1 - u))*v*w2)/(Math.Pow((u2 + u12), 2)) - (2*(2*u - 2*(1 - u))*(b2x201*u2 + b1x201*u12)*v*w2)/(Math.Pow((u2 + u12), 3))))) + 6*(2*v*w*((v*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(u*v + u*w + v*w)) + (w*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((u + w - 2*u*w)*(u*v + u*w + v*w)) - (v*w*(v + w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + u*v*(-((1 - 2*v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((Math.Pow((u + v - 2*u*v), 2))*(u*v + u*w + v*w)) + (-((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(v + w))/(Math.Pow((u*v + u*w + v*w), 2)) + (-b23x3x111*v + b23x2x111*(1 - v))/(u*v + u*w + v*w))/(u + v - 2*u*v)) + u*w*(-((1 - 2*w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(u*v + u*w + v*w)) + ((-b12x1x111*w + b12x2x111*(1 - w))/(u*v + u*w + v*w) - ((v + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)))/(u + w - 2*u*w))) + u*v*w*((2*v*w*(Math.Pow((v + w), 2))*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 3))*(v + w - 2*v*w)) + 2*v*(-((1 - 2*v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((Math.Pow((u + v - 2*u*v), 2))*(u*v + u*w + v*w)) + (-((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(v + w))/(Math.Pow((u*v + u*w + v*w), 2)) + (-b23x3x111*v + b23x2x111*(1 - v))/(u*v + u*w + v*w))/(u + v - 2*u*v)) + 2*w*(-((1 - 2*w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(u*v + u*w + v*w)) + ((-b12x1x111*w + b12x2x111*(1 - w))/(u*v + u*w + v*w) - ((v + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)))/(u + w - 2*u*w)) + u*v*(-((1 - 2*v)*(-((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(v + w))/(Math.Pow((u*v + u*w + v*w), 2)) + (-b23x3x111*v + b23x2x111*(1 - v))/(u*v + u*w + v*w)))/(Math.Pow((u + v - 2*u*v), 2)) + (-((-b23x3x111*v + b23x2x111*(1 - v))*(v + w))/(Math.Pow((u*v + u*w + v*w), 2)) - (-(2*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(Math.Pow((v + w), 2)))/(Math.Pow((u*v + u*w + v*w), 3)) + ((-b23x3x111*v + b23x2x111*(1 - v))*(v + w))/(Math.Pow((u*v + u*w + v*w), 2))))/(u + v - 2*u*v) - (1 - 2*v)*(-(2*(1 - 2*v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((Math.Pow((u + v - 2*u*v), 3))*(u*v + u*w + v*w)) + (-((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(v + w))/(Math.Pow((u*v + u*w + v*w), 2)) + (-b23x3x111*v + b23x2x111*(1 - v))/(u*v + u*w + v*w))/(Math.Pow((u + v - 2*u*v), 2)))) + u*w*(-((1 - 2*w)*((-b12x1x111*w + b12x2x111*(1 - w))/(u*v + u*w + v*w) - ((v + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2))))/(Math.Pow((u + w - 2*u*w), 2)) + (-((v + w)*(-b12x1x111*w + b12x2x111*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)) - (v + w)*((-b12x1x111*w + b12x2x111*(1 - w))/(Math.Pow((u*v + u*w + v*w), 2)) - (2*(v + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 3))))/(u + w - 2*u*w) - (1 - 2*w)*(-(2*(1 - 2*w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 3))*(u*v + u*w + v*w)) + ((-b12x1x111*w + b12x2x111*(1 - w))/(u*v + u*w + v*w) - ((v + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)))/(Math.Pow((u + w - 2*u*w), 2)))))) - (3*(((2*b2x102*u - 2*b3x102*(1 - u))*v2)/(u2 + u12) - ((2*u - 2*(1 - u))*(b2x102*u2 + b3x102*u12)*v2)/(Math.Pow((u2 + u12), 2))) + (6*u*(b3x120*v2 + b2x120*v12))/(v2 + v12) + (6*(b3x210*v2 + b1x210*v12)*w)/(v2 + v12) + 3*((2*(2*b2x201*u - 2*b1x201*(1 - u))*v*w)/(u2 + u12) - (2*(2*u - 2*(1 - u))*(b2x201*u2 + b1x201*u12)*v*w)/(Math.Pow((u2 + u12), 2))) + 6*u*v*((2*b1x021*w - 2*b2x021*(1 - w))/(w2 + w12) - ((2*w - 2*(1 - w))*(b1x021*w2 + b2x021*w12))/(Math.Pow((w2 + w12), 2))) + 3*v2*((2*b1x012*w - 2*b3x012*(1 - w))/(w2 + w12) - ((2*w - 2*(1 - w))*(b1x012*w2 + b3x012*w12))/(Math.Pow((w2 + w12), 2))) - ((6*(b2x201*u2 + b1x201*u12)*v)/(u2 + u12) + (6*u*(b3x210*v2 + b1x210*v12))/(v2 + v12) + 6*b300*w + 3*u2*v*(-((2*w - 2*(1 - w))*(2*b1x021*w - 2*b2x021*(1 - w)))/(Math.Pow((w2 + w12), 2)) + (2*b1x021 + 2*b2x021)/(w2 + w12) - ((4*(b1x021*w2 + b2x021*w12))/(Math.Pow((w2 + w12), 2)) + (2*w - 2*(1 - w))*((2*b1x021*w - 2*b2x021*(1 - w))/(Math.Pow((w2 + w12), 2)) - (2*(2*w - 2*(1 - w))*(b1x021*w2 + b2x021*w12))/(Math.Pow((w2 + w12), 3))))) + 3*u*v2*(-((2*w - 2*(1 - w))*(2*b1x012*w - 2*b3x012*(1 - w)))/(Math.Pow((w2 + w12), 2)) + (2*b1x012 + 2*b3x012)/(w2 + w12) - ((4*(b1x012*w2 + b3x012*w12))/(Math.Pow((w2 + w12), 2)) + (2*w - 2*(1 - w))*((2*b1x012*w - 2*b3x012*(1 - w))/(Math.Pow((w2 + w12), 2)) - (2*(2*w - 2*(1 - w))*(b1x012*w2 + b3x012*w12))/(Math.Pow((w2 + w12), 3))))) + 6*u*v*(-(2*u*v*(u + v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(Math.Pow((u*v + u*w + v*w), 2))) + 2*u*((b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w))/((u + w - 2*u*w)*(u*v + u*w + v*w)) + w*(-((1 - 2*u)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(u*v + u*w + v*w)) + ((-b12x2x111*u + b12x1x111*(1 - u))/(u*v + u*w + v*w) - ((u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)))/(u + w - 2*u*w))) + 2*v*((b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w))/((u*v + u*w + v*w)*(v + w - 2*v*w)) + w*(-((u + v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + ((-b31x3x111*v + b31x1x111*(1 - v))/(v + w - 2*v*w) - ((1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(u*v + u*w + v*w))) + w*((2*u*v*(Math.Pow((u + v), 2))*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(Math.Pow((u*v + u*w + v*w), 3))) + u*(-(2*(1 - 2*u)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(u*v + u*w + v*w)) + (2*((-b12x2x111*u + b12x1x111*(1 - u))/(u*v + u*w + v*w) - ((u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2))))/(u + w - 2*u*w) + w*(-((1 - 2*u)*((-b12x2x111*u + b12x1x111*(1 - u))/(u*v + u*w + v*w) - ((u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2))))/(Math.Pow((u + w - 2*u*w), 2)) + (-((-b12x2x111*u + b12x1x111*(1 - u))*(u + v))/(Math.Pow((u*v + u*w + v*w), 2)) - (u + v)*((-b12x2x111*u + b12x1x111*(1 - u))/(Math.Pow((u*v + u*w + v*w), 2)) - (2*(u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 3))))/(u + w - 2*u*w) - (1 - 2*u)*(-(2*(1 - 2*u)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 3))*(u*v + u*w + v*w)) + ((-b12x2x111*u + b12x1x111*(1 - u))/(u*v + u*w + v*w) - ((u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)))/(Math.Pow((u + w - 2*u*w), 2))))) + v*(-(2*(u + v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + (2*((-b31x3x111*v + b31x1x111*(1 - v))/(v + w - 2*v*w) - ((1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2))))/(u*v + u*w + v*w) + w*(-((u + v)*((-b31x3x111*v + b31x1x111*(1 - v))/(v + w - 2*v*w) - ((1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2))))/(Math.Pow((u*v + u*w + v*w), 2)) + (-((1 - 2*v)*(-b31x3x111*v + b31x1x111*(1 - v)))/(Math.Pow((v + w - 2*v*w), 2)) - (1 - 2*v)*((-b31x3x111*v + b31x1x111*(1 - v))/(Math.Pow((v + w - 2*v*w), 2)) - (2*(1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 3))))/(u*v + u*w + v*w) - (u + v)*(-(2*(u + v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 3))*(v + w - 2*v*w)) + ((-b31x3x111*v + b31x1x111*(1 - v))/(v + w - 2*v*w) - ((1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(Math.Pow((u*v + u*w + v*w), 2)))))))) + 6*(v*((u*v*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(u*v + u*w + v*w)) + (u*w*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((u + w - 2*u*w)*(u*v + u*w + v*w)) + (v*w*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((u*v + u*w + v*w)*(v + w - 2*v*w)) + w*(-(u*v*(u + v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(Math.Pow((u*v + u*w + v*w), 2))) + u*((b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w))/((u + w - 2*u*w)*(u*v + u*w + v*w)) + w*(-((1 - 2*u)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(u*v + u*w + v*w)) + ((-b12x2x111*u + b12x1x111*(1 - u))/(u*v + u*w + v*w) - ((u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)))/(u + w - 2*u*w))) + v*((b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w))/((u*v + u*w + v*w)*(v + w - 2*v*w)) + w*(-((u + v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + ((-b31x3x111*v + b31x1x111*(1 - v))/(v + w - 2*v*w) - ((1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(u*v + u*w + v*w))))) + u*v*((v*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(u*v + u*w + v*w)) + (w*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((u + w - 2*u*w)*(u*v + u*w + v*w)) - (v*w*(v + w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + u*v*(-((1 - 2*v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((Math.Pow((u + v - 2*u*v), 2))*(u*v + u*w + v*w)) + (-((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(v + w))/(Math.Pow((u*v + u*w + v*w), 2)) + (-b23x3x111*v + b23x2x111*(1 - v))/(u*v + u*w + v*w))/(u + v - 2*u*v)) + u*w*(-((1 - 2*w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(u*v + u*w + v*w)) + ((-b12x1x111*w + b12x2x111*(1 - w))/(u*v + u*w + v*w) - ((v + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)))/(u + w - 2*u*w)) + w*(-(v*(u + v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(Math.Pow((u*v + u*w + v*w), 2))) + (b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w))/((u + w - 2*u*w)*(u*v + u*w + v*w)) + w*(-((1 - 2*u)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(u*v + u*w + v*w)) + ((-b12x2x111*u + b12x1x111*(1 - u))/(u*v + u*w + v*w) - ((u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)))/(u + w - 2*u*w)) + u*v*(((u + v)*(1 - 2*v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((Math.Pow((u + v - 2*u*v), 2))*(Math.Pow((u*v + u*w + v*w), 2))) + (-((u + v)*(-b23x3x111*v + b23x2x111*(1 - v)))/(Math.Pow((u*v + u*w + v*w), 2)) - (b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(-(2*(u + v)*(v + w))/(Math.Pow((u*v + u*w + v*w), 3)) + 1/(Math.Pow((u*v + u*w + v*w), 2))))/(u + v - 2*u*v)) - v*(((v + w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + w*((b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + (v + w)*(-(2*(u + v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 3))*(v + w - 2*v*w)) + ((-b31x3x111*v + b31x1x111*(1 - v))/(v + w - 2*v*w) - ((1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(Math.Pow((u*v + u*w + v*w), 2))))) + u*(-((1 - 2*w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(u*v + u*w + v*w)) + ((-b12x1x111*w + b12x2x111*(1 - w))/(u*v + u*w + v*w) - ((v + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)))/(u + w - 2*u*w) + w*(-((1 - 2*u)*((-b12x1x111*w + b12x2x111*(1 - w))/(u*v + u*w + v*w) - ((v + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2))))/(Math.Pow((u + w - 2*u*w), 2)) + ((-b12x1x111 - b12x2x111)/(u*v + u*w + v*w) - ((u + v)*(-b12x1x111*w + b12x2x111*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)) - ((b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w))/(Math.Pow((u*v + u*w + v*w), 2)) + (v + w)*((-b12x2x111*u + b12x1x111*(1 - u))/(Math.Pow((u*v + u*w + v*w), 2)) - (2*(u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 3)))))/(u + w - 2*u*w) - (-(2*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(u*v + u*w + v*w)) + (1 - 2*w)*(-(2*(1 - 2*u)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 3))*(u*v + u*w + v*w)) + ((-b12x2x111*u + b12x1x111*(1 - u))/(u*v + u*w + v*w) - ((u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)))/(Math.Pow((u + w - 2*u*w), 2)))))))))) - (3*(((2*b2x102*u - 2*b3x102*(1 - u))*v2)/(u2 + u12) - ((2*u - 2*(1 - u))*(b2x102*u2 + b3x102*u12)*v2)/(Math.Pow((u2 + u12), 2))) + (6*u*(b3x120*v2 + b2x120*v12))/(v2 + v12) + (6*(b3x210*v2 + b1x210*v12)*w)/(v2 + v12) + 6*(((2*b2x201*u - 2*b1x201*(1 - u))*v*w)/(u2 + u12) - ((2*u - 2*(1 - u))*(b2x201*u2 + b1x201*u12)*v*w)/(Math.Pow((u2 + u12), 2))) + 6*u*v*((2*b1x021*w - 2*b2x021*(1 - w))/(w2 + w12) - ((2*w - 2*(1 - w))*(b1x021*w2 + b2x021*w12))/(Math.Pow((w2 + w12), 2))) + 3*v2*((2*b1x012*w - 2*b3x012*(1 - w))/(w2 + w12) - ((2*w - 2*(1 - w))*(b1x012*w2 + b3x012*w12))/(Math.Pow((w2 + w12), 2))) + 6*(v*((u*v*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(u*v + u*w + v*w)) + (u*w*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((u + w - 2*u*w)*(u*v + u*w + v*w)) + (v*w*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((u*v + u*w + v*w)*(v + w - 2*v*w)) + w*(-(u*v*(u + v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(Math.Pow((u*v + u*w + v*w), 2))) + u*((b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w))/((u + w - 2*u*w)*(u*v + u*w + v*w)) + w*(-((1 - 2*u)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(u*v + u*w + v*w)) + ((-b12x2x111*u + b12x1x111*(1 - u))/(u*v + u*w + v*w) - ((u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)))/(u + w - 2*u*w))) + v*((b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w))/((u*v + u*w + v*w)*(v + w - 2*v*w)) + w*(-((u + v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + ((-b31x3x111*v + b31x1x111*(1 - v))/(v + w - 2*v*w) - ((1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(u*v + u*w + v*w))))) + u*v*((v*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(u*v + u*w + v*w)) + (w*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((u + w - 2*u*w)*(u*v + u*w + v*w)) - (v*w*(v + w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + u*v*(-((1 - 2*v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((Math.Pow((u + v - 2*u*v), 2))*(u*v + u*w + v*w)) + (-((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(v + w))/(Math.Pow((u*v + u*w + v*w), 2)) + (-b23x3x111*v + b23x2x111*(1 - v))/(u*v + u*w + v*w))/(u + v - 2*u*v)) + u*w*(-((1 - 2*w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(u*v + u*w + v*w)) + ((-b12x1x111*w + b12x2x111*(1 - w))/(u*v + u*w + v*w) - ((v + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)))/(u + w - 2*u*w)) + w*((b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w))/((u + w - 2*u*w)*(u*v + u*w + v*w)) + w*(-((1 - 2*u)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(u*v + u*w + v*w)) + ((-b12x2x111*u + b12x1x111*(1 - u))/(u*v + u*w + v*w) - ((u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)))/(u + w - 2*u*w)) + v*(-((v + w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + w*(-((v + w)*((-b31x3x111*v + b31x1x111*(1 - v))/(v + w - 2*v*w) - ((1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2))))/(Math.Pow((u*v + u*w + v*w), 2)) - (-(2*(u + v)*(v + w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 3))*(v + w - 2*v*w)) + (b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w))))) - ((v*(u + v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(Math.Pow((u*v + u*w + v*w), 2))) + u*v*((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))/((u + v - 2*u*v)*(Math.Pow((u*v + u*w + v*w), 2))) + (u + v)*(-((1 - 2*v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((Math.Pow((u + v - 2*u*v), 2))*(Math.Pow((u*v + u*w + v*w), 2))) + (-(2*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(v + w))/(Math.Pow((u*v + u*w + v*w), 3)) + (-b23x3x111*v + b23x2x111*(1 - v))/(Math.Pow((u*v + u*w + v*w), 2)))/(u + v - 2*u*v)))) + u*(-((1 - 2*w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(u*v + u*w + v*w)) + ((-b12x1x111*w + b12x2x111*(1 - w))/(u*v + u*w + v*w) - ((v + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)))/(u + w - 2*u*w) + w*(-((1 - 2*w)*((-b12x2x111*u + b12x1x111*(1 - u))/(u*v + u*w + v*w) - ((u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2))))/(Math.Pow((u + w - 2*u*w), 2)) + (-((-b12x2x111*u + b12x1x111*(1 - u))*(v + w))/(Math.Pow((u*v + u*w + v*w), 2)) + (-b12x1x111 - b12x2x111)/(u*v + u*w + v*w) - ((b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w))/(Math.Pow((u*v + u*w + v*w), 2)) + (u + v)*((-b12x1x111*w + b12x2x111*(1 - w))/(Math.Pow((u*v + u*w + v*w), 2)) - (2*(v + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 3)))))/(u + w - 2*u*w) - (-(2*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(u*v + u*w + v*w)) + (1 - 2*u)*(-(2*(1 - 2*w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 3))*(u*v + u*w + v*w)) + ((-b12x1x111*w + b12x2x111*(1 - w))/(u*v + u*w + v*w) - ((v + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)))/(Math.Pow((u + w - 2*u*w), 2))))))))));


return res;
}

public double GetDv(int triIndex, int index, double u, double v, double w){
double u2 = u * u;
double v2 = v * v;
double w2 = w * w;
double u12 = (1 - u) * (1 - u);
double v12 = (1 - v) * (1 - v);
double w12 = (1 - w) * (1 - w);
 EdgePatch e3 = edgePatches[3 * triIndex];
  EdgePatch e1 = edgePatches[3 * triIndex + 1];
  EdgePatch e2 = edgePatches[3 * triIndex + 2];
 

double b300 = e1.b300[index];
double b030 = e2.b300[index];
double b003 = e3.b300[index];
double b1x201 = e1.bp1_201[index];
double b2x201 = e1.bp2_201[index];
double b3x102 = e2.bp2_021[index];
double b2x102 = e2.bp1_021[index];
double b3x012 = e3.bp1_201[index];
double b1x012 = e3.bp2_201[index];
double b2x021 = e1.bp2_021[index];
double b1x021 = e1.bp1_021[index];
double b2x120 = e2.bp1_201[index];
double b3x120 = e2.bp2_201[index];
double b1x210 = e3.bp2_021[index];
double b3x210 = e3.bp1_021[index];
double b12x1x111 = e1.bp1_111[index];
double b12x2x111 = e1.bp2_111[index];
double b23x2x111 = e2.bp1_111[index];
double b23x3x111 = e2.bp2_111[index];
double b31x3x111 = e3.bp1_111[index];
double b31x1x111 = e3.bp2_111[index];


double res = 3*b003*v2 + (6*(b2x102*u2 + b3x102*u12)*v*w)/(u2 + u12) + (3*(b2x201*u2 + b1x201*u12)*w2)/(u2 + u12) + 3*u2*(((2*b3x120*v - 2*b2x120*(1 - v))*w)/(v2 + v12) - ((2*v - 2*(1 - v))*(b3x120*v2 + b2x120*v12)*w)/(Math.Pow((v2 + v12), 2))) + 3*u*(((2*b3x210*v - 2*b1x210*(1 - v))*w2)/(v2 + v12) - ((2*v - 2*(1 - v))*(b3x210*v2 + b1x210*v12)*w2)/(Math.Pow((v2 + v12), 2))) + (3*u2*(b1x021*w2 + b2x021*w12))/(w2 + w12) + (6*u*v*(b1x012*w2 + b3x012*w12))/(w2 + w12) + 6*u*(w*((u*v*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(u*v + u*w + v*w)) + (u*w*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((u + w - 2*u*w)*(u*v + u*w + v*w)) + (v*w*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((u*v + u*w + v*w)*(v + w - 2*v*w))) + v*w*(-(u*w*(u + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((u + w - 2*u*w)*(Math.Pow((u*v + u*w + v*w), 2))) + (w*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((u*v + u*w + v*w)*(v + w - 2*v*w)) + v*w*(-((u + w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + ((-b31x1x111*w + b31x3x111*(1 - w))/(v + w - 2*v*w) - ((1 - 2*w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(u*v + u*w + v*w)) + u*((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))/((u + v - 2*u*v)*(u*v + u*w + v*w)) + v*(-((1 - 2*u)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((Math.Pow((u + v - 2*u*v), 2))*(u*v + u*w + v*w)) + (-((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(u + w))/(Math.Pow((u*v + u*w + v*w), 2)) + (-b23x2x111*u + b23x3x111*(1 - u))/(u*v + u*w + v*w))/(u + v - 2*u*v))))) - ((3*(b2x102*u2 + b3x102*u12)*v2)/(u2 + u12) + (3*u2*(b3x120*v2 + b2x120*v12))/(v2 + v12) + (6*(b2x201*u2 + b1x201*u12)*v*w)/(u2 + u12) + (6*u*(b3x210*v2 + b1x210*v12)*w)/(v2 + v12) + 3*b300*w2 + 3*u2*v*((2*b1x021*w - 2*b2x021*(1 - w))/(w2 + w12) - ((2*w - 2*(1 - w))*(b1x021*w2 + b2x021*w12))/(Math.Pow((w2 + w12), 2))) + 3*u*v2*((2*b1x012*w - 2*b3x012*(1 - w))/(w2 + w12) - ((2*w - 2*(1 - w))*(b1x012*w2 + b3x012*w12))/(Math.Pow((w2 + w12), 2))) + 6*u*v*((u*v*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(u*v + u*w + v*w)) + (u*w*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((u + w - 2*u*w)*(u*v + u*w + v*w)) + (v*w*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((u*v + u*w + v*w)*(v + w - 2*v*w)) + w*(-(u*v*(u + v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(Math.Pow((u*v + u*w + v*w), 2))) + u*((b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w))/((u + w - 2*u*w)*(u*v + u*w + v*w)) + w*(-((1 - 2*u)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(u*v + u*w + v*w)) + ((-b12x2x111*u + b12x1x111*(1 - u))/(u*v + u*w + v*w) - ((u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)))/(u + w - 2*u*w))) + v*((b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w))/((u*v + u*w + v*w)*(v + w - 2*v*w)) + w*(-((u + v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + ((-b31x3x111*v + b31x1x111*(1 - v))/(v + w - 2*v*w) - ((1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(u*v + u*w + v*w))))));


return res;
}

public double GetDu(int triIndex, int index, double u, double v, double w){
double u2 = u * u;
double v2 = v * v;
double w2 = w * w;
double u12 = (1 - u) * (1 - u);
double v12 = (1 - v) * (1 - v);
double w12 = (1 - w) * (1 - w);
 EdgePatch e3 = edgePatches[3 * triIndex];
  EdgePatch e1 = edgePatches[3 * triIndex + 1];
  EdgePatch e2 = edgePatches[3 * triIndex + 2];
 

double b300 = e1.b300[index];
double b030 = e2.b300[index];
double b003 = e3.b300[index];
double b1x201 = e1.bp1_201[index];
double b2x201 = e1.bp2_201[index];
double b3x102 = e2.bp2_021[index];
double b2x102 = e2.bp1_021[index];
double b3x012 = e3.bp1_201[index];
double b1x012 = e3.bp2_201[index];
double b2x021 = e1.bp2_021[index];
double b1x021 = e1.bp1_021[index];
double b2x120 = e2.bp1_201[index];
double b3x120 = e2.bp2_201[index];
double b1x210 = e3.bp2_021[index];
double b3x210 = e3.bp1_021[index];
double b12x1x111 = e1.bp1_111[index];
double b12x2x111 = e1.bp2_111[index];
double b23x2x111 = e2.bp1_111[index];
double b23x3x111 = e2.bp2_111[index];
double b31x3x111 = e3.bp1_111[index];
double b31x1x111 = e3.bp2_111[index];


double res = 3*b030*u2 + (6*u*(b3x120*v2 + b2x120*v12)*w)/(v2 + v12) + (3*(b3x210*v2 + b1x210*v12)*w2)/(v2 + v12) + 3*(((2*b2x102*u - 2*b3x102*(1 - u))*v2*w)/(u2 + u12) - ((2*u - 2*(1 - u))*(b2x102*u2 + b3x102*u12)*v2*w)/(Math.Pow((u2 + u12), 2))) + 3*(((2*b2x201*u - 2*b1x201*(1 - u))*v*w2)/(u2 + u12) - ((2*u - 2*(1 - u))*(b2x201*u2 + b1x201*u12)*v*w2)/(Math.Pow((u2 + u12), 2))) + (6*u*v*(b1x021*w2 + b2x021*w12))/(w2 + w12) + (3*v2*(b1x012*w2 + b3x012*w12))/(w2 + w12) + 6*(v*w*((u*v*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(u*v + u*w + v*w)) + (u*w*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((u + w - 2*u*w)*(u*v + u*w + v*w)) + (v*w*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((u*v + u*w + v*w)*(v + w - 2*v*w))) + u*v*w*((v*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(u*v + u*w + v*w)) + (w*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((u + w - 2*u*w)*(u*v + u*w + v*w)) - (v*w*(v + w)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + u*v*(-((1 - 2*v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((Math.Pow((u + v - 2*u*v), 2))*(u*v + u*w + v*w)) + (-((b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v))*(v + w))/(Math.Pow((u*v + u*w + v*w), 2)) + (-b23x3x111*v + b23x2x111*(1 - v))/(u*v + u*w + v*w))/(u + v - 2*u*v)) + u*w*(-((1 - 2*w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(u*v + u*w + v*w)) + ((-b12x1x111*w + b12x2x111*(1 - w))/(u*v + u*w + v*w) - ((v + w)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)))/(u + w - 2*u*w)))) - ((3*(b2x102*u2 + b3x102*u12)*v2)/(u2 + u12) + (3*u2*(b3x120*v2 + b2x120*v12))/(v2 + v12) + (6*(b2x201*u2 + b1x201*u12)*v*w)/(u2 + u12) + (6*u*(b3x210*v2 + b1x210*v12)*w)/(v2 + v12) + 3*b300*w2 + 3*u2*v*((2*b1x021*w - 2*b2x021*(1 - w))/(w2 + w12) - ((2*w - 2*(1 - w))*(b1x021*w2 + b2x021*w12))/(Math.Pow((w2 + w12), 2))) + 3*u*v2*((2*b1x012*w - 2*b3x012*(1 - w))/(w2 + w12) - ((2*w - 2*(1 - w))*(b1x012*w2 + b3x012*w12))/(Math.Pow((w2 + w12), 2))) + 6*u*v*((u*v*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(u*v + u*w + v*w)) + (u*w*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((u + w - 2*u*w)*(u*v + u*w + v*w)) + (v*w*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((u*v + u*w + v*w)*(v + w - 2*v*w)) + w*(-(u*v*(u + v)*(b23x3x111*(1 - u)*v + b23x2x111*u*(1 - v)))/((u + v - 2*u*v)*(Math.Pow((u*v + u*w + v*w), 2))) + u*((b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w))/((u + w - 2*u*w)*(u*v + u*w + v*w)) + w*(-((1 - 2*u)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/((Math.Pow((u + w - 2*u*w), 2))*(u*v + u*w + v*w)) + ((-b12x2x111*u + b12x1x111*(1 - u))/(u*v + u*w + v*w) - ((u + v)*(b12x1x111*(1 - u)*w + b12x2x111*u*(1 - w)))/(Math.Pow((u*v + u*w + v*w), 2)))/(u + w - 2*u*w))) + v*((b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w))/((u*v + u*w + v*w)*(v + w - 2*v*w)) + w*(-((u + v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/((Math.Pow((u*v + u*w + v*w), 2))*(v + w - 2*v*w)) + ((-b31x3x111*v + b31x1x111*(1 - v))/(v + w - 2*v*w) - ((1 - 2*v)*(b31x1x111*(1 - v)*w + b31x3x111*v*(1 - w)))/(Math.Pow((v + w - 2*v*w), 2)))/(u*v + u*w + v*w))))));


return res;
}



        //============================================================================
        //============================================================================
        //============================================================================

        private Dictionary<string, FloatingPoint> InitBlendingsValues(int triIndex, int index, double u, double v, double w) // index = x|y|z
        {
            Dictionary<string, FloatingPoint> values = new Dictionary<string, FloatingPoint>();

            //------
            EdgePatch e3 = edgePatches[3 * triIndex];
            EdgePatch e1 = edgePatches[3 * triIndex + 1];
            EdgePatch e2 = edgePatches[3 * triIndex + 2];


            values["b300"] = e1.b300[index];
            values["b030"] = e2.b300[index];
            values["b003"] = e3.b300[index];

            //------

            values["b1x201"] = e1.bp1_201[index];
            values["b2x201"] = e1.bp2_201[index];

            values["b3x102"] = e2.bp2_021[index];
            values["b2x102"] = e2.bp1_021[index];

            values["b3x012"] = e3.bp1_201[index];
            values["b1x012"] = e3.bp2_201[index];

            values["b2x021"] = e1.bp2_021[index];
            values["b1x021"] = e1.bp1_021[index];

            values["b2x120"] = e2.bp1_201[index];
            values["b3x120"] = e2.bp2_201[index];

            values["b1x210"] = e3.bp2_021[index];
            values["b3x210"] = e3.bp1_021[index];

            //------

            values["b12x1x111"] = e1.bp1_111[index];
            values["b12x2x111"] = e1.bp2_111[index];

            values["b23x2x111"] = e2.bp1_111[index];
            values["b23x3x111"] = e2.bp2_111[index];

            values["b31x3x111"] = e3.bp1_111[index];
            values["b31x1x111"] = e3.bp2_111[index];

            //------

            values["u"] = u;
            values["v"] = v;
            values["w"] = w;

            return values;
        }


        private string GenerateSourceCode(string name, string expr, 
            [CallerFilePath] string sourceFilePath = "") 
        {
			return "";

            Dictionary<string, string> mapping = new Dictionary<string, string>();

          

            mapping["b300"] = "e1.b300[index]";
            mapping["b030"] = "e2.b300[index]";
            mapping["b003"] = "e3.b300[index]";

            //------

            mapping["b1x201"] = "e1.bp1_201[index]";
            mapping["b2x201"] = "e1.bp2_201[index]";

            mapping["b3x102"] = "e2.bp2_021[index]";
            mapping["b2x102"] = "e2.bp1_021[index]";

            mapping["b3x012"] = "e3.bp1_201[index]";
            mapping["b1x012"] = "e3.bp2_201[index]";

            mapping["b2x021"] = "e1.bp2_021[index]";
            mapping["b1x021"] = "e1.bp1_021[index]";

            mapping["b2x120"] = "e2.bp1_201[index]";
            mapping["b3x120"] = "e2.bp2_201[index]";

            mapping["b1x210"] = "e3.bp2_021[index]";
            mapping["b3x210"] = "e3.bp1_021[index]";

            //------

            mapping["b12x1x111"] = "e1.bp1_111[index]";
            mapping["b12x2x111"] = "e1.bp2_111[index]";

            mapping["b23x2x111"] = "e2.bp1_111[index]";
            mapping["b23x3x111"] = "e2.bp2_111[index]";

            mapping["b31x3x111"] = "e3.bp1_111[index]";
            mapping["b31x1x111"] = "e3.bp2_111[index]";



            string expr2 = "";

            expr2 += "public double Get" + name + "(int triIndex, int index, double u, double v, double w){\n";

            expr2 += "double u2 = u * u;\n";
            expr2 += "double v2 = v * v;\n";
            expr2 += "double w2 = w * w;\n";


            expr2 += "double u12 = (1 - u) * (1 - u);\n";
            expr2 += "double v12 = (1 - v) * (1 - v);\n";
            expr2 += "double w12 = (1 - w) * (1 - w);\n";

           
            expr2 += " EdgePatch e3 = edgePatches[3 * triIndex];\n ";
            expr2 += " EdgePatch e1 = edgePatches[3 * triIndex + 1];\n ";
            expr2 += " EdgePatch e2 = edgePatches[3 * triIndex + 2];\n ";

            expr2 += "\n";
            expr2 += "\n";

            foreach (var k in mapping)
            {
                expr2 += "double " + k.Key + " = " + k.Value + ";\n";
            }

            expr2 += "\n";
            expr2 += "\n";



            expr = expr.Replace("u^2", "u2");
            expr = expr.Replace("v^2", "v2");
            expr = expr.Replace("w^2", "w2");
            expr = expr.Replace("u ^ 2", "u2");
            expr = expr.Replace("v ^ 2", "v2");
            expr = expr.Replace("w ^ 2", "w2");

            expr = expr.Replace("(1 - u)^2", "u12");
            expr = expr.Replace("(1 - v)^2", "v12");
            expr = expr.Replace("(1 - w)^2", "w12");
            expr = expr.Replace("(1 - u) ^ 2", "u12");
            expr = expr.Replace("(1 - v) ^ 2", "v12");
            expr = expr.Replace("(1 - w) ^ 2", "w12");
            
            expr = ProcessPower(expr); 

            expr2 += "double res = " + expr + ";\n";


            

            expr2 += "\n";
            expr2 += "\n";
            expr2 += "return res;";
            expr2 += "\n";

            expr2 += "}\n";

            //this.GenerateToFile(sourceFilePath, expr2);
			this.GenerateToFile("d://PNG1Patch.cs", expr2);
            

            return expr2;
        }

        public string ProcessPower(string expr) 
        {
            for (int i = 0; i < expr.Length;) 
            {
                if (expr[i] == '^') 
                {
                    int start = i;

                    int bracketCount = 0;
                    while (true) 
                    {
                        if (expr[i] == ')') 
                        {
                            bracketCount++;
                        }
                        if (expr[i] == '(')
                        {
                            bracketCount--;
                        }
                        

                        if ((expr[i] == '(') && (bracketCount == 0))
                        {
                            break;
                        }

                        i--;
                    }

                    int totalStart = i;
                    string tmp = "";
                    for (int j = i; j < start; j++) 
                    {
                        tmp += expr[j];
                    }

                    //go forward to find pow
                    i = start;
                    while (Char.IsDigit(expr[i]) == false) 
                    {
                        i++;
                    }
                    int totalEnd = i;

                    int pow = expr[i] - '0';

                    string tmp2 = expr.Substring(totalStart, totalEnd - totalStart + 1);

                    expr = expr.Replace(tmp2, "(Math.Pow(" + tmp +", " + pow + "))");
                    
                }
                

                i++;
            }

            return expr;
        }

        public string ReplaceFirst(string text, string search, string replace)
        {
            int pos = text.IndexOf(search);
            if (pos < 0)
            {
                return text;
            }
            return text.Substring(0, pos) + replace + text.Substring(pos + search.Length);
        }

        public void GenerateToFile(string filePath, string data) 
        {
            string currentFile = "";

            // Open the file to read from.
            using (StreamReader sr = File.OpenText(filePath))
            {
                
                string s = "";
                while ((s = sr.ReadLine()) != null)
                {
                    currentFile += s;
                    currentFile += "\n";
                }
            }

            string data2 = "\n";
            data2 += " /* [GENERATAED CODE] */";
            data2 += "\n";
            data2 += data;
            currentFile = ReplaceFirst(currentFile, "/* [GENERATAED CODE] */", data2);


			using (StreamWriter sw = File.CreateText(filePath))
            {
                sw.WriteLine(currentFile);                
            }
            
             
            Console.WriteLine("Finish");
        }
       

        public Point3D Evaluate(double u, double v, double w, int index,
            out double mean, out double gauss)
        {

            gauss = 0;
            mean = 0;

            
            //double du_x = Eval.Evaluate(this.InitBlendingsValues(index, 0, u, v, w), this.duExpr).RealValue;
            //double du_y = Eval.Evaluate(this.InitBlendingsValues(index, 1, u, v, w), this.duExpr).RealValue;
            //double du_z = Eval.Evaluate(this.InitBlendingsValues(index, 2, u, v, w), this.duExpr).RealValue;

            double du_x = this.GetDu(index, 0, u, v, w);
            double du_y = this.GetDu(index, 1, u, v, w);
            double du_z = this.GetDu(index, 2, u, v, w);

            //double dv_x = Eval.Evaluate(this.InitBlendingsValues(index, 0, u, v, w), this.dvExpr).RealValue;
            //double dv_y = Eval.Evaluate(this.InitBlendingsValues(index, 1, u, v, w), this.dvExpr).RealValue;
            //double dv_z = Eval.Evaluate(this.InitBlendingsValues(index, 2, u, v, w), this.dvExpr).RealValue;

			double dv_x = this.GetDv(index, 0, u, v, w);
			double dv_y = this.GetDv(index, 1, u, v, w);
			double dv_z = this.GetDv(index, 2, u, v, w);

            
            Point3D du = new Point3D(du_x, du_y, du_z);
            Point3D dv = new Point3D(dv_x, dv_y, dv_z);
            Point3D n = Point3D.CrossProduct(ref dv, ref du);

            //n = du;
            n.Normalize();
            //return n;

            //double duu_x = Eval.Evaluate(this.InitBlendingsValues(index, 0, u, v, w), this.duuExpr).RealValue;
            //double duu_y = Eval.Evaluate(this.InitBlendingsValues(index, 1, u, v, w), this.duuExpr).RealValue;
            //double duu_z = Eval.Evaluate(this.InitBlendingsValues(index, 2, u, v, w), this.duuExpr).RealValue;

            double duu_x = this.GetDuu(index, 0, u, v, w);
			double duu_y = this.GetDuu(index, 1, u, v, w);
			double duu_z = this.GetDuu(index, 2, u, v, w);

            //double dvv_x = Eval.Evaluate(this.InitBlendingsValues(index, 0, u, v, w), this.dvvExpr).RealValue;
            //double dvv_y = Eval.Evaluate(this.InitBlendingsValues(index, 1, u, v, w), this.dvvExpr).RealValue;
            //double dvv_z = Eval.Evaluate(this.InitBlendingsValues(index, 2, u, v, w), this.dvvExpr).RealValue;

			double dvv_x = this.GetDvv(index, 0, u, v, w);
			double dvv_y = this.GetDvv(index, 1, u, v, w);
			double dvv_z = this.GetDvv(index, 2, u, v, w);


            //double duv_x = Eval.Evaluate(this.InitBlendingsValues(index, 0, u, v, w), this.duvExpr).RealValue;
            //double duv_y = Eval.Evaluate(this.InitBlendingsValues(index, 1, u, v, w), this.duvExpr).RealValue;
            //double duv_z = Eval.Evaluate(this.InitBlendingsValues(index, 2, u, v, w), this.duvExpr).RealValue;

			double duv_x = this.GetDuv(index, 0, u, v, w);
			double duv_y = this.GetDuv(index, 1, u, v, w);
			double duv_z = this.GetDuv(index, 2, u, v, w);



            Point3D Fu = new Point3D(du_x, du_y, du_z);
            Point3D Fv = new Point3D(dv_x, dv_y, dv_z);

            Point3D Fuu = new Point3D(duu_x, duu_y, duu_z);
            Point3D Fvv = new Point3D(dvv_x, dvv_y, dvv_z);
            Point3D Fuv = new Point3D(duv_x, duv_y, duv_z);

            double E = Point3D.DotProduct(ref Fu, ref Fu);
            double F = Point3D.DotProduct(ref Fu, ref Fv);
            double G = Point3D.DotProduct(ref Fv, ref Fv);

            double tmp = Point3D.CrossProduct(ref Fu, ref Fv).Abs();


            if (Utils.IsZero(tmp))
            {
                return n;
            }

            double L = Point3D.DotProduct(ref Fu, Point3D.CrossProduct(ref Fv, ref Fuu)) / tmp;
            double M = Point3D.DotProduct(ref Fu, Point3D.CrossProduct(ref Fv, ref Fuv)) / tmp;
            double N = Point3D.DotProduct(ref Fu, Point3D.CrossProduct(ref Fv, ref Fvv)) / tmp;


            tmp = (E * G - F * F);

            if (Utils.IsZero(tmp))
            {
                return n;
            }

            double K = (L * N - M * M) / (tmp); //gauss
            double H = (G * L - 2 * F * M + E * N) / (2 * tmp); //mean

            gauss = K;
            mean = H;



            return n;

        }

        public Point3D EvaluateNormal(double u, double v, double w, int index)
        {
            double du_x = this.GetDu(index, 0, u, v, w);
            double du_y = this.GetDu(index, 1, u, v, w);
            double du_z = this.GetDu(index, 2, u, v, w);

            double dv_x = this.GetDv(index, 0, u, v, w);
            double dv_y = this.GetDv(index, 1, u, v, w);
            double dv_z = this.GetDv(index, 2, u, v, w);


            Point3D du = new Point3D(du_x, du_y, du_z);
            Point3D dv = new Point3D(dv_x, dv_y, dv_z);
            Point3D n = Point3D.CrossProduct(ref dv, ref du);

            n.Normalize();
            return n;
        }

        public Point3D EvaluatePoint(double u, double v, double w, int index)
        {
            double x = Eval.Evaluate(this.InitBlendingsValues(index, 0, u, v, w), this.functionExpr).RealValue;
            double y = Eval.Evaluate(this.InitBlendingsValues(index, 1, u, v, w), this.functionExpr).RealValue;
            double z = Eval.Evaluate(this.InitBlendingsValues(index, 2, u, v, w), this.functionExpr).RealValue;

            Point3D p = new Point3D(x, y, z);

            return p;
        }

    }
}







